@@ -4,6 +4,7 @@
  * Copyright (C) 2003-2005, Evan Battaglia <gtoevan@gmx.net>
  * Copyright (C) 2006, Quy Tonthat <qtonthat@gmail.com>
  * Copyright (C) 2013, Guilhem Bonnefille <guilhem.bonnefille@gmail.com>
+ * Copyright (C) 2015, Rob Norris <rw_norris@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,36 +26,21 @@
  * SECTION:babel
  * @short_description: running external programs and redirecting to TRWLayers.
  *
- * GPSBabel may not be necessary for everything -- for instance,
- *   use a_babel_convert_from_shellcommand() with input_file_type == %NULL
- *   for an external program that outputs GPX.
+ * GPSBabel may not be necessary for everything,
+ *  one can use shell_command option but this will be OS platform specific
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "viking.h"
 #include "gpx.h"
 #include "babel.h"
 #include "preferences.h"
-#include <stdio.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <string.h>
-#include <glib.h>
-#include <glib/gstdio.h>
-#include <glib/gi18n.h>
 
 /* TODO in the future we could have support for other shells (change command strings), or not use a shell at all */
 #define BASH_LOCATION "/bin/bash"
 
-/**
- * List of supported protocols.
- */
-const gchar *PROTOS[] = { "http://", "https://", "ftp://", NULL };
-
 /**
  * Path to gpsbabel
  */
@@ -146,8 +132,9 @@ gboolean a_babel_convert( VikTrwLayer *vt, const char *babelargs, BabelStatusFun
   gchar *name_src = a_gpx_write_tmp_file ( vt, NULL );
 
   if ( name_src ) {
-    ret = a_babel_convert_from ( vt, bargs, name_src, cb, user_data, not_used );
-    g_remove(name_src);
+    ProcessOptions po = { bargs, name_src, NULL, NULL, NULL };
+    ret = a_babel_convert_from ( vt, &po, cb, user_data, not_used );
+    (void)g_remove(name_src);
     g_free(name_src);
   }
 
@@ -183,6 +170,13 @@ static gboolean babel_general_convert( BabelStatusFunc cb, gchar **args, gpointe
   GError *error = NULL;
   gint babel_stdout;
 
+  if ( vik_debug ) {
+    (void)g_printf ( "%s:", __FUNCTION__ );
+    for ( guint i=0; args[i]; i++ )
+      (void)g_printf ( " %s", args[i] );
+    (void)g_printf ( "\n" );
+  }
+
   if (!g_spawn_async_with_pipes (NULL, args, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL, &babel_stdout, NULL, &error)) {
     g_warning ("Async command failed: %s", error->message);
     g_error_free(error);
@@ -205,6 +199,9 @@ static gboolean babel_general_convert( BabelStatusFunc cb, gchar **args, gpointe
 
     g_child_watch_add ( pid, (GChildWatchFunc) babel_watch, NULL );
 
+    // Useful to see in case of any errors,
+    //  although they don't always occur on the last line output
+    g_debug ( "%s: last received line is=\"%s\"", __FUNCTION__, line );
     ret = TRUE;
   }
     
@@ -242,9 +239,10 @@ static gboolean babel_general_convert_from( VikTrwLayer *vt, BabelStatusFunc cb,
 
     f = g_fopen(name_dst, "r");
     if (f) {
-      ret = a_gpx_read_file ( vt, f );
+      gchar *dirpath = g_path_get_dirname ( name_dst );
+      ret = a_gpx_read_file ( vt, f, dirpath );
+      g_free ( dirpath );
       fclose(f);
-      f = NULL;
     }
   }
     
@@ -316,37 +314,18 @@ gboolean a_babel_convert_from_filter( VikTrwLayer *vt, const char *babelargs, co
           g_strfreev(sub_filters);
     } else
       g_critical("gpsbabel not found in PATH");
-    g_remove(name_dst);
+    (void)g_remove(name_dst);
     g_free(name_dst);
   }
 
   return ret;
 }
 
-/**
- * a_babel_convert_from:
- * @vt:        The TRW layer to place data into. Duplicate items will be overwritten.
- * @babelargs: A string containing gpsbabel command line options.  This string
- *             must include the input file type (-i) option.
- * @from:      The file name to convert from
- * @cb:	       Optional callback function. Same usage as in a_babel_convert().
- * @user_data: passed along to cb
- * @not_used:  Must use NULL
- *
- * Loads data into a trw layer from a file, using gpsbabel.  This routine is synchronous;
- * that is, it will block the calling program until the conversion is done. To avoid blocking, call
- * this routine from a worker thread.
- *
- * Returns: %TRUE on success
- */
-gboolean a_babel_convert_from( VikTrwLayer *vt, const char *babelargs, const char *from, BabelStatusFunc cb, gpointer user_data, gpointer not_used )
-{
-  return a_babel_convert_from_filter ( vt, babelargs, from, NULL, cb, user_data, not_used ); 
-}
 /**
  * a_babel_convert_from_shellcommand:
  * @vt: The #VikTrwLayer where to insert the collected data
  * @input_cmd: the command to run
+ * @input_file_type:
  * @cb:	       Optional callback function. Same usage as in a_babel_convert().
  * @user_data: passed along to cb
  * @not_used:  Must use NULL
@@ -385,7 +364,7 @@ gboolean a_babel_convert_from_shellcommand ( VikTrwLayer *vt, const char *input_
     ret = babel_general_convert_from ( vt, cb, args, name_dst, user_data );
     g_free ( args );
     g_free ( shell_command );
-    g_remove(name_dst);
+    (void)g_remove(name_dst);
     g_free(name_dst);
   }
 
@@ -393,25 +372,25 @@ gboolean a_babel_convert_from_shellcommand ( VikTrwLayer *vt, const char *input_
 }
 
 /**
- * a_babel_convert_from_url:
+ * a_babel_convert_from_url_filter:
  * @vt: The #VikTrwLayer where to insert the collected data
  * @url: the URL to fetch
- * @babelfilters: the filter arguments to pass to gpsbabel
- * @cb:	       Optional callback function. Same usage as in a_babel_convert().
- * @user_data: passed along to cb
- * @options:   download options. Maybe NULL.
+ * @input_type:   If input_type is %NULL, input must be GPX.
+ * @babelfilters: The filter arguments to pass to gpsbabel
+ * @cb:	          Optional callback function. Same usage as in a_babel_convert().
+ * @user_data:    Passed along to cb
+ * @options:      Download options. If %NULL then default download options will be used.
  *
- * Download the file pointed by the URL and optionally uses GPSBabel to convert from input_file_type.
- * If input_file_type is %NULL, input must be GPX.
- * If input_file_type and babelfilters are %NULL, gpsbabel is not used.
+ * Download the file pointed by the URL and optionally uses GPSBabel to convert from input_type.
+ * If input_type and babelfilters are %NULL, gpsbabel is not used.
  *
  * Returns: %TRUE on successful invocation of GPSBabel or read of the GPX
  *
  */
-gboolean a_babel_convert_from_url_filter ( VikTrwLayer *vt, const char *url, const char *input_type, const char *babelfilters, BabelStatusFunc cb, gpointer user_data, DownloadMapOptions *options )
+gboolean a_babel_convert_from_url_filter ( VikTrwLayer *vt, const char *url, const char *input_type, const char *babelfilters, BabelStatusFunc cb, gpointer user_data, DownloadFileOptions *options )
 {
   // If no download options specified, use defaults:
-  DownloadMapOptions myoptions = { FALSE, FALSE, NULL, 2, NULL, NULL, NULL };
+  DownloadFileOptions myoptions = { FALSE, FALSE, NULL, 2, NULL, NULL, NULL };
   if ( options )
     myoptions = *options;
   gint fd_src;
@@ -425,7 +404,7 @@ gboolean a_babel_convert_from_url_filter ( VikTrwLayer *vt, const char *url, con
   if ((fd_src = g_file_open_tmp("tmp-viking.XXXXXX", &name_src, NULL)) >= 0) {
     g_debug ("%s: temporary file: %s", __FUNCTION__, name_src);
     close(fd_src);
-    g_remove(name_src);
+    (void)g_remove(name_src);
 
     fetch_ret = a_http_download_get_url(url, "", name_src, &myoptions, NULL);
     if (fetch_ret == DOWNLOAD_SUCCESS) {
@@ -437,13 +416,21 @@ gboolean a_babel_convert_from_url_filter ( VikTrwLayer *vt, const char *url, con
         g_debug("%s: directly read GPX file %s", __FUNCTION__, name_src);
         FILE *f = g_fopen(name_src, "r");
         if (f) {
-          ret = a_gpx_read_file ( vt, f );
+          gchar *dirpath = g_path_get_dirname ( name_src );
+          ret = a_gpx_read_file ( vt, f, dirpath );
+          g_free ( dirpath );
           fclose(f);
-          f = NULL;
+        }
+        // Try to avoid adding the description if URL is OAuth signed
+        if ( !g_ascii_strncasecmp(url, "?oauth_consumer_key=", 20) ) {
+          VikTRWMetadata *meta = vik_trw_layer_get_metadata(vt);
+          if ( meta && !meta->description ) {
+            meta->description = g_strdup ( url );
+          }
         }
       }
     }
-    util_remove(name_src);
+    (void)util_remove(name_src);
     g_free(babelargs);
     g_free(name_src);
   }
@@ -452,59 +439,29 @@ gboolean a_babel_convert_from_url_filter ( VikTrwLayer *vt, const char *url, con
 }
 
 /**
- * a_babel_convert_from_url:
- * @vt: The #VikTrwLayer where to insert the collected data
- * @url: the URL to fetch
- * @cb:	       Optional callback function. Same usage as in a_babel_convert().
- * @user_data: passed along to cb
- * @options:   download options. Maybe NULL.
+ * a_babel_convert_from:
+ * @vt:               The TRW layer to place data into. Duplicate items will be overwritten.
+ * @process_options:  The options to control the appropriate processing function. See #ProcessOptions for more detail
+ * @cb:               Optional callback function. Same usage as in a_babel_convert().
+ * @user_data:        passed along to cb
+ * @download_options: If downloading from a URL use these options (may be NULL)
  *
- * Download the file pointed by the URL and optionally uses GPSBabel to convert from input_file_type.
- * If input_file_type is %NULL, doesn't use GPSBabel. Input must be GPX.
- *
- * Returns: %TRUE on successful invocation of GPSBabel or read of the GPX
+ * Loads data into a trw layer from a file, using gpsbabel.  This routine is synchronous;
+ * that is, it will block the calling program until the conversion is done. To avoid blocking, call
+ * this routine from a worker thread.
  *
+ * Returns: %TRUE on success
  */
-gboolean a_babel_convert_from_url ( VikTrwLayer *vt, const char *url, const char *input_type, BabelStatusFunc cb, gpointer user_data, DownloadMapOptions *options )
+gboolean a_babel_convert_from ( VikTrwLayer *vt, ProcessOptions *process_options, BabelStatusFunc cb, gpointer user_data, DownloadFileOptions *download_options )
 {
-  return a_babel_convert_from_url_filter ( vt, url, input_type, NULL, cb, user_data, options );
-}
-
-/**
- * a_babel_convert_from_url_or_shell:
- * @vt: The #VikTrwLayer where to insert the collected data
- * @url: the URL to fetch
- * @cb:	       Optional callback function. Same usage as in a_babel_convert().
- * @user_data: passed along to cb
- * @options:   download options. Maybe NULL.
- *
- * Download the file pointed by the URL and optionally uses GPSBabel to convert from input_file_type.
- * If input_file_type is %NULL, doesn't use GPSBabel. Input must be GPX.
- *
- * Returns: %TRUE on successful invocation of GPSBabel or read of the GPX
- *
- */
-gboolean a_babel_convert_from_url_or_shell ( VikTrwLayer *vt, const char *input, const char *input_type, BabelStatusFunc cb, gpointer user_data, DownloadMapOptions *options )
-{
-  
-  /* Check nature of input */
-  gboolean isUrl = FALSE;
-  int i = 0;
-  for (i = 0 ; PROTOS[i] != NULL ; i++)
-  {
-    const gchar *proto = PROTOS[i];
-    if (strncmp (input, proto, strlen(proto)) == 0)
-    {
-      /* Procotol matches: save result */
-      isUrl = TRUE;
-    }
-  }
-  
-  /* Do the job */
-  if (isUrl)
-    return a_babel_convert_from_url (vt, input, input_type, cb, user_data, options);
-  else
-    return a_babel_convert_from_shellcommand (vt, input, input_type, cb, user_data, options);
+  if ( !process_options ) return FALSE;
+  if ( process_options->url )
+    return a_babel_convert_from_url_filter ( vt, process_options->url, process_options->input_file_type, process_options->babel_filters, cb, user_data, download_options );
+  if ( process_options->babelargs )
+    return a_babel_convert_from_filter ( vt, process_options->babelargs, process_options->filename, process_options->babel_filters, cb, user_data, download_options );
+  if ( process_options->shell_command )
+    return a_babel_convert_from_shellcommand ( vt, process_options->shell_command, process_options->input_file_type, cb, user_data, download_options );
+  return FALSE;
 }
 
 static gboolean babel_general_convert_to( VikTrwLayer *vt, VikTrack *trk, BabelStatusFunc cb, gchar **args, const gchar *name_src, gpointer user_data )
@@ -522,8 +479,8 @@ static gboolean babel_general_convert_to( VikTrwLayer *vt, VikTrack *trk, BabelS
  * a_babel_convert_to:
  * @vt:             The TRW layer from which data is taken.
  * @track:          Operate on the individual track if specified. Use NULL when operating on a TRW layer
- * @babelargs:      A string containing gpsbabel command line options.  In addition to any filters, this string
- *                 must include the input file type (-i) option.
+ * @babelargs:      A string containing gpsbabel output command line options.
+ *                   (i.e. must the contain '-o' file type and possibly any filter parameters)
  * @to:             Filename or device the data is written to.
  * @cb:		   Optional callback function. Same usage as in a_babel_convert.
  * @user_data: passed along to cb
@@ -555,12 +512,12 @@ gboolean a_babel_convert_to( VikTrwLayer *vt, VikTrack *track, const char *babel
       args[i++] = gpsbabel_loc;
       args[i++] = "-i";
       args[i++] = "gpx";
+      args[i++] = "-f";
+      args[i++] = name_src;
       for (j = 0; sub_args[j]; j++)
         /* some version of gpsbabel can not take extra blank arg */
         if (sub_args[j][0] != '\0')
           args[i++] = sub_args[j];
-      args[i++] = "-f";
-      args[i++] = name_src;
       args[i++] = "-F";
       args[i++] = (char *)to;
       args[i] = NULL;
@@ -570,7 +527,7 @@ gboolean a_babel_convert_to( VikTrwLayer *vt, VikTrack *track, const char *babel
       g_strfreev(sub_args);
     } else
       g_critical("gpsbabel not found in PATH");
-    g_remove(name_src);
+    (void)g_remove(name_src);
     g_free(name_src);
   }
 
@@ -664,8 +621,7 @@ static gboolean load_feature ()
     args[i] = NULL;
 
     ret = babel_general_convert (load_feature_cb, args, NULL);
-  } else
-    g_critical("gpsbabel not found in PATH");
+  }
 
   return ret;
 }
@@ -688,7 +644,7 @@ void a_babel_init ()
   // Basic guesses - could use %ProgramFiles% but this is simpler:
   if ( g_file_test ( "C:\\Program Files (x86)\\GPSBabel\\gpsbabel.exe", G_FILE_TEST_EXISTS ) )
     // 32 bit location on a 64 bit system
-    vlpd.s = "C:\\Program Files (x86)\\GPSBabel\gpsbabel.exe";
+    vlpd.s = "C:\\Program Files (x86)\\GPSBabel\\gpsbabel.exe";
   else
     vlpd.s = "C:\\Program Files\\GPSBabel\\gpsbabel.exe";
 #else
@@ -712,10 +668,10 @@ void a_babel_post_init ()
   if ( g_strcmp0 ( gpsbabel, "gpsbabel" ) == 0 ) {
     gpsbabel_loc = g_find_program_in_path( "gpsbabel" );
     if ( !gpsbabel_loc )
-      g_critical( "gpsbabel not found in PATH" );
+      g_warning( "gpsbabel not found in PATH" );
   }
   else
-    gpsbabel_loc = (gchar*)gpsbabel;
+    gpsbabel_loc = g_strdup ( gpsbabel );
 
   // Unlikely to package unbuffer on Windows so ATM don't even bother trying
   // Highly unlikely unbuffer is available on a Windows system otherwise
@@ -725,7 +681,11 @@ void a_babel_post_init ()
     g_warning( "unbuffer not found in PATH" );
 #endif
 
-  load_feature ();
+  if ( gpsbabel_loc ) {
+    if ( !load_feature() ) {
+      g_warning ( "%s: running gpsbabel to get features failed", __FUNCTION__ );
+    }
+  }
 }
 
 /**
