@@ -2,7 +2,7 @@
 /*
  * viking -- GPS Data and Topo Analyzer, Explorer, and Manager
  *
- * Copyright (C) 2012, Rob Norris <rw_norris@hotmail.com>
+ * Copyright (C) 2012-2015, Rob Norris <rw_norris@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,14 +19,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <glib/gprintf.h>
-#include <glib/gi18n.h>
-#include <glib/gstdio.h>
-
 #include <expat.h>
 
 #include "viking.h"
@@ -39,8 +31,8 @@
 /**
  * See http://wiki.openstreetmap.org/wiki/API_v0.6#GPS_Traces
  */
-#define DS_OSM_TRACES_GPX_URL_FMT "api.openstreetmap.org/api/0.6/gpx/%d/data"
-#define DS_OSM_TRACES_GPX_FILES "api.openstreetmap.org/api/0.6/user/gpx_files"
+#define DS_OSM_TRACES_GPX_URL_FMT "https://api.openstreetmap.org/api/0.6/gpx/%d/data"
+#define DS_OSM_TRACES_GPX_FILES "https://api.openstreetmap.org/api/0.6/user/gpx_files"
 
 typedef struct {
 	GtkWidget *user_entry;
@@ -51,8 +43,8 @@ typedef struct {
 
 static gpointer datasource_osm_my_traces_init ( acq_vik_t *avt );
 static void datasource_osm_my_traces_add_setup_widgets ( GtkWidget *dialog, VikViewport *vvp, gpointer user_data );
-static void datasource_osm_my_traces_get_cmd_string ( gpointer user_data, gchar **args, gchar **extra, DownloadMapOptions *options );
-static gboolean datasource_osm_my_traces_process  ( VikTrwLayer *vtl, const gchar *cmd, const gchar *extra, BabelStatusFunc status_cb, acq_dialog_widgets_t *adw, DownloadMapOptions *options_unused );
+static void datasource_osm_my_traces_get_process_options ( gpointer user_data, ProcessOptions *po, DownloadFileOptions *options, const gchar *notused1, const gchar *notused2 );
+static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, ProcessOptions *process_options, BabelStatusFunc status_cb, acq_dialog_widgets_t *adw, DownloadFileOptions *options_unused );
 static void datasource_osm_my_traces_cleanup ( gpointer data );
 
 VikDataSourceInterface vik_datasource_osm_my_traces_interface = {
@@ -66,8 +58,8 @@ VikDataSourceInterface vik_datasource_osm_my_traces_interface = {
   (VikDataSourceInitFunc)		    datasource_osm_my_traces_init,
   (VikDataSourceCheckExistenceFunc)	NULL,
   (VikDataSourceAddSetupWidgetsFunc)datasource_osm_my_traces_add_setup_widgets,
-  (VikDataSourceGetCmdStringFunc)	datasource_osm_my_traces_get_cmd_string,
-  (VikDataSourceProcessFunc)		datasource_osm_my_traces_process,
+  (VikDataSourceGetProcessOptionsFunc) datasource_osm_my_traces_get_process_options,
+  (VikDataSourceProcessFunc)           datasource_osm_my_traces_process,
   (VikDataSourceProgressFunc)		NULL,
   (VikDataSourceAddProgressWidgetsFunc)	NULL,
   (VikDataSourceCleanupFunc)		datasource_osm_my_traces_cleanup,
@@ -83,6 +75,8 @@ VikDataSourceInterface vik_datasource_osm_my_traces_interface = {
 static gpointer datasource_osm_my_traces_init ( acq_vik_t *avt )
 {
   datasource_osm_my_traces_t *data = g_malloc(sizeof(*data));
+  /* Keep reference to viewport */
+  data->vvp = avt->vvp;
   // Reuse GPS functions
   // Haven't been able to get the thread method to work reliably (or get progress feedback)
   // So thread version is disabled ATM
@@ -97,46 +91,42 @@ static gpointer datasource_osm_my_traces_init ( acq_vik_t *avt )
 
 static void datasource_osm_my_traces_add_setup_widgets ( GtkWidget *dialog, VikViewport *vvp, gpointer user_data )
 {
-	datasource_osm_my_traces_t *data = (datasource_osm_my_traces_t *)user_data;
+	if ( osm_use_basic_auth() ) {
+		datasource_osm_my_traces_t *data = (datasource_osm_my_traces_t *)user_data;
+		GtkWidget *user_label;
+		GtkWidget *password_label;
+		user_label = gtk_label_new(_("Email/username:"));
+		data->user_entry = gtk_entry_new();
 
-	GtkWidget *user_label;
-	GtkWidget *password_label;
-	user_label = gtk_label_new(_("Username:"));
-	data->user_entry = gtk_entry_new();
+		gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), user_label, FALSE, FALSE, 0 );
+		gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), data->user_entry, FALSE, FALSE, 0 );
+		gtk_widget_set_tooltip_markup ( GTK_WIDGET(data->user_entry), _("The email or username used to login to OSM") );
 
-	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), user_label, FALSE, FALSE, 0 );
-	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), data->user_entry, FALSE, FALSE, 0 );
-	gtk_widget_set_tooltip_markup ( GTK_WIDGET(data->user_entry), _("The email or username used to login to OSM") );
+		password_label = gtk_label_new ( _("Password:") );
+		data->password_entry = gtk_entry_new ();
 
-	password_label = gtk_label_new ( _("Password:") );
-	data->password_entry = gtk_entry_new ();
+		gtk_widget_set_tooltip_markup ( GTK_WIDGET(data->password_entry), _("The password used to login to OSM") );
 
-	gtk_widget_set_tooltip_markup ( GTK_WIDGET(data->password_entry), _("The password used to login to OSM") );
+		osm_login_widgets (data->user_entry, data->password_entry);
 
-	osm_login_widgets (data->user_entry, data->password_entry);
-
-	/* Packing all widgets */
-	GtkBox *box = GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog)));
-	gtk_box_pack_start ( box, password_label, FALSE, FALSE, 0 );
-	gtk_box_pack_start ( box, data->password_entry, FALSE, FALSE, 0 );
+		/* Packing all widgets */
+		GtkBox *box = GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog)));
+		gtk_box_pack_start ( box, password_label, FALSE, FALSE, 0 );
+		gtk_box_pack_start ( box, data->password_entry, FALSE, FALSE, 0 );
+	}
 	gtk_widget_show_all ( dialog );
-
-	/* Keep reference to viewport */
-	data->vvp = vvp;
 }
 
-static void datasource_osm_my_traces_get_cmd_string ( gpointer user_data, gchar **args, gchar **extra, DownloadMapOptions *options )
+static void datasource_osm_my_traces_get_process_options ( gpointer user_data, ProcessOptions *po, DownloadFileOptions *options, const gchar *notused1, const gchar *notused2 )
 {
-	datasource_osm_my_traces_t *data = (datasource_osm_my_traces_t*) user_data;
-
-    /* overwrite authentication info */
-	osm_set_login ( gtk_entry_get_text ( GTK_ENTRY(data->user_entry) ),
-	                gtk_entry_get_text ( GTK_ENTRY(data->password_entry) ) );
-
-	// If going to use the values passed back into the process function parameters then these need to be set.
+	if ( osm_use_basic_auth() ) {
+		datasource_osm_my_traces_t *data = (datasource_osm_my_traces_t*) user_data;
+		/* overwrite authentication info */
+		osm_set_login ( gtk_entry_get_text ( GTK_ENTRY(data->user_entry) ),
+		                gtk_entry_get_text ( GTK_ENTRY(data->password_entry) ) );
+	}
+	// If going to use the values passed back into the process function parameters then they need to be set.
 	// But ATM we aren't
-	*args = NULL;
-	*extra = NULL;
 	options = NULL;
 }
 
@@ -531,15 +521,8 @@ static void none_found ( GtkWindow *gw )
  */
 static void set_in_current_view_property ( VikTrwLayer *vtl, datasource_osm_my_traces_t *data, GList *gl )
 {
-	gdouble min_lat, max_lat, min_lon, max_lon;
-	/* get Viewport bounding box */
-	vik_viewport_get_min_max_lat_lon ( data->vvp, &min_lat, &max_lat, &min_lon, &max_lon );
-
-	LatLonBBox bbox;
-	bbox.north = max_lat;
-	bbox.east = max_lon;
-	bbox.south = min_lat;
-	bbox.west = min_lon;
+	// get viewport bounding box
+	LatLonBBox bbox = vik_viewport_get_bbox ( data->vvp );
 
 	GList *iterator = gl;
 	while ( iterator ) {
@@ -560,16 +543,34 @@ static void set_in_current_view_property ( VikTrwLayer *vtl, datasource_osm_my_t
 	}
 }
 
-static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, const gchar *cmd, const gchar *extra, BabelStatusFunc status_cb, acq_dialog_widgets_t *adw, DownloadMapOptions *options_unused )
+static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, ProcessOptions *process_options, BabelStatusFunc status_cb, acq_dialog_widgets_t *adw, DownloadFileOptions *options_unused )
 {
-	//datasource_osm_my_traces_t *data = (datasource_osm_my_traces_t *)adw->user_data;
-
 	gboolean result;
 
-	gchar *user_pass = osm_get_login();
+	gchar *user_pass = NULL;
+	gchar *uri = NULL;
+	if ( osm_use_basic_auth() ) {
+		uri = g_strdup ( DS_OSM_TRACES_GPX_FILES );
+		user_pass = osm_get_login();
+	}
+#ifdef HAVE_OAUTH_H
+	else {
+		datasource_osm_my_traces_t *data = (datasource_osm_my_traces_t *)adw->user_data;
+		uri = osm_oauth_sign_url ( DS_OSM_TRACES_GPX_FILES, NULL );
+		if ( !uri ) {
+			a_dialog_error_msg ( VIK_GTK_WINDOW_FROM_WIDGET(data->vvp), _("Ensure the OSM access token preferences are setup.") );
+			return FALSE;
+		}
+	}
+#endif
 
 	// Support .zip + bzip2 files directly
-	DownloadMapOptions options = { FALSE, FALSE, NULL, 2, NULL, user_pass, a_try_decompress_file }; // Allow a couple of redirects
+	DownloadFileOptions options = { FALSE, FALSE, NULL, 2, NULL, user_pass, a_try_decompress_file }; // Allow a couple of redirects
+
+	gchar *tmpname = a_download_uri_to_tmp_file ( uri, &options );
+	g_free ( uri );
+    if ( !tmpname )
+		return FALSE;
 
 	xml_data *xd = g_malloc ( sizeof (xml_data) );
 	//xd->xpath = g_string_new ( "" );
@@ -578,22 +579,24 @@ static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, const gchar
 	xd->current_gpx_meta_data = new_gpx_meta_data_t();
 	xd->list_of_gpx_meta_data = NULL;
 
-	gchar *tmpname = a_download_uri_to_tmp_file ( DS_OSM_TRACES_GPX_FILES, &options );
 	result = read_gpx_files_metadata_xml ( tmpname, xd );
 	// Test already downloaded metadata file: eg:
 	//result = read_gpx_files_metadata_xml ( "/tmp/viking-download.GI47PW", xd );
 
 	if ( tmpname ) {
-		util_remove ( tmpname );
+		(void)util_remove ( tmpname );
 		g_free ( tmpname );
 	}
 
-	if ( ! result )
+	if ( ! result ) {
+		g_free ( xd );
 		return FALSE;
+	}
 
 	if ( g_list_length ( xd->list_of_gpx_meta_data ) == 0 ) {
 		if (!vik_datasource_osm_my_traces_interface.is_thread)
 			none_found ( GTK_WINDOW(adw->vw) );
+		g_free ( xd );
 		return FALSE;
 	}
 
@@ -639,7 +642,14 @@ static gboolean datasource_osm_my_traces_process ( VikTrwLayer *vtl, const gchar
 		if ( gpx_id ) {
 			gchar *url = g_strdup_printf ( DS_OSM_TRACES_GPX_URL_FMT, gpx_id );
 
-			result = a_babel_convert_from_url ( vtlX, url, "gpx", status_cb, adw, &options );
+			// NB download type is GPX (or a compressed version)
+			ProcessOptions my_po = *process_options;
+#ifdef HAVE_OAUTH_H
+			my_po.url = osm_oauth_sign_url ( url, NULL );
+#else
+			my_po.url = url;
+#endif
+			result = a_babel_convert_from ( vtlX, &my_po, status_cb, adw, &options );
 			// TODO investigate using a progress bar:
 			// http://developer.gnome.org/gtk/2.24/GtkProgressBar.html
 
