@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2003-2005, Evan Battaglia <gtoevan@gmx.net>
  * Copyright (C) 2008, Hein Ragas <viking@ragas.nl>
- * Copyright (C) 2010-2014, Rob Norris <rw_norris@hotmail.com>
+ * Copyright (C) 2010-2016, Rob Norris <rw_norris@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,21 +20,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "viking.h"
 #include "degrees_converters.h"
 #include "authors.h"
 #include "documenters.h"
-#include "ui_util.h"
 
-#include <glib/gi18n.h>
-
-#include <stdlib.h>
-#include <string.h>
 #include <ctype.h>
 
 void a_dialog_msg ( GtkWindow *parent, gint type, const gchar *info, const gchar *extra )
@@ -44,6 +34,17 @@ void a_dialog_msg ( GtkWindow *parent, gint type, const gchar *info, const gchar
   gtk_widget_destroy ( msgbox );
 }
 
+/**
+ * a_dialog_goto_latlon:
+ *
+ * @parent:  Parent window
+ * @ll:      The returned #LatLon location
+ * @old:     Initialize the dialog with this #LatLon location
+ *
+ * A simple dialog to get a lat/lon location
+ *
+ * Returns: FALSE if the dialog was cancelled
+ */
 gboolean a_dialog_goto_latlon ( GtkWindow *parent, struct LatLon *ll, const struct LatLon *old )
 {
   GtkWidget *dialog = gtk_dialog_new_with_buttons (_("Go to Lat/Lon"),
@@ -54,38 +55,26 @@ gboolean a_dialog_goto_latlon ( GtkWindow *parent, struct LatLon *ll, const stru
                                                   GTK_STOCK_OK,
                                                   GTK_RESPONSE_ACCEPT,
                                                   NULL);
-  GtkWidget *latlabel, *lonlabel;
-  GtkWidget *lat, *lon;
-  gchar *tmp_lat, *tmp_lon;
 
-  latlabel = gtk_label_new (_("Latitude:"));
-  lat = gtk_entry_new ();
-  tmp_lat = g_strdup_printf ( "%f", old->lat );
-  gtk_entry_set_text ( GTK_ENTRY(lat), tmp_lat );
-  g_free ( tmp_lat );
+  GtkWidget *latlabel = gtk_label_new (_("Latitude:"));
+  GtkWidget *lat = ui_spin_button_new ( (GtkAdjustment*)gtk_adjustment_new(old->lat,-90,90.0,0.05,0.1,0), 0.1, 6 );
 
-  lonlabel = gtk_label_new (_("Longitude:"));
-  lon = gtk_entry_new ();
-  tmp_lon = g_strdup_printf ( "%f", old->lon );
-  gtk_entry_set_text ( GTK_ENTRY(lon), tmp_lon );
-  g_free ( tmp_lon );
-
-  gtk_widget_show ( latlabel );
-  gtk_widget_show ( lonlabel );
-  gtk_widget_show ( lat );
-  gtk_widget_show ( lon );
+  GtkWidget *lonlabel = gtk_label_new (_("Longitude:"));
+  GtkWidget *lon = ui_spin_button_new ( (GtkAdjustment*)gtk_adjustment_new(old->lon,-180.0,180.0,0.05,0.1,0), 0.1, 6 );
 
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), latlabel,  FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), lat, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), lonlabel,  FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), lon,  FALSE, FALSE, 0);
 
-  // 'ok' when press return in the entry
+  // 'ok' when press return on an entry
   g_signal_connect_swapped (lat, "activate", G_CALLBACK(a_dialog_response_accept), dialog);
   g_signal_connect_swapped (lon, "activate", G_CALLBACK(a_dialog_response_accept), dialog);
 
   gtk_dialog_set_default_response ( GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT );
 
+  gtk_widget_show_all ( dialog );
+
   if ( gtk_dialog_run ( GTK_DIALOG(dialog) ) == GTK_RESPONSE_ACCEPT )
   {
     ll->lat = convert_dms_to_dec ( gtk_entry_get_text ( GTK_ENTRY(lat) ) );
@@ -110,20 +99,13 @@ gboolean a_dialog_goto_utm ( GtkWindow *parent, struct UTM *utm, const struct UT
                                                   NULL);
   GtkWidget *norlabel, *easlabel, *nor, *eas;
   GtkWidget *zonehbox, *zonespin, *letterentry;
-  gchar *tmp_eas, *tmp_nor;
   gchar tmp_letter[2];
 
   norlabel = gtk_label_new (_("Northing:"));
-  nor = gtk_entry_new ();
-  tmp_nor = g_strdup_printf("%ld", (long) old->northing );
-  gtk_entry_set_text ( GTK_ENTRY(nor), tmp_nor );
-  g_free ( tmp_nor );
+  nor = ui_spin_button_new ( (GtkAdjustment*)gtk_adjustment_new(old->northing,0,9999999,1,250,0), 1, 0 );
 
   easlabel = gtk_label_new (_("Easting:"));
-  eas = gtk_entry_new ();
-  tmp_eas = g_strdup_printf("%ld", (long) old->easting );
-  gtk_entry_set_text ( GTK_ENTRY(eas), tmp_eas );
-  g_free ( tmp_eas );
+  eas = ui_spin_button_new ( (GtkAdjustment*)gtk_adjustment_new(old->easting,0,9999999,1,250,0), 1, 0 );
 
   zonehbox = gtk_hbox_new ( FALSE, 0 );
   gtk_box_pack_start ( GTK_BOX(zonehbox), gtk_label_new ( _("Zone:") ), FALSE, FALSE, 5 );
@@ -138,21 +120,16 @@ gboolean a_dialog_goto_utm ( GtkWindow *parent, struct UTM *utm, const struct UT
   gtk_entry_set_text ( GTK_ENTRY(letterentry), tmp_letter );
   gtk_box_pack_start ( GTK_BOX(zonehbox), letterentry, FALSE, FALSE, 5 );
 
-  gtk_widget_show ( norlabel );
-  gtk_widget_show ( easlabel );
-  gtk_widget_show ( nor );
-  gtk_widget_show ( eas );
-
-  gtk_widget_show_all ( zonehbox );
-
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), norlabel, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), nor, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), easlabel,  FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), eas,  FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), zonehbox,  FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), zonehbox,  FALSE, FALSE, 2);
 
   gtk_dialog_set_default_response ( GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT );
 
+  gtk_widget_show_all ( dialog );
+
   if ( gtk_dialog_run ( GTK_DIALOG(dialog) ) == GTK_RESPONSE_ACCEPT )
   {
     const gchar *letter;
@@ -303,14 +280,6 @@ gchar *a_dialog_new_track ( GtkWindow *parent, gchar *default_name, gboolean is_
   return NULL;
 }
 
-static void today_clicked (GtkWidget *cal)
-{
-  GDateTime *now = g_date_time_new_now_local ();
-  gtk_calendar_select_month ( GTK_CALENDAR(cal), g_date_time_get_month(now)-1, g_date_time_get_year(now) );
-  gtk_calendar_select_day ( GTK_CALENDAR(cal), g_date_time_get_day_of_month(now) );
-  g_date_time_unref ( now );
-}
-
 /**
  * a_dialog_get_date:
  *
@@ -344,7 +313,7 @@ gchar *a_dialog_get_date ( GtkWindow *parent, const gchar *title )
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), today, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), cal, FALSE, FALSE, 0);
 
-  g_signal_connect_swapped ( G_OBJECT(today), "clicked", G_CALLBACK(today_clicked), cal );
+  g_signal_connect_swapped ( G_OBJECT(today), "clicked", G_CALLBACK(vu_calendar_set_to_today), cal );
 
   gtk_widget_show_all ( dialog );
 
@@ -469,6 +438,26 @@ static void split_spin_focused ( GtkSpinButton *spin, GtkWidget *pass_along[1] )
   gtk_toggle_button_set_active    (GTK_TOGGLE_BUTTON(pass_along[0]), 1);
 }
 
+static void time_1m_clicked_cb ( GtkButton *button, GtkWidget *spin )
+{
+  gtk_spin_button_set_value ( GTK_SPIN_BUTTON(spin), 1.0 );
+}
+
+static void time_1h_clicked_cb ( GtkButton *button, GtkWidget *spin )
+{
+  gtk_spin_button_set_value ( GTK_SPIN_BUTTON(spin), 60.0 );
+}
+
+static void time_6h_clicked_cb ( GtkButton *button, GtkWidget *spin )
+{
+  gtk_spin_button_set_value ( GTK_SPIN_BUTTON(spin), 6*60.0 );
+}
+
+static void time_1d_clicked_cb ( GtkButton *button, GtkWidget *spin )
+{
+  gtk_spin_button_set_value ( GTK_SPIN_BUTTON(spin), 24*60.0 );
+}
+
 gboolean a_dialog_time_threshold ( GtkWindow *parent, gchar *title_text, gchar *label_text, guint *thr )
 {
   GtkWidget *dialog = gtk_dialog_new_with_buttons (title_text, 
@@ -479,20 +468,21 @@ gboolean a_dialog_time_threshold ( GtkWindow *parent, gchar *title_text, gchar *
                                                   GTK_STOCK_OK,
                                                   GTK_RESPONSE_ACCEPT,
                                                   NULL);
-  GtkWidget *table, *t1, *t2, *t3, *t4, *spin, *label;
+  GtkWidget *table, *t1, *t2, *t3, *t4, *t5, *spin, *label;
   GtkWidget *pass_along[1];
 
-  table = gtk_table_new ( 4, 2, FALSE );
+  table = gtk_table_new ( 5, 2, FALSE );
   gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), table, TRUE, TRUE, 0 );
 
   label = gtk_label_new (label_text);
 
   t1 = gtk_radio_button_new_with_label ( NULL, _("1 min") );
   t2 = gtk_radio_button_new_with_label_from_widget ( GTK_RADIO_BUTTON(t1), _("1 hour") );
-  t3 = gtk_radio_button_new_with_label_from_widget ( GTK_RADIO_BUTTON(t2), _("1 day") );
-  t4 = gtk_radio_button_new_with_label_from_widget ( GTK_RADIO_BUTTON(t3), _("Custom (in minutes):") );
+  t3 = gtk_radio_button_new_with_label_from_widget ( GTK_RADIO_BUTTON(t2), _("6 hours") );
+  t4 = gtk_radio_button_new_with_label_from_widget ( GTK_RADIO_BUTTON(t3), _("1 day") );
+  t5 = gtk_radio_button_new_with_label_from_widget ( GTK_RADIO_BUTTON(t4), _("Custom (in minutes):") );
 
-  pass_along[0] = t4;
+  pass_along[0] = t5;
 
   spin = gtk_spin_button_new ( (GtkAdjustment *) gtk_adjustment_new ( *thr, 0, 65536, 1, 5, 0 ), 1, 0 );
 
@@ -501,12 +491,18 @@ gboolean a_dialog_time_threshold ( GtkWindow *parent, gchar *title_text, gchar *
   gtk_table_attach_defaults ( GTK_TABLE(table), t2, 0, 1, 2, 3 );
   gtk_table_attach_defaults ( GTK_TABLE(table), t3, 0, 1, 3, 4 );
   gtk_table_attach_defaults ( GTK_TABLE(table), t4, 0, 1, 4, 5 );
-  gtk_table_attach_defaults ( GTK_TABLE(table), spin, 1, 2, 4, 5 );
+  gtk_table_attach_defaults ( GTK_TABLE(table), t5, 0, 1, 5, 6 );
+  gtk_table_attach_defaults ( GTK_TABLE(table), spin, 1, 2, 5, 6 );
 
   gtk_widget_show_all ( table );
 
   g_signal_connect ( G_OBJECT(spin), "grab-focus", G_CALLBACK(split_spin_focused), pass_along );
 
+  g_signal_connect ( G_OBJECT(t1), "clicked", G_CALLBACK(time_1m_clicked_cb), spin );
+  g_signal_connect ( G_OBJECT(t2), "clicked", G_CALLBACK(time_1h_clicked_cb), spin );
+  g_signal_connect ( G_OBJECT(t3), "clicked", G_CALLBACK(time_6h_clicked_cb), spin );
+  g_signal_connect ( G_OBJECT(t4), "clicked", G_CALLBACK(time_1d_clicked_cb), spin );
+
   gtk_dialog_set_default_response ( GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT );
 
   if ( gtk_dialog_run ( GTK_DIALOG(dialog) ) == GTK_RESPONSE_ACCEPT )
@@ -516,8 +512,10 @@ gboolean a_dialog_time_threshold ( GtkWindow *parent, gchar *title_text, gchar *
     } else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(t2))) {
       *thr = 60;
     } else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(t3))) {
-      *thr = 60 * 24;
+      *thr = 60 * 6;
     } else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(t4))) {
+      *thr = 60 * 24;
+    } else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(t5))) {
       *thr = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(spin) );
     }
     gtk_widget_destroy ( dialog );
@@ -624,10 +622,9 @@ void a_dialog_list ( GtkWindow *parent, const gchar *title, GArray *array, gint
 
 void a_dialog_about ( GtkWindow *parent )
 {
-  const gchar *program_name = PACKAGE_NAME;
   const gchar *version = VIKING_VERSION;
   const gchar *website = VIKING_URL;
-  const gchar *copyright = "2003-2008, Evan Battaglia\n2008-2013, Viking's contributors";
+  gchar *copyright = g_strdup_printf(_("2003-2008, Evan Battaglia\n2008-%s, Viking's contributors"), THEYEAR);
   const gchar *comments = _("GPS Data and Topo Analyzer, Explorer, and Manager.");
   const gchar *license = _("This program is free software; you can redistribute it and/or modify "
 			"it under the terms of the GNU General Public License as published by "
@@ -645,7 +642,6 @@ void a_dialog_about ( GtkWindow *parent )
 
   // Would be nice to use gtk_about_dialog_add_credit_section (), but that requires gtk 3.4
   // For now shove it in the 'artists' section so at least the information is easily visible
-  // Something more advanced might have proper version information too...
   const gchar *libs[] = {
     "Compiled in libraries:",
     // Default libs
@@ -662,6 +658,9 @@ void a_dialog_about ( GtkWindow *parent )
 #endif
 #ifdef HAVE_LIBCURL
     "libcurl",
+#endif
+#ifdef HAVE_EXPAT_H
+    "libexpat",
 #endif
     // Actually optional libs
 #ifdef HAVE_LIBGPS
@@ -682,11 +681,23 @@ void a_dialog_about ( GtkWindow *parent )
 #ifdef HAVE_LIBBZ2
     "libbz2",
 #endif
+#ifdef HAVE_LIBZIP
+    "libzip",
+#endif
 #ifdef HAVE_LIBSQLITE3
     "libsqlite3",
 #endif
 #ifdef HAVE_LIBMAPNIK
     "libmapnik",
+#endif
+#ifdef HAVE_LIBNETTLE
+    "libnettle",
+#endif
+#ifdef HAVE_LIBGEOCLUE_2
+    "libgeoclue2",
+#endif
+#ifdef HAVE_LIBOAUTH
+    "liboauth",
 #endif
     NULL
   };
@@ -698,8 +709,6 @@ void a_dialog_about ( GtkWindow *parent )
 #endif
 
   gtk_show_about_dialog (parent,
-	/* TODO do not set program-name and correctly set info for g_get_application_name */
-  	"program-name", program_name,
 	"version", version,
 	"website", website,
 	"comments", comments,
@@ -712,6 +721,7 @@ void a_dialog_about ( GtkWindow *parent )
 	"translator-credits", _("Translation is coordinated on http://launchpad.net/viking"),
 	"artists", libs,
 	NULL);
+  g_free(copyright);
 }
 
 gboolean a_dialog_map_n_zoom(GtkWindow *parent, gchar *mapnames[], gint default_map, gchar *zoom_list[], gint default_zoom, gint *selected_map, gint *selected_zoom)
@@ -789,3 +799,94 @@ void a_dialog_license ( GtkWindow *parent, const gchar *map, const gchar *licens
   } while (response != GTK_RESPONSE_DELETE_EVENT && response != GTK_RESPONSE_OK);
   gtk_widget_destroy (dialog);
 }
+
+/**
+ * Mainly for debug
+ * Run Time or Build information that can be easily extracted from the libraries in use
+ */
+#ifdef HAVE_EXPAT_H
+#include <expat.h>
+#endif
+#ifdef HAVE_LIBCURL
+#include <curl/curl.h>
+#endif
+#ifdef HAVE_GPS_H
+#include <gps.h>
+#endif
+#ifdef HAVE_LIBGEXIV2
+#include <gexiv2/gexiv2.h>
+#endif
+#ifdef HAVE_MAGIC_H
+#include <magic.h>
+#endif
+#ifdef HAVE_BZLIB_H
+#include <bzlib.h>
+#endif
+#ifdef HAVE_ZIP_H
+#include <zip.h>
+#endif
+#ifdef HAVE_SQLITE3_H
+#include "sqlite3.h"
+#endif
+#ifdef HAVE_LIBMAPNIK
+#include <mapnik/version.hpp>
+#endif
+#ifdef HAVE_OAUTH_H
+#include <oauth.h>
+#endif
+
+void a_dialog_build_info ( GtkWindow *parent )
+{
+  GString *msg = g_string_new ( "" );
+  g_string_append_printf ( msg, "GLIB version: %d.%d.%d\n", glib_major_version, glib_minor_version, glib_micro_version  );
+#if GTK_CHECK_VERSION(3,0,0)
+  g_string_append_printf ( msg, "GTK version: %d.%d.%d\n", gtk_get_major_version(), gtk_get_minor_version(), gtk_get_micro_version() );
+#else
+  g_string_append_printf ( msg, "GTK API version: %d.%d.%d\n", GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION );
+#endif
+  g_string_append_printf ( msg, "GDK Pixbuf version: %s\n", gdk_pixbuf_version );
+  g_string_append_printf ( msg, "Cairo version: %s\n", cairo_version_string() );
+  g_string_append_printf ( msg, "Pango version: %s\n", pango_version_string() );
+#ifdef HAVE_EXPAT_H
+  g_string_append_printf ( msg, "Expat version: %s\n", XML_ExpatVersion()  );
+#endif
+#ifdef HAVE_LIBCURL
+  g_string_append_printf ( msg, "libcurl version: %s\n", curl_version() );
+#endif
+#ifdef HAVE_GPS_H
+  g_string_append_printf ( msg, "GPSD API version: %d.%d\n", GPSD_API_MAJOR_VERSION, GPSD_API_MINOR_VERSION );
+#endif
+#ifdef HAVE_LIBGEXIV2
+  g_string_append_printf ( msg, "gexiv2 version: %06d\n", gexiv2_get_version() );
+#endif
+#ifdef HAVE_MAGIC_H
+#ifdef MAGIC_VERSION
+  g_string_append_printf ( msg, "magic API version: %d\n", MAGIC_VERSION );
+#endif
+#endif
+#ifdef HAVE_BZLIB_H
+  g_string_append_printf ( msg, "bzip2 version: %s\n", BZ2_bzlibVersion() );
+#endif
+#ifdef HAVE_ZIP_H
+#ifdef LIBZIP_VERSION
+  g_string_append_printf ( msg, "libzip version: %s\n", LIBZIP_VERSION );
+#endif
+#endif
+#ifdef HAVE_SQLITE3_H
+#ifdef SQLITE_VERSION
+  g_string_append_printf ( msg, "libsqlite version: %s\n", SQLITE_VERSION );
+#endif
+#endif
+#ifdef HAVE_LIBMAPNIK
+#ifdef MAPNIK_VERSION_STRING
+  g_string_append_printf ( msg, "mapnik version: %s\n", MAPNIK_VERSION_STRING );
+#endif
+#endif
+#ifdef HAVE_OAUTH_H
+#ifdef LIBOAUTH_VERSION
+  g_string_append_printf ( msg, "liboauth version: %s\n", LIBOAUTH_VERSION );
+#endif
+#endif
+  a_dialog_info_msg ( parent, msg->str );
+  g_string_free ( msg, TRUE );
+}
