@@ -19,17 +19,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <glib.h>
-#include <glib/gstdio.h>
-#include <glib/gprintf.h>
-#include <glib/gi18n.h>
-
 #include "viking.h"
 #include "geonamessearch.h"
 
@@ -38,16 +27,11 @@
 #define g_mapped_file_unref g_mapped_file_free
 #endif
 
-/**
- * See http://www.geonames.org/export/wikipedia-webservice.html#wikipediaBoundingBox
- */
-// Translators may wish to change this setting as appropriate to get Wikipedia articles in that language
-#define GEONAMES_LANG N_("en")
 // TODO - offer configuration of this value somewhere
 //  ATM decided it's not essential enough to warrant putting in the preferences
 #define GEONAMES_MAX_ENTRIES 20
 
-#define GEONAMES_WIKIPEDIA_URL_FMT "http://api.geonames.org/wikipediaBoundingBoxJSON?formatted=true&north=%s&south=%s&east=%s&west=%s&lang=%s&maxRows=%d&username=viking"
+#define GEONAMES_WIKIPEDIA_URL_FMT "http://api.geonames.org/wikipediaBoundingBoxJSON?formatted=true&north=%s&south=%s&east=%s&west=%s&lang=%s&maxRows=%d&username="VIK_CONFIG_GEONAMES_USERNAME
 
 #define GEONAMES_FEATURE_PATTERN "\"feature\": \""
 #define GEONAMES_LONGITUDE_PATTERN "\"lng\": "
@@ -81,7 +65,7 @@ static found_geoname *new_found_geoname()
   ret->desc = NULL;
   ret->ll.lat = 0.0;
   ret->ll.lon = 0.0;
-  ret->elevation = VIK_DEFAULT_ALTITUDE;
+  ret->elevation = NAN;
   return ret;
 }
 
@@ -412,8 +396,7 @@ static GList *get_entries_from_file(gchar *file_name)
   return(found_places);
 }
 
-
-void a_geonames_wikipedia_box ( VikWindow *vw, VikTrwLayer *vtl, struct LatLon maxmin[2] )
+void a_geonames_wikipedia_box ( VikWindow *vw, VikTrwLayer *vtl, LatLonBBox bbox )
 {
   gchar *uri;
   gchar *tmpname;
@@ -423,12 +406,31 @@ void a_geonames_wikipedia_box ( VikWindow *vw, VikTrwLayer *vtl, struct LatLon m
   VikWaypoint *wiki_wp;
   found_geoname *wiki_geoname;
 
+  /**
+   * See http://www.geonames.org/export/wikipedia-webservice.html#wikipediaBoundingBox
+   */
+  // Wikipedia articles supported languages are de,en,es,fr,it,nl,pl,pt,ru,zh (default = en)
+  gboolean got_language = FALSE;
+  const gchar *language = g_getenv("LANGUAGE");
+  gchar lang[3] = "en";
+  if (strlen(language) > 1) {
+    got_language = TRUE;
+  } else {
+    language = g_getenv("LANG");
+    if (strlen(language) > 1)
+      got_language = TRUE;
+  }
+  if ( got_language ) {
+    lang[0] = language[0];
+    lang[1] = language[1];
+  }
+
   /* encode doubles in a C locale */
-  gchar *north = a_coords_dtostr(maxmin[0].lat);
-  gchar *south = a_coords_dtostr(maxmin[1].lat);
-  gchar *east = a_coords_dtostr(maxmin[0].lon);
-  gchar *west = a_coords_dtostr(maxmin[1].lon);
-  uri = g_strdup_printf ( GEONAMES_WIKIPEDIA_URL_FMT, north, south, east, west, GEONAMES_LANG, GEONAMES_MAX_ENTRIES );
+  gchar *north = a_coords_dtostr(bbox.north);
+  gchar *south = a_coords_dtostr(bbox.south);
+  gchar *east = a_coords_dtostr(bbox.east);
+  gchar *west = a_coords_dtostr(bbox.west);
+  uri = g_strdup_printf ( GEONAMES_WIKIPEDIA_URL_FMT, north, south, east, west, lang, GEONAMES_MAX_ENTRIES );
   g_free(north); north = NULL;
   g_free(south); south = NULL;
   g_free(east);  east = NULL;
@@ -441,9 +443,9 @@ void a_geonames_wikipedia_box ( VikWindow *vw, VikTrwLayer *vtl, struct LatLon m
   wiki_places = get_entries_from_file(tmpname);
   if (g_list_length(wiki_places) == 0) {
     none_found(vw);
-    return;
+    goto done;
   }
-  selected = a_select_geoname_from_list(VIK_GTK_WINDOW_FROM_WIDGET(vw), wiki_places, TRUE, "Select articles", "Select the articles you want to add.");
+  selected = a_select_geoname_from_list(VIK_GTK_WINDOW_FROM_WIDGET(vw), wiki_places, TRUE, _("Select articles"), _("Select the articles you want to add."));
   wp_runner = selected;
   while (wp_runner) {
     wiki_geoname = (found_geoname *)wp_runner->data;
@@ -456,17 +458,21 @@ void a_geonames_wikipedia_box ( VikWindow *vw, VikTrwLayer *vtl, struct LatLon m
     // Use the featue type to generate a suitable waypoint icon
     //  http://www.geonames.org/wikipedia/wikipedia_features.html
     // Only a few values supported as only a few symbols make sense
+    // The feature name may not be translated
+    // Double check the english and the translated
     if ( wiki_geoname->feature ) {
-      if ( !strcmp (wiki_geoname->feature, "city") )
+      if ( !strcmp (wiki_geoname->feature, "city") || !strcmp (wiki_geoname->feature, _("city")))
         vik_waypoint_set_symbol(wiki_wp, "city (medium)");
-      if ( !strcmp (wiki_geoname->feature, "edu") )
+      if ( !strcmp (wiki_geoname->feature, "edu") || !strcmp (wiki_geoname->feature, _("edu")))
         vik_waypoint_set_symbol(wiki_wp, "school");
-      if ( !strcmp (wiki_geoname->feature, "airport") )
+      if ( !strcmp (wiki_geoname->feature, "airport") || !strcmp (wiki_geoname->feature, _("airport")))
         vik_waypoint_set_symbol(wiki_wp, "airport");
-      if ( !strcmp (wiki_geoname->feature, "mountain") )
+      if ( !strcmp (wiki_geoname->feature, "mountain") || !strcmp (wiki_geoname->feature, _("mountain")))
         vik_waypoint_set_symbol(wiki_wp, "summit");
-      if ( !strcmp (wiki_geoname->feature, "forest") )
+      if ( !strcmp (wiki_geoname->feature, "forest") || !strcmp (wiki_geoname->feature, _("forest")))
         vik_waypoint_set_symbol(wiki_wp, "forest");
+      if ( !strcmp (wiki_geoname->feature, "church") || !strcmp (wiki_geoname->feature, _("church")))
+        vik_waypoint_set_symbol(wiki_wp, "church");
     }
     vik_trw_layer_filein_add_waypoint ( vtl, wiki_geoname->name, wiki_wp );
     wp_runner = g_list_next(wp_runner);
@@ -474,8 +480,7 @@ void a_geonames_wikipedia_box ( VikWindow *vw, VikTrwLayer *vtl, struct LatLon m
   free_geoname_list(wiki_places);
   free_geoname_list(selected);
   g_free(uri);
-  if (tmpname) {
-    util_remove(tmpname);
-    g_free(tmpname);
-  }
+done:
+  (void)util_remove(tmpname);
+  g_free(tmpname);
 }
