@@ -33,7 +33,6 @@ G_BEGIN_DECLS
 #define VIKING_URL PACKAGE_URL
 
 #define ALTI_TO_MPP 1.4017295
-#define MPP_TO_ALTI 0.7134044
 
 #define VIK_FEET_IN_METER 3.2808399
 #define VIK_METERS_TO_FEET(X) ((X)*VIK_FEET_IN_METER)
@@ -58,13 +57,37 @@ G_BEGIN_DECLS
 #define VIK_MPS_TO_KPH(X) ((X)*VIK_KPH_IN_MPS)
 #define VIK_KPH_TO_MPS(X) ((X)/VIK_KPH_IN_MPS)
 
+/* Pace is a running unit of "speed" basically defined as the
+time it takes to cover a given distance. It is useful for two
+reasons: first, usually the distance to cover is fixed and
+therefore the needed time can be found with a multiplication
+(instead of a division) which is easy to perform by mind;
+second, it is more sensitive to small differences, so it is
+useful in races when differences between getting the podium
+or not often depends on small differences in speed.
+
+Note that pace is infinite for speed=0, or very large
+for very small speeds, which are usually not interesting
+cases to analyze for a runner. To avoid overflows, and to
+avoid the graph y-scale being overwhelmed by these numbers,
+a cutting at 60s/km or equivalent (in other units) has been set*/
+
+/* PACE_SPK, Seconds per Kilometer */
+#define VIK_MPS_TO_PACE_SPK(X) ((X) > 1.0 ?  3600./((X)      * VIK_KPH_IN_MPS) : 0)
+
+/* PACE_MPK, Minutes per Kilometer */
+#define VIK_MPS_TO_PACE_MPK(X) ((X) > 1.0 ?  60./((X)        * VIK_KPH_IN_MPS) : 0)
+
+/* PACE_SPM, Seconds per Mile */
+#define VIK_MPS_TO_PACE_SPM(X) ((X) > 1.0 ?  5793.6384 /((X) * VIK_KPH_IN_MPS) : 0)
+
+/* PACE_MPM, Minutes per Mile */
+#define VIK_MPS_TO_PACE_MPM(X) ((X) > 1.0 ?  96.56064 /((X)  * VIK_KPH_IN_MPS) : 0)
+
 #define VIK_KNOTS_IN_MPS 1.94384449
 #define VIK_MPS_TO_KNOTS(X) ((X)*VIK_KNOTS_IN_MPS)
 #define VIK_KNOTS_TO_MPS(X) ((X)/VIK_KNOTS_IN_MPS)
 
-#define VIK_DEFAULT_ALTITUDE 0.0
-#define VIK_DEFAULT_DOP 0.0
-
 #define VIK_GTK_WINDOW_FROM_WIDGET(x) GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(x)))
 #define VIK_GTK_WINDOW_FROM_LAYER(x) VIK_GTK_WINDOW_FROM_WIDGET(VIK_LAYER(x)->vt)
 
@@ -114,6 +137,10 @@ typedef enum {
   VIK_UNITS_SPEED_MILES_PER_HOUR,
   VIK_UNITS_SPEED_METRES_PER_SECOND,
   VIK_UNITS_SPEED_KNOTS,
+  VIK_UNITS_SPEED_SECONDS_PER_KM,
+  VIK_UNITS_SPEED_MINUTES_PER_KM,
+  VIK_UNITS_SPEED_SECONDS_PER_MILE,
+  VIK_UNITS_SPEED_MINUTES_PER_MILE,
 } vik_units_speed_t;
 
 vik_units_speed_t a_vik_get_units_speed ( );
@@ -170,6 +197,8 @@ typedef enum {
 
 vik_gpx_export_wpt_sym_name_t a_vik_gpx_export_wpt_sym_name ( );
 
+const gchar* a_vik_gpx_export_creator ( );
+
 #ifndef WINDOWS
 /* Windows automatically uses the system defined viewer
    ATM for other systems need to specify the program to use */
@@ -192,6 +221,25 @@ gboolean a_vik_get_ask_for_create_track_name ( );
 
 gboolean a_vik_get_create_track_tooltip ( );
 
+gboolean a_vik_get_open_files_in_selected_layer ( );
+
+gboolean a_vik_get_calendar_show_day_names ( );
+
+typedef enum {
+ VIK_POSITIONAL_NONE=0,
+ VIK_POSITIONAL_BOTTOM,
+ VIK_POSITIONAL_MIDDLE,
+ VIK_POSITIONAL_TOP,
+} vik_positional_t;
+
+vik_positional_t a_vik_get_ruler_area_label_pos ( );
+
+gboolean a_vik_get_invert_scroll_direction ( );
+
+gboolean a_vik_get_scroll_to_zoom ( );
+
+gboolean a_vik_get_auto_trackpoint_select ( );
+
 gboolean a_vik_get_restore_window_state ( );
 
 gboolean a_vik_get_add_default_map_layer ( );
@@ -229,6 +277,9 @@ gint a_vik_get_recent_number_files ( );
 #define VIKING_PREFERENCES_STARTUP_GROUP_KEY "viking.startup"
 #define VIKING_PREFERENCES_STARTUP_NAMESPACE "viking.startup."
 
+// Additions to 'Stock Icons'
+#define VIK_ICON_SHOW_PICTURE "vik-icon-Show Picture"
+
 G_END_DECLS
 
 #endif
