@@ -19,22 +19,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#ifdef HAVE_MATH_H
-#include <math.h>
-#endif
 
 #include "viking.h"
-
 #include <ctype.h>
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
-
-#include <stdlib.h>
 /* strtod */
 
 typedef struct {
@@ -44,7 +31,6 @@ typedef struct {
 
 static void a_gpspoint_write_track ( const gpointer id, const VikTrack *t, FILE *f );
 static void a_gpspoint_write_trackpoint ( VikTrackpoint *tp, TP_write_info_type *write_info );
-static void a_gpspoint_write_waypoint ( const gpointer id, const VikWaypoint *wp, FILE *f );
 
 /* outline for file gpspoint.c
 
@@ -59,15 +45,17 @@ if trackpoint, make trackpoint, store to current track (error / skip if none)
 */
 
 /* Thanks to etrex-cache's gpsbabel's gpspoint.c for starting me off! */
-
-static char line_buffer[2048];
+#define VIKING_LINE_SIZE 4096
+static char line_buffer[VIKING_LINE_SIZE];
 
 #define GPSPOINT_TYPE_NONE 0
 #define GPSPOINT_TYPE_WAYPOINT 1
 #define GPSPOINT_TYPE_TRACKPOINT 2
 #define GPSPOINT_TYPE_ROUTEPOINT 3
 #define GPSPOINT_TYPE_TRACK 4
-#define GPSPOINT_TYPE_ROUTE 5
+#define GPSPOINT_TYPE_TRACK_END 5
+#define GPSPOINT_TYPE_ROUTE 6
+#define GPSPOINT_TYPE_ROUTE_END 7
 
 static VikTrack *current_track; /* pointer to pointer to first GList */
 
@@ -76,15 +64,18 @@ static struct LatLon line_latlon;
 static gchar *line_name;
 static gchar *line_comment;
 static gchar *line_description;
+static gchar *line_source;
+static gchar *line_xtype;
 static gchar *line_color;
 static gint line_name_label = 0;
 static gint line_dist_label = 0;
 static gchar *line_image;
 static gchar *line_symbol;
+static gdouble line_image_direction = NAN;
+static VikWaypointImageDirectionRef line_image_direction_ref = WP_IMAGE_DIRECTION_REF_TRUE;
 static gboolean line_newsegment = FALSE;
-static gboolean line_has_timestamp = FALSE;
-static time_t line_timestamp = 0;
-static gdouble line_altitude = VIK_DEFAULT_ALTITUDE;
+static gdouble line_timestamp = NAN;
+static gdouble line_altitude = NAN;
 static gboolean line_visible = TRUE;
 
 static gboolean line_extended = FALSE;
@@ -92,16 +83,19 @@ static gdouble line_speed = NAN;
 static gdouble line_course = NAN;
 static gint line_sat = 0;
 static gint line_fix = 0;
+static gdouble line_hdop = NAN;
+static gdouble line_vdop = NAN;
+static gdouble line_pdop = NAN;
 /* other possible properties go here */
 
 
-static void gpspoint_process_tag ( const gchar *tag, gint len );
-static void gpspoint_process_key_and_value ( const gchar *key, gint key_len, const gchar *value, gint value_len );
+static void gpspoint_process_tag ( const gchar *tag, guint len );
+static void gpspoint_process_key_and_value ( const gchar *key, guint key_len, const gchar *value, guint value_len );
 
 static gchar *slashdup(const gchar *str)
 {
-  guint16 len = strlen(str);
-  guint16 need_bs_count, i, j;
+  size_t len = strlen(str);
+  size_t need_bs_count, i, j;
   gchar *rv;
   for ( i = 0, need_bs_count = 0; i < len; i++ )
     if ( str[i] == '\\' || str[i] == '"' )
@@ -112,6 +106,10 @@ static gchar *slashdup(const gchar *str)
     if ( str[i] == '\\' || str[i] == '"' )
       rv[j++] = '\\';
     rv[j] = str[i];
+    // Basic normalization of strings - replace Linefeed and Carriage returns as blanks.
+    //  although allowed in GPX Spec - Viking file format can't handle multi-line strings yet...
+    if ( str[i] == '\n' || str[i] == '\r' )
+      rv[j] = ' ';
   }
   rv[j] = '\0';
   return rv;
@@ -151,6 +149,16 @@ static gchar *deslashndup ( const gchar *str, guint16 len )
   return rv;
 }
 
+
+static void trackpoints_end ()
+{
+  if ( current_track )
+    if ( current_track->trackpoints ) {
+      current_track->trackpoints = g_list_reverse ( current_track->trackpoints );
+      current_track = NULL;
+    }
+}
+
 /*
  * Returns whether file read was a success
  * No obvious way to test for a 'gpspoint' file,
@@ -160,15 +168,15 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
   VikCoordMode coord_mode = vik_trw_layer_get_coord_mode ( trw );
   gchar *tag_start, *tag_end;
   g_assert ( f != NULL && trw != NULL );
-  line_type = 0;
-  line_timestamp = 0;
+  line_type = GPSPOINT_TYPE_NONE;
+  line_timestamp = NAN;
   line_newsegment = FALSE;
   line_image = NULL;
   line_symbol = NULL;
   current_track = NULL;
   gboolean have_read_something = FALSE;
 
-  while (fgets(line_buffer, 2048, f))
+  while (fgets(line_buffer, VIKING_LINE_SIZE, f))
   {
     gboolean inside_quote = 0;
     gboolean backslash = 0;
@@ -189,7 +197,7 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
       /* my addition: find first non-whitespace character. if the null, skip line. */
       while (*tag_start != '\0' && isspace(*tag_start))
         tag_start++;
-      if (tag_start == '\0')
+      if (*tag_start == '\0')
         break;
 
       if (*tag_start == '#')
@@ -208,20 +216,26 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
           inside_quote = !inside_quote;
       }
 
-      gpspoint_process_tag ( tag_start, tag_end - tag_start );
+      // Won't have super massively long strings, so potential truncation in cast is acceptable.
+      guint len = (guint)(tag_end - tag_start);
+      gpspoint_process_tag ( tag_start, len );
 
       if (*tag_end == '\0' )
         break;
       else
         tag_start = tag_end+1;
     }
+    if (line_type == GPSPOINT_TYPE_TRACK_END || line_type == GPSPOINT_TYPE_ROUTE_END) {
+      trackpoints_end ();
+    }
     if (line_type == GPSPOINT_TYPE_WAYPOINT && line_name)
     {
+      // Handle a badly formatted file in case of missing explicit track/route end (this shouldn't happen)
+      trackpoints_end ();
       have_read_something = TRUE;
       VikWaypoint *wp = vik_waypoint_new();
       wp->visible = line_visible;
       wp->altitude = line_altitude;
-      wp->has_timestamp = line_has_timestamp;
       wp->timestamp = line_timestamp;
 
       vik_coord_load_from_latlon ( &(wp->coord), coord_mode, &line_latlon );
@@ -236,18 +250,21 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
       if ( line_description )
         vik_waypoint_set_description ( wp, line_description );
 
+      if ( line_source )
+        vik_waypoint_set_source ( wp, line_source );
+
+      if ( line_xtype )
+        vik_waypoint_set_type ( wp, line_xtype );
+
       if ( line_image ) {
-        // Ensure the filename is absolute
-        if ( g_path_is_absolute ( line_image ) )
-          vik_waypoint_set_image ( wp, line_image );
-        else {
-          // Otherwise create the absolute filename from the directory of the .vik file & and the relative filename
-          gchar *full = g_strconcat(dirpath, G_DIR_SEPARATOR_S, line_image, NULL);
-          gchar *absolute = file_realpath_dup ( full ); // resolved into the canonical name
-          vik_waypoint_set_image ( wp, absolute );
-          g_free ( absolute );
-          g_free ( full );
-        }
+        gchar *fn = util_make_absolute_filename ( line_image, dirpath );
+        vik_waypoint_set_image ( wp, fn ? fn : line_image );
+        g_free ( fn );
+      }
+
+      if ( !isnan(line_image_direction) ) {
+        wp->image_direction = line_image_direction;
+        wp->image_direction_ref = line_image_direction_ref;
       }
 
       if ( line_symbol )
@@ -255,6 +272,8 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
     }
     else if ((line_type == GPSPOINT_TYPE_TRACK || line_type == GPSPOINT_TYPE_ROUTE) && line_name)
     {
+      // Handle a badly formatted file in case of missing explicit track/route end (this shouldn't happen)
+      trackpoints_end ();
       have_read_something = TRUE;
       VikTrack *pl = vik_track_new();
       // NB don't set defaults here as all properties are stored in the GPS_POINT format
@@ -272,6 +291,12 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
       if ( line_description )
         vik_track_set_description ( pl, line_description );
 
+      if ( line_source )
+        vik_track_set_source ( pl, line_source );
+
+      if ( line_xtype )
+        vik_track_set_type ( pl, line_xtype );
+
       if ( line_color )
       {
         if ( gdk_color_parse ( line_color, &(pl->color) ) )
@@ -294,7 +319,6 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
       VikTrackpoint *tp = vik_trackpoint_new();
       vik_coord_load_from_latlon ( &(tp->coord), coord_mode, &line_latlon );
       tp->newsegment = line_newsegment;
-      tp->has_timestamp = line_has_timestamp;
       tp->timestamp = line_timestamp;
       tp->altitude = line_altitude;
       vik_trackpoint_set_name ( tp, line_name );
@@ -303,8 +327,13 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
         tp->course = line_course;
         tp->nsats = line_sat;
         tp->fix_mode = line_fix;
+        tp->hdop = line_hdop;
+        tp->vdop = line_vdop;
+        tp->pdop = line_pdop;
       }
-      current_track->trackpoints = g_list_append ( current_track->trackpoints, tp );
+      // Much faster to prepend and then reverse list once all points read in
+      // Especially if hunderds of thousands or more trackpoints in a file
+      current_track->trackpoints = g_list_prepend ( current_track->trackpoints, tp );
     }
 
     if (line_name) 
@@ -314,6 +343,10 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
       g_free ( line_comment );
     if (line_description)
       g_free ( line_description );
+    if (line_source)
+      g_free ( line_source );
+    if (line_xtype)
+      g_free ( line_xtype );
     if (line_color)
       g_free ( line_color );
     if (line_image)
@@ -322,14 +355,17 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
       g_free ( line_symbol );
     line_comment = NULL;
     line_description = NULL;
+    line_source = NULL;
+    line_xtype = NULL;
     line_color = NULL;
     line_image = NULL;
+    line_image_direction = NAN;
+    line_image_direction_ref = WP_IMAGE_DIRECTION_REF_TRUE;
     line_symbol = NULL;
     line_type = GPSPOINT_TYPE_NONE;
     line_newsegment = FALSE;
-    line_has_timestamp = FALSE;
-    line_timestamp = 0;
-    line_altitude = VIK_DEFAULT_ALTITUDE;
+    line_timestamp = NAN;
+    line_altitude = NAN;
     line_visible = TRUE;
     line_symbol = NULL;
 
@@ -338,10 +374,16 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
     line_course = NAN;
     line_sat = 0;
     line_fix = 0;
+    line_hdop = NAN;
+    line_vdop = NAN;
+    line_pdop = NAN;
     line_name_label = 0;
     line_dist_label = 0;
   }
 
+  // Handle a badly formatted file in case of missing explicit track/route end (this shouldn't happen)
+  trackpoints_end ();
+
   return have_read_something;
 }
 
@@ -353,7 +395,7 @@ gboolean a_gpspoint_read_file(VikTrwLayer *trw, FILE *f, const gchar *dirpath )
 
 So we must determine end of tag name, start of value, end of value.
 */
-static void gpspoint_process_tag ( const gchar *tag, gint len )
+static void gpspoint_process_tag ( const gchar *tag, guint len )
 {
   const gchar *key_end, *value_start, *value_end;
 
@@ -381,7 +423,7 @@ static void gpspoint_process_tag ( const gchar *tag, gint len )
       else
       {
         if (*(tag+len-1) == '"')
-        value_end = tag + len - 1;
+          value_end = tag + len - 1;
         else
           return; /* bogus */
       }
@@ -389,6 +431,10 @@ static void gpspoint_process_tag ( const gchar *tag, gint len )
     else
       value_end = tag + len; /* value start really IS value start. */
 
+    // Detect broken lines which end without any text or the enclosing ". i.e. like: comment="
+    if ( (value_end - value_start) < 0 )
+      return;
+
     gpspoint_process_key_and_value(tag, key_end - tag, value_start, value_end - value_start);
   }
 }
@@ -396,7 +442,7 @@ static void gpspoint_process_tag ( const gchar *tag, gint len )
 /*
 value = NULL for none
 */
-static void gpspoint_process_key_and_value ( const gchar *key, gint key_len, const gchar *value, gint value_len )
+static void gpspoint_process_key_and_value ( const gchar *key, guint key_len, const gchar *value, guint value_len )
 {
   if (key_len == 4 && strncasecmp( key, "type", key_len ) == 0 )
   {
@@ -404,12 +450,16 @@ static void gpspoint_process_key_and_value ( const gchar *key, gint key_len, con
       line_type = GPSPOINT_TYPE_NONE;
     else if (value_len == 5 && strncasecmp( value, "track", value_len ) == 0 )
       line_type = GPSPOINT_TYPE_TRACK;
+    else if (value_len == 8 && strncasecmp( value, "trackend", value_len ) == 0 )
+      line_type = GPSPOINT_TYPE_TRACK_END;
     else if (value_len == 10 && strncasecmp( value, "trackpoint", value_len ) == 0 )
       line_type = GPSPOINT_TYPE_TRACKPOINT;
     else if (value_len == 8 && strncasecmp( value, "waypoint", value_len ) == 0 )
       line_type = GPSPOINT_TYPE_WAYPOINT;
     else if (value_len == 5 && strncasecmp( value, "route", value_len ) == 0 )
       line_type = GPSPOINT_TYPE_ROUTE;
+    else if (value_len == 8 && strncasecmp( value, "routeend", value_len ) == 0 )
+      line_type = GPSPOINT_TYPE_ROUTE_END;
     else if (value_len == 10 && strncasecmp( value, "routepoint", value_len ) == 0 )
       line_type = GPSPOINT_TYPE_ROUTEPOINT;
     else
@@ -433,6 +483,17 @@ static void gpspoint_process_key_and_value ( const gchar *key, gint key_len, con
     if (line_description == NULL)
       line_description = deslashndup ( value, value_len );
   }
+  else if (key_len == 6 && strncasecmp( key, "source", key_len ) == 0 && value != NULL)
+  {
+    if (line_source == NULL)
+      line_source = deslashndup ( value, value_len );
+  }
+  // NB using 'xtype' to differentiate from our own 'type' key
+  else if (key_len == 5 && strncasecmp( key, "xtype", key_len ) == 0 && value != NULL)
+  {
+    if (line_xtype == NULL)
+      line_xtype = deslashndup ( value, value_len );
+  }
   else if (key_len == 5 && strncasecmp( key, "color", key_len ) == 0 && value != NULL)
   {
     if (line_color == NULL)
@@ -451,6 +512,14 @@ static void gpspoint_process_key_and_value ( const gchar *key, gint key_len, con
     if (line_image == NULL)
       line_image = deslashndup ( value, value_len );
   }
+  else if (key_len == 15 && strncasecmp( key, "image_direction", key_len ) == 0 && value != NULL)
+  {
+    line_image_direction = g_ascii_strtod(value, NULL);
+  }
+  else if (key_len == 19 && strncasecmp( key, "image_direction_ref", key_len ) == 0 && value != NULL)
+  {
+    line_image_direction_ref = atoi(value);
+  }
   else if (key_len == 8 && strncasecmp( key, "latitude", key_len ) == 0 && value != NULL)
   {
     line_latlon.lat = g_ascii_strtod(value, NULL);
@@ -463,7 +532,7 @@ static void gpspoint_process_key_and_value ( const gchar *key, gint key_len, con
   {
     line_altitude = g_ascii_strtod(value, NULL);
   }
-  else if (key_len == 7 && strncasecmp( key, "visible", key_len ) == 0 && value[0] != 'y' && value[0] != 'Y' && value[0] != 't' && value[0] != 'T')
+  else if (key_len == 7 && strncasecmp( key, "visible", key_len ) == 0 && value != NULL && value[0] != 'y' && value[0] != 'Y' && value[0] != 't' && value[0] != 'T')
   {
     line_visible = FALSE;
   }
@@ -474,8 +543,6 @@ static void gpspoint_process_key_and_value ( const gchar *key, gint key_len, con
   else if (key_len == 8 && strncasecmp( key, "unixtime", key_len ) == 0 && value != NULL)
   {
     line_timestamp = g_ascii_strtod(value, NULL);
-    if ( line_timestamp != 0x80000000 )
-      line_has_timestamp = TRUE;
   }
   else if (key_len == 10 && strncasecmp( key, "newsegment", key_len ) == 0 && value != NULL)
   {
@@ -501,32 +568,57 @@ static void gpspoint_process_key_and_value ( const gchar *key, gint key_len, con
   {
     line_fix = atoi(value);
   }
+  else if (key_len == 4 && strncasecmp( key, "hdop", key_len ) == 0 && value != NULL)
+  {
+    line_hdop = g_ascii_strtod(value, NULL);
+  }
+  else if (key_len == 4 && strncasecmp( key, "vdop", key_len ) == 0 && value != NULL)
+  {
+    line_vdop = g_ascii_strtod(value, NULL);
+  }
+  else if (key_len == 4 && strncasecmp( key, "pdop", key_len ) == 0 && value != NULL)
+  {
+    line_pdop = g_ascii_strtod(value, NULL);
+  }
 }
 
-static void a_gpspoint_write_waypoint ( const gpointer id, const VikWaypoint *wp, FILE *f )
+typedef struct {
+  FILE *file;
+  const gchar *dirpath;
+} WritingContext;
+
+static void a_gpspoint_write_waypoint ( const gpointer id, const VikWaypoint *wp, WritingContext *wc )
 {
-  static struct LatLon ll;
-  gchar *s_lat, *s_lon;
-  // Sanity clause
-  if ( wp && !(wp->name) ) {
+  struct LatLon ll;
+  gchar s_lat[COORDS_STR_BUFFER_SIZE];
+  gchar s_lon[COORDS_STR_BUFFER_SIZE];
+  // Sanity clauses
+  if ( !wp )
     return;
-  }
+  if ( !(wp->name) )
+    return;
+  if ( !wc )
+    return;
+
+  FILE *f = wc->file;
+
   vik_coord_to_latlon ( &(wp->coord), &ll );
-  s_lat = a_coords_dtostr(ll.lat);
-  s_lon = a_coords_dtostr(ll.lon);
+  a_coords_dtostr_buffer ( ll.lat, s_lat );
+  a_coords_dtostr_buffer ( ll.lon, s_lon );
   gchar *tmp_name = slashdup(wp->name);
   fprintf ( f, "type=\"waypoint\" latitude=\"%s\" longitude=\"%s\" name=\"%s\"", s_lat, s_lon, tmp_name );
   g_free ( tmp_name );
-  g_free ( s_lat ); 
-  g_free ( s_lon );
 
-  if ( wp->altitude != VIK_DEFAULT_ALTITUDE ) {
-    gchar *s_alt = a_coords_dtostr(wp->altitude);
+  if ( !isnan(wp->altitude) ) {
+    gchar s_alt[COORDS_STR_BUFFER_SIZE];
+    a_coords_dtostr_buffer ( wp->altitude, s_alt );
     fprintf ( f, " altitude=\"%s\"", s_alt );
-    g_free(s_alt);
   }
-  if ( wp->has_timestamp )
-    fprintf ( f, " unixtime=\"%ld\"", wp->timestamp );
+  if ( !isnan(wp->timestamp) ) {
+    gchar s_tm[COORDS_STR_BUFFER_SIZE];
+    a_coords_dtostr_buffer ( wp->timestamp, s_tm );
+    fprintf ( f, " unixtime=\"%s\"", s_tm );
+  }
   if ( wp->comment )
   {
     gchar *tmp_comment = slashdup(wp->comment);
@@ -539,27 +631,43 @@ static void a_gpspoint_write_waypoint ( const gpointer id, const VikWaypoint *wp
     fprintf ( f, " description=\"%s\"", tmp_description );
     g_free ( tmp_description );
   }
+  if ( wp->source )
+  {
+    gchar *tmp_source = slashdup(wp->source);
+    fprintf ( f, " source=\"%s\"", tmp_source );
+    g_free ( tmp_source );
+  }
+  if ( wp->type )
+  {
+    gchar *tmp_type = slashdup(wp->type);
+    fprintf ( f, " xtype=\"%s\"", tmp_type );
+    g_free ( tmp_type );
+  }
   if ( wp->image )
   {
     gchar *tmp_image = NULL;
-    gchar *cwd = NULL;
     if ( a_vik_get_file_ref_format() == VIK_FILE_REF_FORMAT_RELATIVE ) {
-      cwd = g_get_current_dir();
-      if ( cwd )
-        tmp_image = g_strdup ( file_GetRelativeFilename ( cwd, wp->image ) );
+      if ( wc->dirpath )
+        tmp_image = g_strdup ( file_GetRelativeFilename ( (gchar*)wc->dirpath, wp->image ) );
     }
 
-    // if cwd not available - use image filename as is
+    // if tmp_image not available - use image filename as is
     // this should be an absolute path as set in thumbnails
-    if ( !cwd )
+    if ( !tmp_image )
       tmp_image = slashdup(wp->image);
 
     if ( tmp_image )
       fprintf ( f, " image=\"%s\"", tmp_image );
 
-    g_free ( cwd );
     g_free ( tmp_image );
   }
+  if ( !isnan(wp->image_direction) )
+  {
+    gchar *tmp = util_formatd ( "%.2f", wp->image_direction );
+    fprintf ( f, " image_direction=\"%s\"", tmp );
+    g_free ( tmp );
+    fprintf ( f, " image_direction_ref=\"%d\"", wp->image_direction_ref );
+  }
   if ( wp->symbol )
   {
     // Due to changes in garminsymbols - the symbol name is now in Title Case
@@ -576,19 +684,17 @@ static void a_gpspoint_write_waypoint ( const gpointer id, const VikWaypoint *wp
 
 static void a_gpspoint_write_trackpoint ( VikTrackpoint *tp, TP_write_info_type *write_info )
 {
-  static struct LatLon ll;
-  gchar *s_lat, *s_lon;
+  struct LatLon ll;
+  gchar s_lat[COORDS_STR_BUFFER_SIZE];
+  gchar s_lon[COORDS_STR_BUFFER_SIZE];
+  gchar s_alt[COORDS_STR_BUFFER_SIZE];
   vik_coord_to_latlon ( &(tp->coord), &ll );
 
   FILE *f = write_info->f;
 
-  /* TODO: modify a_coords_dtostr() to accept (optional) buffer
-   * instead of doing malloc/free everytime */
-  s_lat = a_coords_dtostr(ll.lat);
-  s_lon = a_coords_dtostr(ll.lon);
+  a_coords_dtostr_buffer ( ll.lat, s_lat );
+  a_coords_dtostr_buffer ( ll.lon, s_lon );
   fprintf ( f, "type=\"%spoint\" latitude=\"%s\" longitude=\"%s\"", write_info->is_route ? "route" : "track", s_lat, s_lon );
-  g_free ( s_lat ); 
-  g_free ( s_lon );
 
   if ( tp->name ) {
     gchar *name = slashdup(tp->name);
@@ -596,32 +702,50 @@ static void a_gpspoint_write_trackpoint ( VikTrackpoint *tp, TP_write_info_type
     g_free(name);
   }
 
-  if ( tp->altitude != VIK_DEFAULT_ALTITUDE ) {
-    gchar *s_alt = a_coords_dtostr(tp->altitude);
+  if ( !isnan(tp->altitude) ) {
+    a_coords_dtostr_buffer ( tp->altitude, s_alt );
     fprintf ( f, " altitude=\"%s\"", s_alt );
-    g_free(s_alt);
   }
-  if ( tp->has_timestamp )
-    fprintf ( f, " unixtime=\"%ld\"", tp->timestamp );
+  if ( !isnan(tp->timestamp) ) {
+    gchar s_tm[COORDS_STR_BUFFER_SIZE];
+    a_coords_dtostr_buffer ( tp->timestamp, s_tm );
+    fprintf ( f, " unixtime=\"%s\"", s_tm );
+  }
   if ( tp->newsegment )
     fprintf ( f, " newsegment=\"yes\"" );
 
   if (!isnan(tp->speed) || !isnan(tp->course) || tp->nsats > 0) {
     fprintf ( f, " extended=\"yes\"" );
     if (!isnan(tp->speed)) {
-      gchar *s_speed = a_coords_dtostr(tp->speed);
+      gchar s_speed[COORDS_STR_BUFFER_SIZE];
+      a_coords_dtostr_buffer ( tp->speed, s_speed );
       fprintf ( f, " speed=\"%s\"", s_speed );
-      g_free(s_speed);
     }
     if (!isnan(tp->course)) {
-      gchar *s_course = a_coords_dtostr(tp->course);
+      gchar s_course[COORDS_STR_BUFFER_SIZE];
+      a_coords_dtostr_buffer ( tp->course, s_course );
       fprintf ( f, " course=\"%s\"", s_course );
-      g_free(s_course);
     }
     if (tp->nsats > 0)
       fprintf ( f, " sat=\"%d\"", tp->nsats );
     if (tp->fix_mode > 0)
       fprintf ( f, " fix=\"%d\"", tp->fix_mode );
+
+    if ( !isnan(tp->hdop) ) {
+      gchar ss[COORDS_STR_BUFFER_SIZE];
+      a_coords_dtostr_buffer ( tp->hdop, ss );
+      fprintf ( f, " hdop=\"%s\"", ss );
+    }
+    if ( !isnan(tp->vdop) ) {
+      gchar ss[COORDS_STR_BUFFER_SIZE];
+      a_coords_dtostr_buffer ( tp->vdop, ss );
+      fprintf ( f, " vdop=\"%s\"", ss );
+    }
+    if ( !isnan(tp->pdop) ) {
+      gchar ss[COORDS_STR_BUFFER_SIZE];
+      a_coords_dtostr_buffer ( tp->pdop, ss );
+      fprintf ( f, " pdop=\"%s\"", ss );
+    }
   }
   fprintf ( f, "\n" );
 }
@@ -651,6 +775,18 @@ static void a_gpspoint_write_track ( const gpointer id, const VikTrack *trk, FIL
     g_free ( tmp );
   }
 
+  if ( trk->source ) {
+    gchar *tmp = slashdup(trk->source);
+    fprintf ( f, " source=\"%s\"", tmp );
+    g_free ( tmp );
+  }
+
+  if ( trk->type ) {
+    gchar *tmp = slashdup(trk->type);
+    fprintf ( f, " xtype=\"%s\"", tmp );
+    g_free ( tmp );
+  }
+
   if ( trk->has_color ) {
     fprintf ( f, " color=#%.2x%.2x%.2x", (int)(trk->color.red/256),(int)(trk->color.green/256),(int)(trk->color.blue/256));
   }
@@ -671,14 +807,15 @@ static void a_gpspoint_write_track ( const gpointer id, const VikTrack *trk, FIL
   fprintf ( f, "type=\"%send\"\n", trk->is_route ? "route" : "track" );
 }
 
-void a_gpspoint_write_file ( VikTrwLayer *trw, FILE *f )
+void a_gpspoint_write_file ( VikTrwLayer *trw, FILE *f, const gchar *dirpath )
 {
   GHashTable *tracks = vik_trw_layer_get_tracks ( trw );
   GHashTable *routes = vik_trw_layer_get_routes ( trw );
   GHashTable *waypoints = vik_trw_layer_get_waypoints ( trw );
 
+  WritingContext wc = { f, dirpath };
   fprintf ( f, "type=\"waypointlist\"\n" );
-  g_hash_table_foreach ( waypoints, (GHFunc) a_gpspoint_write_waypoint, f );
+  g_hash_table_foreach ( waypoints, (GHFunc) a_gpspoint_write_waypoint, &wc );
   fprintf ( f, "type=\"waypointlistend\"\n" );
   g_hash_table_foreach ( tracks, (GHFunc) a_gpspoint_write_track, f );
   g_hash_table_foreach ( routes, (GHFunc) a_gpspoint_write_track, f );
