@@ -1002,7 +1002,7 @@ static TBEditorWidget *tb_editor_create_dialog(VikToolbar *vtb, GtkWindow *paren
 		return NULL;
 	}
 
-	TBEditorWidget *tbw = g_new(TBEditorWidget, 1);
+	TBEditorWidget *tbw = g_new0(TBEditorWidget, 1);
 
 	dialog = gtk_dialog_new_with_buttons(_("Customize Toolbar"),
 	                                     GTK_WINDOW(parent),
@@ -1173,8 +1173,11 @@ void toolbar_configure (VikToolbar *vtb, GtkWidget *toolbar, GtkWindow *parent,
 	}
 	foreach_slist(sl, used_items)
 	{
-		gtk_list_store_append(tbw->store_used, &iter);
-		tb_editor_set_item_values(vtb, sl->data, tbw->store_used, &iter);
+		// Ensure only valid actions are added to the list
+		if ( get_action(vtb, sl->data) ) {
+			gtk_list_store_append(tbw->store_used, &iter);
+			tb_editor_set_item_values(vtb, sl->data, tbw->store_used, &iter);
+		}
 	}
 	/* select first item */
 	path = gtk_tree_path_new_from_string("0");
