@@ -22,21 +22,12 @@
 #include "config.h"
 #endif
 
-#ifdef HAVE_MATH_H
-#include <math.h>
-#endif
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
-#include <stdlib.h>
-#include <glib/gstdio.h>
-#include <glib/gi18n.h>
 
 #include "background.h"
 #include "viking.h"
@@ -45,12 +36,14 @@
 #include "dem.h"
 #include "dems.h"
 #include "icons/icons.h"
+#include "bbox.h"
 
 #define MAPS_CACHE_DIR maps_layer_default_dir()
-
 #define SRTM_CACHE_TEMPLATE "%ssrtm3-%s%s%c%02d%c%03d.hgt.zip"
-#define SRTM_HTTP_SITE "dds.cr.usgs.gov"
-#define SRTM_HTTP_URI  "/srtm/version2_1/SRTM3/"
+
+#define SRTM_HTTP_BASE_URL "https://dds.cr.usgs.gov/srtm/version2_1/SRTM3"
+static gchar *base_url = NULL;
+#define VIK_SETTINGS_SRTM_HTTP_BASE_URL "srtm_http_base_url"
 
 #ifdef VIK_CONFIG_DEM24K
 #define DEM24K_DOWNLOAD_SCRIPT "dem24k.pl"
@@ -63,9 +56,9 @@ static void dem_layer_draw ( VikDEMLayer *vdl, VikViewport *vp );
 static void dem_layer_free ( VikDEMLayer *vdl );
 static VikDEMLayer *dem_layer_create ( VikViewport *vp );
 static const gchar* dem_layer_tooltip( VikDEMLayer *vdl );
-static void dem_layer_marshall( VikDEMLayer *vdl, guint8 **data, gint *len );
-static VikDEMLayer *dem_layer_unmarshall( guint8 *data, gint len, VikViewport *vvp );
-static gboolean dem_layer_set_param ( VikDEMLayer *vdl, guint16 id, VikLayerParamData data, VikViewport *vp, gboolean is_file_operation );
+static void dem_layer_marshall( VikDEMLayer *vdl, guint8 **data, guint *len );
+static VikDEMLayer *dem_layer_unmarshall( guint8 *data, guint len, VikViewport *vvp );
+static gboolean dem_layer_set_param ( VikDEMLayer *vdl, VikLayerSetParam *vlsp );
 static VikLayerParamData dem_layer_get_param ( VikDEMLayer *vdl, guint16 id, gboolean is_file_operation );
 static void dem_layer_post_read ( VikLayer *vl, VikViewport *vp, gboolean from_file );
 static void srtm_draw_existence ( VikViewport *vp );
@@ -81,7 +74,7 @@ static VikLayerParamScale param_scales[] = {
 };
 
 static gchar *params_source[] = {
-	"SRTM Global 90m (3 arcsec)",
+	N_("SRTM Global 90m (3 arcsec)"),
 #ifdef VIK_CONFIG_DEM24K
 	"USA 10m (USGS 24k)",
 #endif
@@ -131,10 +124,12 @@ static gboolean dem_layer_download_release ( VikDEMLayer *vdl, GdkEventButton *e
 static gboolean dem_layer_download_click ( VikDEMLayer *vdl, GdkEventButton *event, VikViewport *vvp );
 
 static VikToolInterface dem_tools[] = {
-  { { "DEMDownload", "vik-icon-DEM Download", N_("_DEM Download"), NULL, N_("DEM Download"), 0 },
+  { &demdl_18_pixbuf,
+    { "DEMDownload", "vik-icon-DEM Download", N_("_DEM Download"), NULL, N_("DEM Download"), 0 },
     (VikToolConstructorFunc) dem_layer_download_create, NULL, NULL, NULL,
     (VikToolMouseFunc) dem_layer_download_click, NULL,  (VikToolMouseFunc) dem_layer_download_release,
     (VikToolKeyFunc) NULL,
+    (VikToolKeyFunc) NULL,
     FALSE,
     GDK_CURSOR_IS_PIXMAP, &cursor_demdl_pixbuf, NULL },
 };
@@ -190,7 +185,7 @@ VikLayerInterface vik_dem_layer_interface = {
   &vikdemlayer_pixbuf,
 
   dem_tools,
-  sizeof(dem_tools) / sizeof(dem_tools[0]),
+  G_N_ELEMENTS(dem_tools),
 
   dem_layer_params,
   NUM_PARAMS,
@@ -243,6 +238,8 @@ VikLayerInterface vik_dem_layer_interface = {
   (VikLayerFuncSelectMove)              NULL,
   (VikLayerFuncSelectRelease)           NULL,
   (VikLayerFuncSelectedViewportMenu)    NULL,
+
+  (VikLayerFuncRefresh)                 NULL,
 };
 
 struct _VikDEMLayer {
@@ -260,6 +257,16 @@ struct _VikDEMLayer {
   GtkMenu *right_click_menu;
 };
 
+// NB Only performed once per program run
+static void vik_dem_class_init ( VikDEMLayerClass *klass )
+{
+  // Note if suppling your own base URL - the site must still follow the Continent directory layout
+  if ( ! a_settings_get_string ( VIK_SETTINGS_SRTM_HTTP_BASE_URL, &base_url ) ) {
+    // Otherwise use the default
+    base_url = g_strdup ( SRTM_HTTP_BASE_URL );
+  }
+}
+
 GType vik_dem_layer_get_type ()
 {
   static GType vdl_type = 0;
@@ -271,7 +278,7 @@ GType vik_dem_layer_get_type ()
       sizeof (VikDEMLayerClass),
       NULL, /* base_init */
       NULL, /* base_finalize */
-      NULL, /* class init */
+      (GClassInitFunc) vik_dem_class_init, /* class init */
       NULL, /* class_finalize */
       NULL, /* class_data */
       sizeof (VikDEMLayer),
@@ -291,12 +298,12 @@ static const gchar* dem_layer_tooltip( VikDEMLayer *vdl )
   return tmp_buf;
 }
 
-static void dem_layer_marshall( VikDEMLayer *vdl, guint8 **data, gint *len )
+static void dem_layer_marshall( VikDEMLayer *vdl, guint8 **data, guint *len )
 {
   vik_layer_marshall_params ( VIK_LAYER(vdl), data, len );
 }
 
-static VikDEMLayer *dem_layer_unmarshall( guint8 *data, gint len, VikViewport *vvp )
+static VikDEMLayer *dem_layer_unmarshall( guint8 *data, guint len, VikViewport *vvp )
 {
   VikDEMLayer *rv = dem_layer_new ( vvp );
   gint i;
@@ -334,11 +341,9 @@ static int dem_layer_load_list_thread ( dem_load_thread_data *dltd, gpointer thr
 
   // ATM as each file is processed the screen is not updated (no mechanism exposed to a_dems_load_list)
   // Thus force draw only at the end, as loading is complete/aborted
-  //gdk_threads_enter();
   // Test is helpful to prevent Gtk-CRITICAL warnings if the program is exitted whilst loading
   if ( IS_VIK_LAYER(dltd->vdl) )
-    vik_layer_emit_update ( VIK_LAYER(dltd->vdl) ); // NB update from background thread
-  //gdk_threads_leave();
+    vik_layer_emit_update ( VIK_LAYER(dltd->vdl) ); // NB update requested from background thread
 
   return result;
 }
@@ -390,50 +395,53 @@ static GList *dem_layer_convert_to_relative_filenaming ( GList *files )
   return files;
 }
 
-gboolean dem_layer_set_param ( VikDEMLayer *vdl, guint16 id, VikLayerParamData data, VikViewport *vp, gboolean is_file_operation )
+gboolean dem_layer_set_param ( VikDEMLayer *vdl, VikLayerSetParam *vlsp )
 {
-  switch ( id )
+  switch ( vlsp->id )
   {
-    case PARAM_COLOR: vdl->color = data.c; gdk_gc_set_rgb_fg_color ( vdl->gcs[0], &(vdl->color) ); break;
-    case PARAM_SOURCE: vdl->source = data.u; break;
-    case PARAM_TYPE: vdl->type = data.u; break;
+    case PARAM_COLOR: vdl->color = vlsp->data.c; gdk_gc_set_rgb_fg_color ( vdl->gcs[0], &(vdl->color) ); break;
+    case PARAM_SOURCE: vdl->source = vlsp->data.u; break;
+    case PARAM_TYPE: vdl->type = vlsp->data.u; break;
     case PARAM_MIN_ELEV:
       /* Convert to store internally
          NB file operation always in internal units (metres) */
-      if (!is_file_operation && a_vik_get_units_height () == VIK_UNITS_HEIGHT_FEET )
-        vdl->min_elev = VIK_FEET_TO_METERS(data.d);
+      if (!vlsp->is_file_operation && a_vik_get_units_height () == VIK_UNITS_HEIGHT_FEET )
+        vdl->min_elev = VIK_FEET_TO_METERS(vlsp->data.d);
       else
-        vdl->min_elev = data.d;
+        vdl->min_elev = vlsp->data.d;
       break;
     case PARAM_MAX_ELEV:
       /* Convert to store internally
          NB file operation always in internal units (metres) */
-      if (!is_file_operation && a_vik_get_units_height () == VIK_UNITS_HEIGHT_FEET )
-        vdl->max_elev = VIK_FEET_TO_METERS(data.d);
+      if (!vlsp->is_file_operation && a_vik_get_units_height () == VIK_UNITS_HEIGHT_FEET )
+        vdl->max_elev = VIK_FEET_TO_METERS(vlsp->data.d);
       else
-        vdl->max_elev = data.d;
+        vdl->max_elev = vlsp->data.d;
       break;
     case PARAM_FILES:
     {
       // Clear out old settings - if any commonalities with new settings they will have to be read again
       a_dems_list_free ( vdl->files );
+
       // Set file list so any other intermediate screen drawing updates will show currently loaded DEMs by the working thread
-      vdl->files = data.sl;
+      vdl->files = vlsp->data.sl;
+      // Ensure resolving of any relative path names
+      util_make_absolute_filenames ( vdl->files, vlsp->dirpath );
+
       // No need for thread if no files
       if ( vdl->files ) {
         // Thread Load
         dem_load_thread_data *dltd = g_malloc ( sizeof(dem_load_thread_data) );
         dltd->vdl = vdl;
-        dltd->vdl->files = data.sl;
 
         a_background_thread ( BACKGROUND_POOL_LOCAL,
-                              VIK_GTK_WINDOW_FROM_WIDGET(vp),
+                              VIK_GTK_WINDOW_FROM_WIDGET(vlsp->vp),
                               _("DEM Loading"),
                               (vik_thr_func) dem_layer_load_list_thread,
                               dltd,
                               (vik_thr_free_func) dem_layer_thread_data_free,
                               (vik_thr_free_func) dem_layer_thread_cancel,
-                              g_list_length ( data.sl ) ); // Number of DEM files
+                              g_list_length ( vlsp->data.sl ) ); // Number of DEM files
       }
       break;
     }
@@ -512,47 +520,18 @@ static inline guint16 get_height_difference(gint16 elev, gint16 new_elev)
     return abs(new_elev - elev);
 }
 
-
 static void vik_dem_layer_draw_dem ( VikDEMLayer *vdl, VikViewport *vp, VikDEM *dem )
 {
   VikDEMColumn *column, *prevcolumn, *nextcolumn;
 
-  struct LatLon dem_northeast, dem_southwest;
-  gdouble max_lat, max_lon, min_lat, min_lon;
+  LatLonBBox vp_bbox = vik_viewport_get_bbox ( vp );
+  LatLonBBox dem_bbox = vik_dem_get_bbox ( dem );
 
   /**** Check if viewport and DEM data overlap ****/
-
-  /* get min, max lat/lon of viewport */
-  vik_viewport_get_min_max_lat_lon ( vp, &min_lat, &max_lat, &min_lon, &max_lon );
-
-  /* get min, max lat/lon of DEM data */
-  if ( dem->horiz_units == VIK_DEM_HORIZ_LL_ARCSECONDS ) {
-    dem_northeast.lat = dem->max_north / 3600.0;
-    dem_northeast.lon = dem->max_east / 3600.0;
-    dem_southwest.lat = dem->min_north / 3600.0;
-    dem_southwest.lon = dem->min_east / 3600.0;
-  } else if ( dem->horiz_units == VIK_DEM_HORIZ_UTM_METERS ) {
-    struct UTM dem_northeast_utm, dem_southwest_utm;
-    dem_northeast_utm.northing = dem->max_north;
-    dem_northeast_utm.easting = dem->max_east;
-    dem_southwest_utm.northing = dem->min_north;
-    dem_southwest_utm.easting = dem->min_east;
-    dem_northeast_utm.zone = dem_southwest_utm.zone = dem->utm_zone;
-    dem_northeast_utm.letter = dem_southwest_utm.letter = dem->utm_letter;
-
-    a_coords_utm_to_latlon(&dem_northeast_utm, &dem_northeast);
-    a_coords_utm_to_latlon(&dem_southwest_utm, &dem_southwest);
+  if ( ! BBOX_INTERSECT(dem_bbox, vp_bbox) ) {
+    return;
   }
 
-  if ( (max_lat > dem_northeast.lat && min_lat > dem_northeast.lat) ||
-       (max_lat < dem_southwest.lat && min_lat < dem_southwest.lat) )
-    return;
-  else if ( (max_lon > dem_northeast.lon && min_lon > dem_northeast.lon) ||
-            (max_lon < dem_southwest.lon && min_lon < dem_southwest.lon) )
-    return;
-  /* else they overlap */
-
-  /**** End Overlap Check ****/
   /* boxes to show where we have DEM instead of actually drawing the DEM.
    * useful if we want to see what areas we have coverage for (if we want
    * to get elevation data for a track) but don't want to cover the map.
@@ -600,10 +579,10 @@ static void vik_dem_layer_draw_dem ( VikDEMLayer *vdl, VikViewport *vp, VikDEM *
     gdouble nscale_deg = dem->north_scale / ((gdouble) 3600);
     gdouble escale_deg = dem->east_scale / ((gdouble) 3600);
 
-    max_lat_as = max_lat * 3600;
-    min_lat_as = min_lat * 3600;
-    max_lon_as = max_lon * 3600;
-    min_lon_as = min_lon * 3600;
+    max_lat_as = vp_bbox.north * 3600;
+    min_lat_as = vp_bbox.south * 3600;
+    max_lon_as = vp_bbox.east * 3600;
+    min_lon_as = vp_bbox.west * 3600;
 
     start_lat_as = MAX(min_lat_as, dem->min_north);
     end_lat_as   = MIN(max_lat_as, dem->max_north);
@@ -632,14 +611,14 @@ static void vik_dem_layer_draw_dem ( VikDEMLayer *vdl, VikViewport *vp, VikDEM *
         // get previous and next column. catch out-of-bound.
 	gint32 new_x = x;
 	new_x -= gradient_skip_factor;
-        if(new_x < 1)
-          prevcolumn = g_ptr_array_index ( dem->columns, x+1);
+        if(new_x < 0)
+          prevcolumn = g_ptr_array_index ( dem->columns, 0);
         else
           prevcolumn = g_ptr_array_index ( dem->columns, new_x);
 	new_x = x;
 	new_x += gradient_skip_factor;
         if(new_x >= dem->n_columns)
-          nextcolumn = g_ptr_array_index ( dem->columns, x-1);
+          nextcolumn = g_ptr_array_index ( dem->columns, dem->n_columns-1);
         else
           nextcolumn = g_ptr_array_index ( dem->columns, new_x);
 
@@ -670,7 +649,8 @@ static void vik_dem_layer_draw_dem ( VikDEMLayer *vdl, VikViewport *vp, VikDEM *
 	  box_height -= box_y;
           // catch box at borders
 	  if(box_width < 0 || box_height < 0)
-		  continue; // skip this. this is out of our viewport anyway. FIXME: why?
+	    // skip this as is out of the viewport (e.g. zoomed in so this point is way off screen)
+	    continue;
 
 	  gboolean below_minimum = FALSE;
           if(vdl->type == DEM_TYPE_HEIGHT) {
@@ -684,9 +664,6 @@ static void vik_dem_layer_draw_dem ( VikDEMLayer *vdl, VikViewport *vp, VikDEM *
           }
 
           {
-	    if(box_width < 0 || box_height < 0) // FIXME: why does this happen?
-              continue;
-
             if(vdl->type == DEM_TYPE_GRADIENT) {
               if( elev == VIK_DEM_INVALID_ELEVATION ) {
                 /* don't draw it */
@@ -696,9 +673,15 @@ static void vik_dem_layer_draw_dem ( VikDEMLayer *vdl, VikViewport *vp, VikDEM *
 		gint32 new_y;
 
 		// calculate gradient from height points all around the current one
+                // Note this code suffers from edge effects, as currently prev & next columns
+                //  (and for rows when y < 0 && y > n_points) should really attempt to read values
+                //  from a different DEM file to get a change value across this particular DEM's boundary.
+                // However it's probably not worth trying to do this as accessing the right DEM is not
+                //  straight-forward and the other DEMs could have differing numbers of columns/points
+                //  and so would need to reconsider scale factors as well...
 		new_y = y - gradient_skip_factor;
 		if(new_y < 0)
-			new_y = y;
+                  new_y = 0;
 		change += get_height_difference(elev, prevcolumn->points[new_y]);
 		change += get_height_difference(elev, column->points[new_y]);
 		change += get_height_difference(elev, nextcolumn->points[new_y]);
@@ -799,7 +782,7 @@ static void vik_dem_layer_draw_dem ( VikDEMLayer *vdl, VikViewport *vp, VikDEM *
     counter.letter = dem->utm_letter;
 
     for ( x=start_x, counter.easting = start_eas; counter.easting <= end_eas; counter.easting += dem->east_scale * skip_factor, x += skip_factor ) {
-      if ( x > 0 && x < dem->n_columns ) {
+      if ( x >= 0 && x < dem->n_columns ) {
         column = g_ptr_array_index ( dem->columns, x );
         for ( y=start_y, counter.northing = start_nor; counter.northing <= end_nor; counter.northing += dem->north_scale * skip_factor, y += skip_factor ) {
           if ( y > column->n_points )
@@ -947,17 +930,18 @@ static void srtm_dem_download_thread ( DEMDownloadParams *p, gpointer threaddata
     return;
   }
 
-  gchar *src_fn = g_strdup_printf("%s%s/%c%02d%c%03d.hgt.zip",
-                SRTM_HTTP_URI,
+  gchar *src_url = g_strdup_printf("%s/%s/%c%02d%c%03d.hgt.zip",
+                base_url,
                 continent_dir,
 		(intlat >= 0) ? 'N' : 'S',
 		ABS(intlat),
 		(intlon >= 0) ? 'E' : 'W',
 		ABS(intlon) );
 
-  static DownloadMapOptions options = { FALSE, FALSE, NULL, 0, a_check_map_file, NULL, NULL };
-  DownloadResult_t result = a_http_download_get_url ( SRTM_HTTP_SITE, src_fn, p->dest, &options, NULL );
+  static DownloadFileOptions options = { FALSE, FALSE, NULL, 5, a_check_map_file, NULL, NULL };
+  DownloadResult_t result = a_http_download_get_url ( src_url, NULL, p->dest, &options, NULL );
   switch ( result ) {
+    case DOWNLOAD_PARAMETERS_ERROR:
     case DOWNLOAD_CONTENT_ERROR:
     case DOWNLOAD_HTTP_ERROR: {
       gchar *msg = g_strdup_printf ( _("DEM download failure for %f, %f"), p->lat, p->lon );
@@ -976,7 +960,7 @@ static void srtm_dem_download_thread ( DEMDownloadParams *p, gpointer threaddata
     default:
       break;
   }
-  g_free ( src_fn );
+  g_free ( src_url );
 }
 
 static gchar *srtm_lat_lon_to_dest_fn ( gdouble lat, gdouble lon )
@@ -1143,8 +1127,8 @@ static gboolean dem_layer_add_file ( VikDEMLayer *vdl, const gchar *filename )
 {
   if ( g_file_test(filename, G_FILE_TEST_EXISTS) == TRUE ) {
     /* only load if file size is not 0 (not in progress) */
-    struct stat sb;
-    stat ( filename, &sb );
+    GStatBuf sb;
+    (void)g_stat ( filename, &sb );
     if ( sb.st_size ) {
       gchar *duped_path = g_strdup(filename);
       vdl->files = g_list_prepend ( vdl->files, duped_path );
@@ -1167,16 +1151,14 @@ static void dem_download_thread ( DEMDownloadParams *p, gpointer threaddata )
   else
     return;
 
-  //gdk_threads_enter();
   g_mutex_lock ( p->mutex );
   if ( p->vdl ) {
     g_object_weak_unref ( G_OBJECT(p->vdl), weak_ref_cb, p );
 
     if ( dem_layer_add_file ( p->vdl, p->dest ) )
-      vik_layer_emit_update ( VIK_LAYER(p->vdl) ); // NB update from background thread
+      vik_layer_emit_update ( VIK_LAYER(p->vdl) ); // NB update requested from background thread
   }
   g_mutex_unlock ( p->mutex );
-  //gdk_threads_leave();
 }
 
 
@@ -1206,9 +1188,8 @@ static void dem_layer_file_info ( GtkWidget *widget, struct LatLon *ll )
 
   gchar *source = NULL;
   if ( continent_dir )
-    source = g_strdup_printf ( "http://%s%s%s/%c%02d%c%03d.hgt.zip",
-                               SRTM_HTTP_SITE,
-                               SRTM_HTTP_URI,
+    source = g_strdup_printf ( "%s/%s/%c%02d%c%03d.hgt.zip",
+                               base_url,
                                continent_dir,
                                (intlat >= 0) ? 'N' : 'S',
                                ABS(intlat),
@@ -1231,7 +1212,7 @@ static void dem_layer_file_info ( GtkWidget *widget, struct LatLon *ll )
 
   if ( g_file_test ( filename, G_FILE_TEST_EXISTS ) ) {
     // Get some timestamp information of the file
-    struct stat stat_buf;
+    GStatBuf stat_buf;
     if ( g_stat ( filename, &stat_buf ) == 0 ) {
       gchar time_buf[64];
       strftime ( time_buf, sizeof(time_buf), "%c", gmtime((const time_t *)&stat_buf.st_mtime) );
@@ -1301,13 +1282,9 @@ static gboolean dem_layer_download_release ( VikDEMLayer *vdl, GdkEventButton *e
   }
   else {
     if ( !vdl->right_click_menu ) {
-      GtkWidget *item;
       vdl->right_click_menu = GTK_MENU ( gtk_menu_new () );
-
-      item = gtk_image_menu_item_new_with_mnemonic ( _("_Show DEM File Information") );
-      gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_INFO, GTK_ICON_SIZE_MENU) );
+      GtkWidget *item = vu_menu_add_item ( vdl->right_click_menu, _("_Show DEM File Information"), GTK_STOCK_INFO, NULL, NULL );
       g_signal_connect ( G_OBJECT(item), "activate", G_CALLBACK(dem_layer_file_info), &ll );
-      gtk_menu_shell_append (GTK_MENU_SHELL(vdl->right_click_menu), item);
     }
 
     gtk_menu_popup ( vdl->right_click_menu, NULL, NULL, NULL, NULL, event->button, event->time );
