@@ -19,24 +19,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "viking.h"
-#include "vikutils.h"
-#include <glib.h>
-#include <glib/gstdio.h>
-#include <glib/gi18n.h>
-#include <string.h>
-#include <math.h>
-#include <stdlib.h>
+#include "vikgeoreflayer.h"
 #include <ctype.h>
 
-#include "ui_util.h"
 #include "preferences.h"
 #include "icons/icons.h"
+#include "vikmapslayer.h"
+
 /*
 static VikLayerParamData image_default ( void )
 {
@@ -69,18 +59,18 @@ enum {
   NUM_PARAMS };
 
 static const gchar* georef_layer_tooltip ( VikGeorefLayer *vgl );
-static void georef_layer_marshall( VikGeorefLayer *vgl, guint8 **data, gint *len );
-static VikGeorefLayer *georef_layer_unmarshall( guint8 *data, gint len, VikViewport *vvp );
-static gboolean georef_layer_set_param ( VikGeorefLayer *vgl, guint16 id, VikLayerParamData data, VikViewport *vp, gboolean is_file_operation );
+static void georef_layer_marshall( VikGeorefLayer *vgl, guint8 **data, guint *len );
+static VikGeorefLayer *georef_layer_unmarshall( guint8 *data, guint len, VikViewport *vvp );
+static gboolean georef_layer_set_param ( VikGeorefLayer *vgl, VikLayerSetParam *vlsp );
 static VikLayerParamData georef_layer_get_param ( VikGeorefLayer *vgl, guint16 id, gboolean is_file_operation );
 static VikGeorefLayer *georef_layer_new ( VikViewport *vvp );
 static VikGeorefLayer *georef_layer_create ( VikViewport *vp );
 static void georef_layer_free ( VikGeorefLayer *vgl );
-static gboolean georef_layer_properties ( VikGeorefLayer *vgl, gpointer vp );
+static gboolean georef_layer_properties ( VikGeorefLayer *vgl, gpointer vp, gboolean have_apply );
 static void georef_layer_draw ( VikGeorefLayer *vgl, VikViewport *vp );
 static void georef_layer_add_menu_items ( VikGeorefLayer *vgl, GtkMenu *menu, gpointer vlp );
 static void georef_layer_set_image ( VikGeorefLayer *vgl, const gchar *image );
-static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindow *w );
+static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindow *w, gboolean have_apply_button );
 static void georef_layer_load_image ( VikGeorefLayer *vgl, VikViewport *vp, gboolean from_file );
 
 /* tools */
@@ -92,17 +82,21 @@ static gboolean georef_layer_zoom_press ( VikGeorefLayer *vgl, GdkEventButton *e
 
 // See comment in viktrwlayer.c for advice on values used
 static VikToolInterface georef_tools[] = {
-  { { "GeorefMoveMap", "vik-icon-Georef Move Map",  N_("_Georef Move Map"), NULL,  N_("Georef Move Map"), 0 },
+  { &geomove_18_pixbuf,
+    { "GeorefMoveMap", "vik-icon-Georef Move Map",  N_("_Georef Move Map"), NULL,  N_("Georef Move Map"), 0 },
     (VikToolConstructorFunc) georef_layer_move_create, NULL, NULL, NULL,
     (VikToolMouseFunc) georef_layer_move_press, NULL, (VikToolMouseFunc) georef_layer_move_release,
     (VikToolKeyFunc) NULL,
+    (VikToolKeyFunc) NULL,
     FALSE,
     GDK_CURSOR_IS_PIXMAP, &cursor_geomove_pixbuf, NULL },
 
-  { { "GeorefZoomTool", "vik-icon-Georef Zoom Tool",  N_("Georef Z_oom Tool"), NULL,  N_("Georef Zoom Tool"), 0 },
+  { &geozoom_18_pixbuf,
+    { "GeorefZoomTool", "vik-icon-Georef Zoom Tool",  N_("Georef Z_oom Tool"), NULL,  N_("Georef Zoom Tool"), 0 },
     (VikToolConstructorFunc) georef_layer_zoom_create, NULL, NULL, NULL,
     (VikToolMouseFunc) georef_layer_zoom_press, NULL, NULL,
     (VikToolKeyFunc) NULL,
+    (VikToolKeyFunc) NULL,
     FALSE,
     GDK_CURSOR_IS_PIXMAP, &cursor_geozoom_pixbuf, NULL },
 };
@@ -167,6 +161,8 @@ VikLayerInterface vik_georef_layer_interface = {
   (VikLayerFuncSelectMove)              NULL,
   (VikLayerFuncSelectRelease)           NULL,
   (VikLayerFuncSelectedViewportMenu)    NULL,
+
+  (VikLayerFuncRefresh)                 NULL,
 };
 
 typedef struct {
@@ -246,12 +242,12 @@ static const gchar* georef_layer_tooltip ( VikGeorefLayer *vgl )
   return vgl->image;
 }
 
-static void georef_layer_marshall( VikGeorefLayer *vgl, guint8 **data, gint *len )
+static void georef_layer_marshall( VikGeorefLayer *vgl, guint8 **data, guint *len )
 {
   vik_layer_marshall_params ( VIK_LAYER(vgl), data, len );
 }
 
-static VikGeorefLayer *georef_layer_unmarshall( guint8 *data, gint len, VikViewport *vvp )
+static VikGeorefLayer *georef_layer_unmarshall( guint8 *data, guint len, VikViewport *vvp )
 {
   VikGeorefLayer *rv = georef_layer_new ( vvp );
   vik_layer_unmarshall_params ( VIK_LAYER(rv), data, len, vvp );
@@ -261,23 +257,39 @@ static VikGeorefLayer *georef_layer_unmarshall( guint8 *data, gint len, VikViewp
   return rv;
 }
 
-static gboolean georef_layer_set_param ( VikGeorefLayer *vgl, guint16 id, VikLayerParamData data, VikViewport *vp, gboolean is_file_operation )
+static gboolean georef_layer_set_param ( VikGeorefLayer *vgl, VikLayerSetParam *vlsp )
 {
-  switch ( id )
+  switch ( vlsp->id )
   {
-    case PARAM_IMAGE: georef_layer_set_image ( vgl, data.s ); break;
-    case PARAM_CN: vgl->corner.northing = data.d; break;
-    case PARAM_CE: vgl->corner.easting = data.d; break;
-    case PARAM_MN: vgl->mpp_northing = data.d; break;
-    case PARAM_ME: vgl->mpp_easting = data.d; break;
-    case PARAM_CZ: if ( data.u <= 60 ) vgl->corner.zone = data.u; break;
-    case PARAM_CL: if ( data.u >= 65 || data.u <= 90 ) vgl->corner.letter = data.u; break;
-    case PARAM_AA: if ( data.u <= 255 ) vgl->alpha = data.u; break;
+    case PARAM_IMAGE: georef_layer_set_image ( vgl, vlsp->data.s ); break;
+    case PARAM_CN: vgl->corner.northing = vlsp->data.d; break;
+    case PARAM_CE: vgl->corner.easting = vlsp->data.d; break;
+    case PARAM_MN: vgl->mpp_northing = vlsp->data.d; break;
+    case PARAM_ME: vgl->mpp_easting = vlsp->data.d; break;
+    case PARAM_CZ: if ( vlsp->data.u <= 60 ) vgl->corner.zone = vlsp->data.u; break;
+    case PARAM_CL: if ( vlsp->data.u >= 65 || vlsp->data.u <= 90 ) vgl->corner.letter = vlsp->data.u; break;
+    case PARAM_AA: if ( vlsp->data.u <= 255 ) vgl->alpha = vlsp->data.u; break;
     default: break;
   }
   return TRUE;
 }
 
+static void create_image_file ( VikGeorefLayer *vgl )
+{
+  // Create in .viking-maps
+  gchar *filename = g_strconcat ( maps_layer_default_dir(), vik_layer_get_name(VIK_LAYER(vgl)), ".jpg", NULL );
+  GError *error = NULL;
+  gdk_pixbuf_save ( vgl->pixbuf, filename, "jpeg", &error, NULL );
+  if ( error ) {
+    g_warning ( "%s", error->message );
+    g_error_free ( error );
+  }
+  else
+    vgl->image = g_strdup ( filename );
+
+  g_free ( filename );
+}
+
 static VikLayerParamData georef_layer_get_param ( VikGeorefLayer *vgl, guint16 id, gboolean is_file_operation )
 {
   VikLayerParamData rv;
@@ -286,14 +298,18 @@ static VikLayerParamData georef_layer_get_param ( VikGeorefLayer *vgl, guint16 i
     case PARAM_IMAGE: {
       gboolean set = FALSE;
       if ( is_file_operation ) {
+        if ( vgl->pixbuf && !vgl->image ) {
+          // Force creation of image file
+          create_image_file ( vgl );
+        }
         if ( a_vik_get_file_ref_format() == VIK_FILE_REF_FORMAT_RELATIVE ) {
           gchar *cwd = g_get_current_dir();
           if ( cwd ) {
             rv.s = file_GetRelativeFilename ( cwd, vgl->image );
-	    if ( !rv.s ) rv.s = "";
-            set = TRUE;
-	  }
-	}
+            if ( !rv.s ) rv.s = "";
+              set = TRUE;
+          }
+        }
       }
       if ( !set )
         rv.s = vgl->image ? vgl->image : "";
@@ -338,6 +354,54 @@ static VikGeorefLayer *georef_layer_new ( VikViewport *vvp )
   return vgl;
 }
 
+/**
+ * Return mpp for the given coords, coord mode and image size.
+ */
+static void georef_layer_mpp_from_coords ( VikCoordMode mode, struct LatLon ll_tl, struct LatLon ll_br, guint width, guint height, gdouble *xmpp, gdouble *ympp )
+{
+  struct LatLon ll_tr;
+  ll_tr.lat = ll_tl.lat;
+  ll_tr.lon = ll_br.lon;
+
+  struct LatLon ll_bl;
+  ll_bl.lat = ll_br.lat;
+  ll_bl.lon = ll_tl.lon;
+
+  gdouble diffx;
+  // UTM mode should be exact MPP
+  gdouble factor = 1.0;
+  if ( mode == VIK_COORD_LATLON ) {
+    // NB the 1.193 - is at the Equator.
+    // http://wiki.openstreetmap.org/wiki/Zoom_levels
+
+    // Convert from actual image MPP to Viking 'pixelfact'
+    gdouble mid_lat = (ll_bl.lat + ll_tr.lat ) / 2.0;
+    // Protect against div by zero (but shouldn't have 90 degrees for mid latitude...)
+    if ( fabs(mid_lat) < 89.9 )
+      factor = cos(DEG2RAD(mid_lat)) * 1.193;
+
+    // Work out the xmpp in a relationship to the centre of the coords
+    // Consider an area with a large difference in latitude, it will have differing factors
+    // Thus an average factor at the centre will have less distortion.
+    struct LatLon ll_ml;
+    ll_ml.lat = mid_lat;
+    ll_ml.lon = ll_tl.lon;
+
+    struct LatLon ll_mr;
+    ll_mr.lat = mid_lat;
+    ll_mr.lon = ll_tr.lon;
+
+    diffx = a_coords_latlon_diff ( &ll_ml, &ll_mr );
+  } else {
+    diffx = a_coords_latlon_diff ( &ll_tl, &ll_tr );
+  }
+
+  *xmpp = (diffx / width) / factor;
+
+  gdouble diffy = a_coords_latlon_diff ( &ll_tl, &ll_bl );
+  *ympp = (diffy / height) / factor;
+}
+
 static void georef_layer_draw ( VikGeorefLayer *vgl, VikViewport *vp )
 {
   if ( vgl->pixbuf )
@@ -360,6 +424,10 @@ static void georef_layer_draw ( VikGeorefLayer *vgl, VikViewport *vp )
       scale = TRUE;
       layer_width = round(vgl->width * vgl->mpp_easting / xmpp);
       layer_height = round(vgl->height * vgl->mpp_northing / ympp);
+
+      // Has the scaling worked?
+      if ( layer_width == 0 || layer_height == 0 )
+	return;
     }
 
     // If image not in viewport bounds - no need to draw it (or bother with any scaling)
@@ -394,10 +462,12 @@ static void georef_layer_draw ( VikGeorefLayer *vgl, VikViewport *vp )
 
 static void georef_layer_free ( VikGeorefLayer *vgl )
 {
-  if ( vgl->image != NULL )
+  if ( vgl->image )
     g_free ( vgl->image );
-  if ( vgl->scaled != NULL )
+  if ( vgl->scaled )
     g_object_unref ( vgl->scaled );
+  if ( vgl->pixbuf )
+    g_object_unref ( vgl->pixbuf );
 }
 
 static VikGeorefLayer *georef_layer_create ( VikViewport *vp )
@@ -405,9 +475,9 @@ static VikGeorefLayer *georef_layer_create ( VikViewport *vp )
   return georef_layer_new ( vp );
 }
 
-static gboolean georef_layer_properties ( VikGeorefLayer *vgl, gpointer vp )
+static gboolean georef_layer_properties ( VikGeorefLayer *vgl, gpointer vp, gboolean have_apply )
 {
-  return georef_layer_dialog ( vgl, vp, VIK_GTK_WINDOW_FROM_WIDGET(vp) );
+  return georef_layer_dialog ( vgl, vp, VIK_GTK_WINDOW_FROM_WIDGET(vp), have_apply );
 }
 
 static void georef_layer_load_image ( VikGeorefLayer *vgl, VikViewport *vp, gboolean from_file )
@@ -437,7 +507,7 @@ static void georef_layer_load_image ( VikGeorefLayer *vgl, VikViewport *vp, gboo
     vgl->width = gdk_pixbuf_get_width ( vgl->pixbuf );
     vgl->height = gdk_pixbuf_get_height ( vgl->pixbuf );
 
-    if ( vgl->pixbuf && vgl->alpha < 255 )
+    if ( vgl->pixbuf && vgl->alpha <= 255 )
       vgl->pixbuf = ui_pixbuf_set_alpha ( vgl->pixbuf, vgl->alpha );
   }
   /* should find length and width here too */
@@ -569,7 +639,7 @@ static void georef_layer_export_params ( gpointer *pass_along[2] )
     }
     else
     {
-      fprintf ( f, "%f\n%f\n%f\n%f\n%f\n%f\n", vgl->mpp_easting, vgl->mpp_northing, 0.0, 0.0, vgl->corner.easting, vgl->corner.northing );
+      fprintf ( f, "%f\n%f\n%f\n%f\n%f\n%f\n", vgl->mpp_easting, 0.0, 0.0, vgl->mpp_northing, vgl->corner.easting, vgl->corner.northing );
       fclose ( f );
       f = NULL;
     }
@@ -583,7 +653,7 @@ static void georef_layer_export_params ( gpointer *pass_along[2] )
  *  Based on simple file name conventions
  * Only attempted if the preference is on.
  */
-static void maybe_read_world_file ( VikFileEntry *vfe, gpointer user_data )
+static gboolean maybe_read_world_file ( VikFileEntry *vfe, gpointer user_data )
 {
   if ( a_preferences_get (VIKING_PREFERENCES_IO_NAMESPACE "georef_auto_read_world_file")->b ) {
     const gchar* filename = vik_file_entry_get_filename(VIK_FILE_ENTRY(vfe));
@@ -612,6 +682,7 @@ static void maybe_read_world_file ( VikFileEntry *vfe, gpointer user_data )
       g_free ( filew );
     }
   }
+  return TRUE;
 }
 
 static struct LatLon get_ll_tl (VikGeorefLayer *vgl)
@@ -731,19 +802,8 @@ static void calculate_mpp_from_coords ( GtkWidget *ww, VikGeorefLayer *vgl )
     struct LatLon ll_tl = get_ll_tl (vgl);
     struct LatLon ll_br = get_ll_br (vgl);
 
-    struct LatLon ll_tr;
-    ll_tr.lat = ll_tl.lat;
-    ll_tr.lon = ll_br.lon;
-
-    struct LatLon ll_bl;
-    ll_bl.lat = ll_br.lat;
-    ll_bl.lon = ll_tl.lon;
-
-    gdouble diffx = a_coords_latlon_diff ( &ll_tl, &ll_tr );
-    gdouble xmpp = diffx / width;
-
-    gdouble diffy = a_coords_latlon_diff ( &ll_tl, &ll_bl );
-    gdouble ympp = diffy / height;
+    gdouble xmpp, ympp;
+    georef_layer_mpp_from_coords ( VIK_COORD_LATLON, ll_tl, ll_br, width, height, &xmpp, &ympp );
 
     gtk_spin_button_set_value ( GTK_SPIN_BUTTON(vgl->cw.x_spin), xmpp );
     gtk_spin_button_set_value ( GTK_SPIN_BUTTON(vgl->cw.y_spin), ympp );
@@ -757,16 +817,15 @@ static void calculate_mpp_from_coords ( GtkWidget *ww, VikGeorefLayer *vgl )
 #define VIK_SETTINGS_GEOREF_TAB "georef_coordinate_tab"
 
 /* returns TRUE if OK was pressed. */
-static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindow *w )
+static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindow *w, gboolean have_apply_button )
 {
-  GtkWidget *dialog = gtk_dialog_new_with_buttons (_("Layer Properties"),
-                                                  w,
-                                                  GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                  GTK_STOCK_CANCEL,
-                                                  GTK_RESPONSE_REJECT,
-                                                  GTK_STOCK_OK,
-                                                  GTK_RESPONSE_ACCEPT,
-                                                  NULL );
+  GtkWidget *dialog = gtk_dialog_new_with_buttons (_("Layer Properties"), w,
+                                                   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                   GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,  NULL );
+  if ( have_apply_button )
+    gtk_dialog_add_button ( GTK_DIALOG(dialog), GTK_STOCK_APPLY, GTK_RESPONSE_APPLY );
+  gtk_dialog_add_button ( GTK_DIALOG(dialog), GTK_STOCK_OK, GTK_RESPONSE_ACCEPT );
+
   /* Default to reject as user really needs to specify map file first */
   gtk_dialog_set_default_response ( GTK_DIALOG(dialog), GTK_RESPONSE_REJECT );
   GtkWidget *response_w = NULL;
@@ -788,11 +847,11 @@ static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindo
   gtk_box_pack_start ( GTK_BOX(wfp_hbox), wfp_button, FALSE, FALSE, 3 );
 
   ce_label = gtk_label_new ( _("Corner pixel easting:") );
-  cw.ce_spin = gtk_spin_button_new ( (GtkAdjustment *) gtk_adjustment_new ( 4, 0.0, 1500000.0, 1, 5, 0 ), 1, 4 );
+  cw.ce_spin = gtk_spin_button_new ( (GtkAdjustment *) gtk_adjustment_new ( 4, -15000000.0, 1500000.0, 1, 5, 0 ), 1, 4 );
   gtk_widget_set_tooltip_text ( GTK_WIDGET(cw.ce_spin), _("the UTM \"easting\" value of the upper-left corner pixel of the map") );
 
   cn_label = gtk_label_new ( _("Corner pixel northing:") );
-  cw.cn_spin = gtk_spin_button_new ( (GtkAdjustment *) gtk_adjustment_new ( 4, 0.0, 9000000.0, 1, 5, 0 ), 1, 4 );
+  cw.cn_spin = gtk_spin_button_new ( (GtkAdjustment *) gtk_adjustment_new ( 4, -15000000.0, 15000000.0, 1, 5, 0 ), 1, 4 );
   gtk_widget_set_tooltip_text ( GTK_WIDGET(cw.cn_spin), _("the UTM \"northing\" value of the upper-left corner pixel of the map") );
 
   xlabel = gtk_label_new ( _("X (easting) scale (mpp): "));
@@ -909,39 +968,49 @@ static gboolean georef_layer_dialog ( VikGeorefLayer *vgl, gpointer vp, GtkWindo
       page_num = 0;
   gtk_notebook_set_current_page ( GTK_NOTEBOOK(cw.tabs), page_num );
 
-  if ( gtk_dialog_run ( GTK_DIALOG(dialog) ) == GTK_RESPONSE_ACCEPT )
-  {
-    align_coords ( vgl );
+  gboolean answer = FALSE;
+  gint resp = GTK_RESPONSE_APPLY;
+  while ( resp == GTK_RESPONSE_APPLY )  {
+    resp = gtk_dialog_run ( GTK_DIALOG(dialog) );
+    if ( resp == GTK_RESPONSE_ACCEPT || resp == GTK_RESPONSE_APPLY ) {
+      align_coords ( vgl );
 
-    vgl->corner.easting = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(cw.ce_spin) );
-    vgl->corner.northing = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(cw.cn_spin) );
-    vgl->corner.zone = gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON(cw.utm_zone_spin) );
-    const gchar *letter = gtk_entry_get_text ( GTK_ENTRY(cw.utm_letter_entry) );
-    if (*letter)
-       vgl->corner.letter = toupper(*letter);
-    vgl->mpp_easting = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(cw.x_spin) );
-    vgl->mpp_northing = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(cw.y_spin) );
-    vgl->ll_br = get_ll_br (vgl);
-    check_br_is_good_or_msg_user ( vgl );
-    if ( g_strcmp0 (vgl->image, vik_file_entry_get_filename(VIK_FILE_ENTRY(cw.imageentry)) ) != 0 )
-    {
-      georef_layer_set_image ( vgl, vik_file_entry_get_filename(VIK_FILE_ENTRY(cw.imageentry)) );
-      georef_layer_load_image ( vgl, VIK_VIEWPORT(vp), FALSE );
+      vgl->corner.easting = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(cw.ce_spin) );
+      vgl->corner.northing = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(cw.cn_spin) );
+      vgl->corner.zone = gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON(cw.utm_zone_spin) );
+      const gchar *letter = gtk_entry_get_text ( GTK_ENTRY(cw.utm_letter_entry) );
+      if (*letter)
+         vgl->corner.letter = toupper(*letter);
+      vgl->mpp_easting = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(cw.x_spin) );
+      vgl->mpp_northing = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(cw.y_spin) );
+      vgl->ll_br = get_ll_br (vgl);
+      check_br_is_good_or_msg_user ( vgl );
+      // TODO check if image has changed otherwise no need to regenerate pixbuf
+      if ( !vgl->pixbuf ) {
+        if ( g_strcmp0 (vgl->image, vik_file_entry_get_filename(VIK_FILE_ENTRY(cw.imageentry)) ) != 0 ) {
+          georef_layer_set_image ( vgl, vik_file_entry_get_filename(VIK_FILE_ENTRY(cw.imageentry)) );
+          georef_layer_load_image ( vgl, VIK_VIEWPORT(vp), FALSE );
+        }
+      }
+
+      vgl->alpha = (guint8) gtk_range_get_value ( GTK_RANGE(alpha_scale) );
+      if ( vgl->pixbuf && vgl->alpha <= 255 )
+        vgl->pixbuf = ui_pixbuf_set_alpha ( vgl->pixbuf, vgl->alpha );
+      if ( vgl->scaled && vgl->alpha <= 255 )
+        vgl->scaled = ui_pixbuf_set_alpha ( vgl->scaled, vgl->alpha );
+
+      a_settings_set_integer ( VIK_SETTINGS_GEOREF_TAB, gtk_notebook_get_current_page(GTK_NOTEBOOK(cw.tabs)) );
+
+      if ( resp == GTK_RESPONSE_APPLY ) {
+        vik_layer_emit_update ( VIK_LAYER(vgl) );
+        answer = FALSE;
+      }
+      else
+        answer = TRUE;
     }
-
-    vgl->alpha = (guint8) gtk_range_get_value ( GTK_RANGE(alpha_scale) );
-    if ( vgl->pixbuf && vgl->alpha < 255 )
-      vgl->pixbuf = ui_pixbuf_set_alpha ( vgl->pixbuf, vgl->alpha );
-    if ( vgl->scaled && vgl->alpha < 255 )
-      vgl->scaled = ui_pixbuf_set_alpha ( vgl->scaled, vgl->alpha );
-
-    a_settings_set_integer ( VIK_SETTINGS_GEOREF_TAB, gtk_notebook_get_current_page(GTK_NOTEBOOK(cw.tabs)) );
-
-    gtk_widget_destroy ( GTK_WIDGET(dialog) );
-    return TRUE;
   }
   gtk_widget_destroy ( GTK_WIDGET(dialog) );
-  return FALSE;
+  return answer;
 }
 
 static void georef_layer_zoom_to_fit ( gpointer vgl_vlp[2] )
@@ -972,32 +1041,14 @@ static void georef_layer_goto_center ( gpointer vgl_vlp[2] )
 static void georef_layer_add_menu_items ( VikGeorefLayer *vgl, GtkMenu *menu, gpointer vlp )
 {
   static gpointer pass_along[2];
-  GtkWidget *item;
   pass_along[0] = vgl;
   pass_along[1] = vlp;
 
-  item = gtk_menu_item_new();
-  gtk_menu_shell_append ( GTK_MENU_SHELL(menu), item );
-  gtk_widget_show ( item );
-
+  vu_menu_add_item ( menu, NULL, NULL, NULL, NULL ); // Just a separator
   /* Now with icons */
-  item = gtk_image_menu_item_new_with_mnemonic ( _("_Zoom to Fit Map") );
-  gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_ZOOM_FIT, GTK_ICON_SIZE_MENU) );
-  g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(georef_layer_zoom_to_fit), pass_along );
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-  gtk_widget_show ( item );
-
-  item = gtk_image_menu_item_new_with_mnemonic ( _("_Goto Map Center") );
-  gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU) );
-  g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(georef_layer_goto_center), pass_along );
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-  gtk_widget_show ( item );
-
-  item = gtk_image_menu_item_new_with_mnemonic ( _("_Export to World File") );
-  gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_HARDDISK, GTK_ICON_SIZE_MENU) );
-  g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(georef_layer_export_params), pass_along );
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-  gtk_widget_show ( item );
+  (void)vu_menu_add_item ( menu, _("_Zoom to Fit Map"), GTK_STOCK_ZOOM_FIT, G_CALLBACK(georef_layer_zoom_to_fit), pass_along );
+  (void)vu_menu_add_item ( menu, _("_Goto Map Center"), GTK_STOCK_JUMP_TO, G_CALLBACK(georef_layer_goto_center), pass_along );
+  (void)vu_menu_add_item ( menu, _("_Export to World File"), GTK_STOCK_HARDDISK, G_CALLBACK(georef_layer_export_params), pass_along );
 }
 
 
@@ -1060,3 +1111,70 @@ static gboolean georef_layer_move_press ( VikGeorefLayer *vgl, GdkEventButton *e
   vgl->click_y = event->y;
   return TRUE;
 }
+
+static void goto_center_ll ( VikViewport *vp,
+                             struct LatLon ll_tl,
+                             struct LatLon ll_br )
+{
+  VikCoord vc_center;
+  struct LatLon ll_center;
+
+  ll_center.lat = (ll_tl.lat + ll_br.lat) / 2.0;
+  ll_center.lon = (ll_tl.lon + ll_br.lon) / 2.0;
+
+  vik_coord_load_from_latlon ( &vc_center, vik_viewport_get_coord_mode (vp), &ll_center );
+  vik_viewport_set_center_coord ( vp, &vc_center, TRUE );
+}
+
+/**
+ * vik_georef_layer_create:
+ *
+ * @name:   Name for the Georef layer. This string is copied.
+ * @pixbuf: Pixbuf becomes owned by the Georef layer and will manage the freeing of it
+ */
+VikGeorefLayer *vik_georef_layer_create ( VikViewport *vp,
+                                          VikLayersPanel *vlp,
+                                          const gchar *name,
+                                          GdkPixbuf *pixbuf,
+                                          VikCoord *coord_tl,
+                                          VikCoord *coord_br )
+{
+  VikGeorefLayer *vgl = georef_layer_new ( vp );
+  vik_layer_rename ( VIK_LAYER(vgl), name );
+
+  vgl->pixbuf = pixbuf;
+
+  vik_coord_to_utm ( coord_tl, &(vgl->corner) );
+  vik_coord_to_latlon ( coord_br, &(vgl->ll_br) );
+
+  if ( vgl->pixbuf ) {
+    vgl->width = gdk_pixbuf_get_width ( vgl->pixbuf );
+    vgl->height = gdk_pixbuf_get_height ( vgl->pixbuf );
+
+    if ( vgl->width > 0 && vgl->height > 0 ) {
+
+      struct LatLon ll_tl;
+      vik_coord_to_latlon ( coord_tl, &ll_tl);
+      struct LatLon ll_br;
+      vik_coord_to_latlon ( coord_br, &ll_br);
+
+      VikCoordMode mode = vik_viewport_get_coord_mode (vp);
+
+      gdouble xmpp, ympp;
+      georef_layer_mpp_from_coords ( mode, ll_tl, ll_br, vgl->width, vgl->height, &xmpp, &ympp );
+      vgl->mpp_easting = xmpp;
+      vgl->mpp_northing = ympp;
+
+      goto_center_ll ( vp, ll_tl, ll_br);
+      // Set best zoom level
+      struct LatLon maxmin[2] = { ll_tl, ll_br };
+      vu_zoom_to_show_latlons ( vik_viewport_get_coord_mode(vp), vp, maxmin );
+
+      return vgl;
+    }
+  }
+
+  // Bad image
+  georef_layer_free ( vgl );
+  return NULL;
+}
