@@ -20,16 +20,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <glib/gi18n.h>
-
 #include "viking.h"
-#include <string.h>
-#include <stdlib.h>
 #include "viklayer_defaults.h"
 
 /* functions common to all layers. */
@@ -45,6 +36,9 @@ extern VikLayerInterface vik_dem_layer_interface;
 #ifdef HAVE_LIBMAPNIK
 extern VikLayerInterface vik_mapnik_layer_interface;
 #endif
+#ifdef HAVE_LIBMAPNIK
+extern VikLayerInterface vik_geoclue_layer_interface;
+#endif
 
 enum {
   VL_UPDATE_SIGNAL,
@@ -55,7 +49,7 @@ static guint layer_signals[VL_LAST_SIGNAL] = { 0 };
 static GObjectClass *parent_class;
 
 static void vik_layer_finalize ( VikLayer *vl );
-static gboolean vik_layer_properties_factory ( VikLayer *vl, VikViewport *vp );
+static gboolean vik_layer_properties_factory ( VikLayer *vl, VikViewport *vp, gboolean have_apply );
 static gboolean layer_defaults_register ( VikLayerTypeEnum type );
 
 G_DEFINE_TYPE (VikLayer, vik_layer, G_TYPE_OBJECT)
@@ -90,47 +84,6 @@ static gboolean idle_draw ( VikLayer *vl )
   return FALSE; // Nothing else to do
 }
 
-/**
- * Draw specified layer
- */
-void vik_layer_emit_update ( VikLayer *vl )
-{
-  if ( vl->visible && vl->realized ) {
-    GThread *thread = vik_window_get_thread ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vl)) );
-    if ( !thread )
-      // Do nothing
-      return;
-
-    vik_window_set_redraw_trigger(vl);
-
-    // Only ever draw when there is time to do so
-    if ( g_thread_self() != thread )
-      // Drawing requested from another (background) thread, so handle via the gdk thread method
-      gdk_threads_add_idle ( (GSourceFunc) idle_draw, vl );
-    else
-      g_idle_add ( (GSourceFunc) idle_draw, vl );
-  }
-}
-
-/**
- * should only be done by VikLayersPanel (hence never used from the background)
- * need to redraw and record trigger when we make a layer invisible.
- */
-void vik_layer_emit_update_although_invisible ( VikLayer *vl )
-{
-  vik_window_set_redraw_trigger(vl);
-  g_idle_add ( (GSourceFunc) idle_draw, vl );
-}
-
-/* doesn't set the trigger. should be done by aggregate layer when child emits update. */
-void vik_layer_emit_update_secondary ( VikLayer *vl )
-{
-  if ( vl->visible )
-    // TODO: this can used from the background - eg in acquire
-    //       so will need to flow background update status through too
-    g_idle_add ( (GSourceFunc) idle_draw, vl );
-}
-
 static VikLayerInterface *vik_layer_interfaces[VIK_LAYER_NUM_TYPES] = {
   &vik_aggregate_layer_interface,
   &vik_trw_layer_interface,
@@ -142,8 +95,91 @@ static VikLayerInterface *vik_layer_interfaces[VIK_LAYER_NUM_TYPES] = {
 #ifdef HAVE_LIBMAPNIK
   &vik_mapnik_layer_interface,
 #endif
+#ifdef HAVE_LIBGEOCLUE_2
+  &vik_geoclue_layer_interface,
+#endif
 };
 
+/**
+ * Draw specified layer (c.f. vik_layer_draw)
+ * The draw is ensured to be in the GUI thread,
+ *  and happens at some point in the main loop update rather than 'now'
+ */
+void vik_layer_redraw ( VikLayer *vl )
+{
+  if ( vl->visible && vl->realized ) {
+    GThread *thread = vik_window_get_thread ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vl)) );
+    if ( !thread )
+      // Do nothing
+      return;
+
+    vik_window_set_redraw_trigger(vl);
+
+    // Only ever draw when there is time to do so
+    if ( g_thread_self() != thread ) {
+      // Drawing requested from another (background) thread, so handle via the gdk thread method
+      (void)gdk_threads_add_idle ( (GSourceFunc)idle_draw, vl );
+    }
+    else {
+      (void)g_idle_add ( (GSourceFunc)idle_draw, vl );
+    }
+  }
+}
+
+/**
+ * An update event
+ * Redraw specified layer and a notification about the update
+ */
+void vik_layer_emit_update ( VikLayer *vl )
+{
+  if ( vl->visible && vl->realized ) {
+    GThread *thread = vik_window_get_thread ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(vl)) );
+    if ( !thread )
+      // Do nothing
+      return;
+
+    vik_window_set_redraw_trigger(vl);
+
+    // Notionally the 'refresh' function could be directly connected to the 'update' signal
+    // However we then have to manage the lifecycle (creation, copying, removing, etc...)
+    //  ATM IMHO it's simpler to call/use the function here as we don't need
+    //  any of the features offered by the signal framework
+    // NB This is separate from the idle_draw as that is used by other kinds of updates as well
+
+    // Only ever draw when there is time to do so
+    if ( g_thread_self() != thread ) {
+      // Drawing requested from another (background) thread, so handle via the gdk thread method
+      (void)gdk_threads_add_idle ( (GSourceFunc)idle_draw, vl );
+      if ( vik_layer_interfaces[vl->type]->refresh )
+	(void)gdk_threads_add_idle ( (GSourceFunc)vik_layer_interfaces[vl->type]->refresh, vl );
+    }
+    else {
+      (void)g_idle_add ( (GSourceFunc)idle_draw, vl );
+      if ( vik_layer_interfaces[vl->type]->refresh )
+	(void)g_idle_add ( (GSourceFunc)vik_layer_interfaces[vl->type]->refresh, vl );
+    }
+  }
+}
+
+/**
+ * should only be done by VikLayersPanel (hence never used from the background)
+ * need to redraw and record trigger when we make a layer invisible.
+ */
+void vik_layer_emit_update_although_invisible ( VikLayer *vl )
+{
+  vik_window_set_redraw_trigger(vl);
+  (void)g_idle_add ( (GSourceFunc)idle_draw, vl );
+}
+
+/* doesn't set the trigger. should be done by aggregate layer when child emits update. */
+void vik_layer_emit_update_secondary ( VikLayer *vl )
+{
+  if ( vl->visible )
+    // TODO: this can used from the background - eg in acquire
+    //       so will need to flow background update status through too
+    (void)g_idle_add ( (GSourceFunc)idle_draw, vl );
+}
+
 VikLayerInterface *vik_layer_get_interface ( VikLayerTypeEnum type )
 {
   g_assert ( type < VIK_LAYER_NUM_TYPES );
@@ -214,7 +250,7 @@ const gchar *vik_layer_get_name ( VikLayer *l )
   return l->name;
 }
 
-time_t vik_layer_get_timestamp ( VikLayer *vl )
+gdouble vik_layer_get_timestamp ( VikLayer *vl )
 {
   if ( vik_layer_interfaces[vl->type]->get_timestamp )
     return vik_layer_interfaces[vl->type]->get_timestamp ( vl );
@@ -232,7 +268,7 @@ VikLayer *vik_layer_create ( VikLayerTypeEnum type, VikViewport *vp, gboolean in
 
   if ( interactive )
   {
-    if ( vik_layer_properties ( new_layer, vp ) )
+    if ( vik_layer_properties ( new_layer, vp, FALSE ) )
       /* We translate the name here */
       /* in order to avoid translating name set by user */
       vik_layer_rename ( VIK_LAYER(new_layer), _(vik_layer_interfaces[type]->name) );
@@ -245,12 +281,18 @@ VikLayer *vik_layer_create ( VikLayerTypeEnum type, VikViewport *vp, gboolean in
   return new_layer;
 }
 
-/* returns TRUE if OK was pressed */
-gboolean vik_layer_properties ( VikLayer *layer, VikViewport *vp )
+/**
+ * @have_apply: Whether the properties dialog should have an apply button
+ *
+ * Returns:
+ *   TRUE if OK was pressed
+ *   (i.e. should redraw as the properties could have changed)
+ */
+gboolean vik_layer_properties ( VikLayer *layer, VikViewport *vp, gboolean have_apply )
 {
   if ( vik_layer_interfaces[layer->type]->properties )
-    return vik_layer_interfaces[layer->type]->properties ( layer, vp );
-  return vik_layer_properties_factory ( layer, vp );
+    return vik_layer_interfaces[layer->type]->properties ( layer, vp, have_apply );
+  return vik_layer_properties_factory ( layer, vp, have_apply );
 }
 
 void vik_layer_draw ( VikLayer *l, VikViewport *vp )
@@ -268,11 +310,11 @@ void vik_layer_change_coord_mode ( VikLayer *l, VikCoordMode mode )
 
 typedef struct {
   VikLayerTypeEnum layer_type;
-  gint len;
+  guint len;
   guint8 data[0];
 } header_t;
 
-void vik_layer_marshall ( VikLayer *vl, guint8 **data, gint *len )
+void vik_layer_marshall ( VikLayer *vl, guint8 **data, guint *len )
 {
   header_t *header;
   if ( vl && vik_layer_interfaces[vl->type]->marshall ) {
@@ -291,20 +333,23 @@ void vik_layer_marshall ( VikLayer *vl, guint8 **data, gint *len )
   }
 }
 
-void vik_layer_marshall_params ( VikLayer *vl, guint8 **data, gint *datalen )
+void vik_layer_marshall_params ( VikLayer *vl, guint8 **data, guint *datalen )
 {
   VikLayerParam *params = vik_layer_get_interface(vl->type)->params;
   VikLayerFuncGetParam get_param = vik_layer_get_interface(vl->type)->get_param;
   GByteArray* b = g_byte_array_new ();
-  gint len;
+  guint len;
 
 #define vlm_append(obj, sz) 	\
   len = (sz);    		\
   g_byte_array_append ( b, (guint8 *)&len, sizeof(len) );	\
   g_byte_array_append ( b, (guint8 *)(obj), len );
 
+  // Store the internal properties first
+  vlm_append(&vl->visible, sizeof(vl->visible));
   vlm_append(vl->name, strlen(vl->name));
 
+  // Now the actual parameters
   if ( params && get_param )
   {
     VikLayerParamData d;
@@ -330,7 +375,7 @@ void vik_layer_marshall_params ( VikLayer *vl, guint8 **data, gint *datalen )
         GList *list = d.sl;
         
         /* write length of list (# of strings) */
-        gint listlen = g_list_length ( list );
+        guint listlen = g_list_length ( list );
         g_byte_array_append ( b, (guint8 *)&listlen, sizeof(listlen) );
 
         /* write each string */
@@ -356,7 +401,7 @@ void vik_layer_marshall_params ( VikLayer *vl, guint8 **data, gint *datalen )
 #undef vlm_append
 }
 
-void vik_layer_unmarshall_params ( VikLayer *vl, guint8 *data, gint datalen, VikViewport *vvp )
+void vik_layer_unmarshall_params ( VikLayer *vl, const guint8 *data, guint datalen, VikViewport *vvp )
 {
   VikLayerParam *params = vik_layer_get_interface(vl->type)->params;
   VikLayerFuncSetParam set_param = vik_layer_get_interface(vl->type)->set_param;
@@ -365,23 +410,27 @@ void vik_layer_unmarshall_params ( VikLayer *vl, guint8 *data, gint datalen, Vik
   
 #define vlm_size (*(gint *)b)
 #define vlm_read(obj)				\
-  memcpy((obj), b+sizeof(gint), vlm_size);	\
-  b += sizeof(gint) + vlm_size;
-  
+  memcpy((obj), b+sizeof(guint), vlm_size);	\
+  b += sizeof(guint) + vlm_size;
+
+  vlm_read(&vl->visible);
+
   s = g_malloc(vlm_size + 1);
   s[vlm_size]=0;
   vlm_read(s);
-  
   vik_layer_rename(vl, s);
-  
   g_free(s);
 
   if ( params && set_param )
   {
-    VikLayerParamData d;
+    VikLayerSetParam vlsp;
+    vlsp.vp                 = vvp;
+    vlsp.is_file_operation  = FALSE;
+    vlsp.dirpath            = NULL;
     guint16 i, params_count = vik_layer_get_interface(vl->type)->params_count;
     for ( i = 0; i < params_count; i++ )
     {
+      vlsp.id = i;
       g_debug("%s: %s", __FUNCTION__, params[i].name);
       switch ( params[i].type )
       {
@@ -389,14 +438,14 @@ void vik_layer_unmarshall_params ( VikLayer *vl, guint8 *data, gint datalen, Vik
 	s = g_malloc(vlm_size + 1);
 	s[vlm_size]=0;
 	vlm_read(s);
-	d.s = s;
-	set_param(vl, i, d, vvp, FALSE);
+	vlsp.data.s = s;
+	(void)set_param(vl, &vlsp);
 	g_free(s);
 	break;
       case VIK_LAYER_PARAM_STRING_LIST:  {
-        gint listlen = vlm_size, j;
+        guint listlen = vlm_size, j;
         GList *list = NULL;
-        b += sizeof(gint); /* skip listlen */;
+        b += sizeof(guint); /* skip listlen */;
 
         for ( j = 0; j < listlen; j++ ) {
           /* get a string */
@@ -405,25 +454,24 @@ void vik_layer_unmarshall_params ( VikLayer *vl, guint8 *data, gint datalen, Vik
 	  vlm_read(s);
           list = g_list_append ( list, s );
         }
-        d.sl = list;
-        set_param(vl, i, d, vvp, FALSE);
+        vlsp.data.sl = list;
+        (void)set_param(vl, &vlsp);
         /* don't free -- string list is responsibility of the layer */
 
         break;
         }
       default:
-	vlm_read(&d);
-	set_param(vl, i, d, vvp, FALSE);
-	break;
+        vlm_read(&vlsp.data);
+        (void)set_param(vl, &vlsp);
+        break;
       }
     }
   }
 }
 
-VikLayer *vik_layer_unmarshall ( guint8 *data, gint len, VikViewport *vvp )
+VikLayer *vik_layer_unmarshall ( const guint8 *data, guint len, VikViewport *vvp )
 {
   header_t *header;
-
   header = (header_t *)data;
   
   if ( vik_layer_interfaces[header->layer_type]->unmarshall ) {
@@ -456,7 +504,7 @@ gboolean vik_layer_selected ( VikLayer *l, gint subtype, gpointer sublayer, gint
   if ( vik_layer_interfaces[l->type]->layer_selected )
     return vik_layer_interfaces[l->type]->layer_selected ( l, subtype, sublayer, type, vlp );
   /* Since no 'layer_selected' function explicitly turn off here */
-  return vik_window_clear_highlight ( (VikWindow *)VIK_GTK_WINDOW_FROM_LAYER(l) );
+  return vik_window_clear_selected ( (VikWindow *)VIK_GTK_WINDOW_FROM_LAYER(l) );
 }
 
 void vik_layer_realize ( VikLayer *l, VikTreeview *vt, GtkTreeIter *layer_iter )
@@ -525,10 +573,10 @@ GdkPixbuf *vik_layer_load_icon ( VikLayerTypeEnum type )
   return NULL;
 }
 
-gboolean vik_layer_set_param ( VikLayer *layer, guint16 id, VikLayerParamData data, gpointer vp, gboolean is_file_operation )
+gboolean vik_layer_set_param ( VikLayer *vl, VikLayerSetParam *vlsp )
 {
-  if ( vik_layer_interfaces[layer->type]->set_param )
-    return vik_layer_interfaces[layer->type]->set_param ( layer, id, data, vp, is_file_operation );
+  if ( vik_layer_interfaces[vl->type]->set_param )
+    return vik_layer_interfaces[vl->type]->set_param ( vl, vlsp );
   return FALSE;
 }
 
@@ -538,7 +586,13 @@ void vik_layer_post_read ( VikLayer *layer, VikViewport *vp, gboolean from_file
     vik_layer_interfaces[layer->type]->post_read ( layer, vp, from_file );
 }
 
-static gboolean vik_layer_properties_factory ( VikLayer *vl, VikViewport *vp )
+// Wrapper function to keep compiler happy
+static void layer_emit_update_internal ( gpointer gp )
+{
+  vik_layer_emit_update ( VIK_LAYER(gp));
+}
+
+static gboolean vik_layer_properties_factory ( VikLayer *vl, VikViewport *vp, gboolean have_apply )
 {
   switch ( a_uibuilder_properties_factory ( _("Layer Properties"),
 					    VIK_GTK_WINDOW_FROM_WIDGET(vp),
@@ -546,12 +600,16 @@ static gboolean vik_layer_properties_factory ( VikLayer *vl, VikViewport *vp )
 					    vik_layer_interfaces[vl->type]->params_count,
 					    vik_layer_interfaces[vl->type]->params_groups,
 					    vik_layer_interfaces[vl->type]->params_groups_count,
-					    (gpointer) vik_layer_interfaces[vl->type]->set_param, 
+					    (gpointer) vik_layer_interfaces[vl->type]->set_param,
+					    NULL,
 					    vl, 
 					    vp,
 					    (gpointer) vik_layer_interfaces[vl->type]->get_param, 
 					    vl,
-					    (gpointer) vik_layer_interfaces[vl->type]->change_param ) ) {
+					    (gpointer) vik_layer_interfaces[vl->type]->change_param,
+					    have_apply,
+					    layer_emit_update_internal,
+					    (gpointer)vl) ) {
     case 0:
     case 3:
       return FALSE;
@@ -654,18 +712,30 @@ void vik_layer_set_defaults ( VikLayer *vl, VikViewport *vvp )
   vl->vvp = vvp;
   VikLayerInterface *vli = vik_layer_get_interface ( vl->type );
   const gchar *layer_name = vli->fixed_layer_name;
-  VikLayerParamData data;
+  VikLayerSetParam vlsp;
 
+  vlsp.is_file_operation = TRUE; // Possibly come from a file
+  vlsp.dirpath = NULL;
+  vlsp.vp      = vvp;
   int i;
   for ( i = 0; i < vli->params_count; i++ ) {
+    vlsp.id = i;
     // Ensure parameter is for use
     if ( vli->params[i].group > VIK_LAYER_NOT_IN_PROPERTIES ) {
       // ATM can't handle string lists
       // only DEM files uses this currently
       if ( vli->params[i].type != VIK_LAYER_PARAM_STRING_LIST ) {
-        data = a_layer_defaults_get ( layer_name, vli->params[i].name, vli->params[i].type );
-        vik_layer_set_param ( vl, i, data, vvp, TRUE ); // Possibly come from a file
+        vlsp.data = a_layer_defaults_get ( layer_name, vli->params[i].name, vli->params[i].type );
+        (void)vik_layer_set_param ( vl, &vlsp );
       }
     }
   }
 }
+
+/**
+ * Make the layer more accessible within the treeview
+ */
+void vik_layer_expand_tree ( VikLayer *vl )
+{
+  vik_treeview_expand ( vl->vt, &vl->iter );
+}
