@@ -2,6 +2,7 @@
  * viking -- GPS Data and Topo Analyzer, Explorer, and Manager
  *
  * Copyright (C) 2003-2005, Evan Battaglia <gtoevan@gmx.net>
+ * Copyright (c) 2018, Rob Norris <rw_norris@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,17 +19,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "viking.h"
-#include "settings.h"
-
-#include <string.h>
-
-#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 
 enum {
@@ -37,12 +28,28 @@ enum {
   VLP_LAST_SIGNAL
 };
 
+// Calendar markup
+typedef enum {
+  VLP_CAL_MU_NONE = 0,
+  VLP_CAL_MU_MARKED,
+  VLP_CAL_MU_DETAIL,
+  VLP_CAL_MU_AUTO,
+  VLP_CAL_MU_LAST,
+} calendar_mu_t;
+
 static guint layers_panel_signals[VLP_LAST_SIGNAL] = { 0 };
 
 static GObjectClass *parent_class;
 
 struct _VikLayersPanel {
   GtkVBox vbox;
+  GtkWidget *hbox;
+  GtkWidget *calendar;
+  // Could use gtk_widget_get_visible () instead, but needs Gtk2.18
+  // ATM still using Gtk2.16 - so manually track the state for now
+  // NB Doesn't consider if the layers panel is shown or not.
+  gboolean cal_shown;
+  calendar_mu_t cal_markup;
 
   VikAggregateLayer *toplayer;
   GtkTreeIter toplayer_iter;
@@ -84,7 +91,7 @@ static void vik_layers_panel_class_init ( VikLayersPanelClass *klass )
 
   layers_panel_signals[VLP_UPDATE_SIGNAL] = g_signal_new ( "update", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (VikLayersPanelClass, update), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
   layers_panel_signals[VLP_DELETE_LAYER_SIGNAL] = g_signal_new ( "delete_layer", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (VikLayersPanelClass, update), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-}
+  }
 
 VikLayersPanel *vik_layers_panel_new ()
 {
@@ -118,51 +125,299 @@ static GtkWidget* layers_panel_create_popup ( VikLayersPanel *vlp, gboolean full
   guint ii;
 
   if ( full ) {
+    (void)vu_menu_add_item ( GTK_MENU(menu), NULL, GTK_STOCK_PROPERTIES, G_CALLBACK(vik_layers_panel_properties), vlp );
     for ( ii = 0; ii < G_N_ELEMENTS(entries); ii++ ) {
-      if ( entries[ii].stock_id ) {
-        menuitem = gtk_image_menu_item_new_with_mnemonic ( entries[ii].label );
-        gtk_image_menu_item_set_image ( (GtkImageMenuItem*)menuitem, gtk_image_new_from_stock (entries[ii].stock_id, GTK_ICON_SIZE_MENU) );
-      }
-      else
-        menuitem = gtk_menu_item_new_with_mnemonic ( entries[ii].label );
-
-      g_signal_connect_swapped ( G_OBJECT(menuitem), "activate", G_CALLBACK(entries[ii].callback), vlp );
-      gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-      gtk_widget_show ( menuitem );
+      (void)vu_menu_add_item ( GTK_MENU(menu), entries[ii].label, entries[ii].stock_id, G_CALLBACK(entries[ii].callback), vlp );
     }
   }
 
   GtkWidget *submenu = gtk_menu_new();
   menuitem = gtk_menu_item_new_with_mnemonic ( _("New Layer") );
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-  gtk_widget_show ( menuitem );
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), submenu );
 
   // Static: so memory accessible yet not continually allocated
   static gpointer lpnl[VIK_LAYER_NUM_TYPES][2];
 
   for ( ii = 0; ii < VIK_LAYER_NUM_TYPES; ii++ ) {
+
     if ( vik_layer_get_interface(ii)->icon ) {
-      menuitem = gtk_image_menu_item_new_with_mnemonic ( vik_layer_get_interface(ii)->name );
+      menuitem = gtk_image_menu_item_new_with_mnemonic ( _(vik_layer_get_interface(ii)->name) );
       gtk_image_menu_item_set_image ( (GtkImageMenuItem*)menuitem, gtk_image_new_from_pixbuf ( vik_layer_load_icon (ii) ) );
     }
     else
-      menuitem = gtk_menu_item_new_with_mnemonic ( vik_layer_get_interface(ii)->name );
+      menuitem = gtk_menu_item_new_with_mnemonic ( _(vik_layer_get_interface(ii)->name) );
 
     lpnl[ii][0] = vlp;
     lpnl[ii][1] = GINT_TO_POINTER(ii);
 
     g_signal_connect_swapped ( G_OBJECT(menuitem), "activate", G_CALLBACK(layers_panel_new_layer), lpnl[ii] );
     gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
-    gtk_widget_show ( menuitem );
   }
 
+  gtk_widget_show_all ( menu );
   return menu;
 }
 
+static void calendar_mark_layer_in_month ( VikLayersPanel *vlp, VikTrwLayer *vtl )
+{
+  guint year, month, day;
+  gtk_calendar_get_date ( GTK_CALENDAR(vlp->calendar), &year, &month, &day );
+  GDate *gd = g_date_new();
+  GHashTable *trks = vik_trw_layer_get_tracks ( vtl ); 
+  GHashTableIter iter;
+  gpointer key, value;
+  // Foreach Track
+  g_hash_table_iter_init ( &iter, trks );
+  while ( g_hash_table_iter_next ( &iter, &key, &value ) ) {
+    VikTrack *trk = VIK_TRACK(value);
+    // First trackpoint
+    if ( trk->trackpoints && !isnan(VIK_TRACKPOINT(trk->trackpoints->data)->timestamp) ) {
+      // Not worried about subsecond resolution here!
+      g_date_set_time_t ( gd, (time_t)VIK_TRACKPOINT(trk->trackpoints->data)->timestamp );
+      // Is in selected month?
+      if ( g_date_get_year(gd) == year && (g_date_get_month(gd) == (month+1)) ) {
+	gtk_calendar_mark_day ( GTK_CALENDAR(vlp->calendar), g_date_get_day(gd) );
+	break;
+      }
+    }
+  }
+  g_date_free ( gd );
+}
+
+/**
+ *
+ */
+void layers_panel_calendar_update ( VikLayersPanel *vlp )
+{
+  // Skip if not shown...
+  if ( !vlp->cal_shown )
+    return;
+
+  gtk_calendar_clear_marks ( GTK_CALENDAR(vlp->calendar) );
+
+  GList *layers = vik_layers_panel_get_all_layers_of_type ( vlp, VIK_LAYER_TRW, TRUE );
+  if ( !layers )
+    return;
+  
+  while ( layers ) {
+    VikTrwLayer *vtl = VIK_TRW_LAYER(layers->data);
+    calendar_mark_layer_in_month ( vlp, vtl );
+    layers = g_list_next ( layers );
+  }
+  g_list_free ( layers );    
+}
+
+/**
+ *
+ */
+void vik_layers_panel_calendar_update ( VikLayersPanel *vlp )
+{
+  if ( vlp->cal_markup == VLP_CAL_MU_NONE )
+    return;
+
+  clock_t begin = clock();
+  layers_panel_calendar_update ( vlp );
+  clock_t end = clock();
+  double time_spent = (double)(end - begin) / CLOCKS_PER_SEC;
+  g_debug ( "%s: %f", __FUNCTION__, time_spent );
+  // Downgrade automatic detail if taking too long
+  // Since ATM the detail method invokes x30 searches,
+  //  it could get noticably slow
+  if ( vlp->cal_markup == VLP_CAL_MU_AUTO )
+    if ( time_spent > (0.5/30.0) )
+      vlp->cal_markup = VLP_CAL_MU_MARKED;
+}
+
+static void layers_calendar_day_selected_dc_cb ( VikLayersPanel *vlp )
+{
+  // Ideally only need to search if the day is marked...
+  guint year, month, day;
+  gtk_calendar_get_date ( GTK_CALENDAR(vlp->calendar), &year, &month, &day );
+  gchar *date_str = g_strdup_printf ( "%d-%02d-%02d", year, month+1, day );
+  (void)vik_aggregate_layer_search_date ( vlp->toplayer, date_str );
+  g_free ( date_str );
+}
+
+/**
+ *
+ */
+gboolean layers_calendar_forward ( VikLayersPanel *vlp )
+{
+  guint year, month, day;
+  gtk_calendar_get_date ( GTK_CALENDAR(vlp->calendar), &year, &month, &day );
+  // Limit the search into the future
+  time_t now = time (NULL);
+  if (now == (time_t) -1) {
+    g_critical ( "%s: current time unavailable", __FUNCTION__ );
+    return TRUE;
+  }
+  GDate *gd_now = g_date_new ();
+  g_date_set_time_t ( gd_now, now );
+
+  GDate *gd = g_date_new_dmy ( day, month+1, year );
+  gboolean found = FALSE;
+  gchar *date_str = NULL;
+  while ( !found && (g_date_compare(gd, gd_now) < 0) ) {
+    // Free previous allocation
+    g_free ( date_str );
+    // Search on the next day
+    g_date_add_days ( gd, 1 );
+    year = g_date_get_year ( gd );
+    month = g_date_get_month ( gd );
+    day = g_date_get_day ( gd );
+    date_str = g_strdup_printf ( "%d-%02d-%02d", year, month, day );
+    found = vik_aggregate_layer_search_date ( vlp->toplayer, date_str );
+  }
+  if ( found ) {
+    gtk_calendar_select_day ( GTK_CALENDAR(vlp->calendar), day );
+    gtk_calendar_select_month ( GTK_CALENDAR(vlp->calendar), month-1, year );
+  } else {
+    a_dialog_info_msg_extra ( GTK_WINDOW(VIK_WINDOW_FROM_WIDGET(vlp)), _("Nothing found up to %s"), date_str );
+  }
+  g_debug ( "%s: %s %d", __FUNCTION__, date_str, found );
+  g_free ( date_str );
+  g_date_free ( gd_now );
+  g_date_free ( gd );
+  return TRUE;
+}
+
+/**
+ *
+ */
+gboolean layers_calendar_back ( VikLayersPanel *vlp )
+{
+  guint year, month, day;
+  gtk_calendar_get_date ( GTK_CALENDAR(vlp->calendar), &year, &month, &day );
+
+  GDate *gd = g_date_new_dmy ( day, month+1, year );
+  gboolean found = FALSE;
+  const guint limit = 3650; // Search up to 10 years ago from selected date
+  guint ii = 0;
+  gchar *date_str = NULL;
+  while ( !found && ii < limit ) {
+    // Free previous allocation
+    g_free ( date_str );
+    // Search on the previous day
+    g_date_subtract_days ( gd, 1 );
+    year = g_date_get_year ( gd );
+    month = g_date_get_month ( gd );
+    day = g_date_get_day ( gd );
+    date_str = g_strdup_printf ( "%d-%02d-%02d", year, month, day );
+    found = vik_aggregate_layer_search_date ( vlp->toplayer, date_str );
+    ii++;
+  }
+  if ( found ) {
+    gtk_calendar_select_day ( GTK_CALENDAR(vlp->calendar), day );
+    gtk_calendar_select_month ( GTK_CALENDAR(vlp->calendar), month-1, year );
+  } else {
+    a_dialog_info_msg_extra ( GTK_WINDOW(VIK_WINDOW_FROM_WIDGET(vlp)), _("Nothing found back to %s"), date_str );
+  }
+  g_debug ( "%s: %s %d", __FUNCTION__, date_str, found );
+  g_free ( date_str );
+  g_date_free ( gd );
+  return TRUE;
+}
+
+/**
+ *
+ */
+gboolean layers_calendar_today ( VikLayersPanel *vlp )
+{
+  vu_calendar_set_to_today ( vlp->calendar );
+  return TRUE;
+}
+
+/**
+ * Enable a right click menu on the calendar
+ */
+static gboolean layers_calendar_button_press_cb ( VikLayersPanel *vlp, GdkEventButton *event )
+{
+  if ( event->button == 3 ) {
+    GtkMenu *menu = GTK_MENU ( gtk_menu_new () );
+    (void)vu_menu_add_item ( menu, NULL, GTK_STOCK_GO_BACK, G_CALLBACK(layers_calendar_back), vlp );
+    (void)vu_menu_add_item ( menu, NULL, GTK_STOCK_GO_FORWARD, G_CALLBACK(layers_calendar_forward), vlp );
+    (void)vu_menu_add_item ( menu, _("_Today"), GTK_STOCK_HOME, G_CALLBACK(layers_calendar_today), vlp );
+    gtk_widget_show_all ( GTK_WIDGET(menu) );
+    gtk_menu_popup ( menu, NULL, NULL, NULL, NULL, event->button, gtk_get_current_event_time() );
+    return TRUE;
+  }
+  return FALSE;
+}
+
+static gchar *calendar_detail ( GtkCalendar *calendar,
+                                guint year,
+                                guint month,
+                                guint day,
+                                gpointer user_data )
+{
+  VikLayersPanel *vlp = (VikLayersPanel*)user_data;
+  // Skip if not shown...
+  if ( !vlp->cal_shown )
+    return NULL;
+
+  // Check that detail is wanted
+  if ( vlp->cal_markup < VLP_CAL_MU_DETAIL )
+    return NULL;
+
+  guint ayear, amonth, aday;
+  gtk_calendar_get_date ( GTK_CALENDAR(vlp->calendar), &ayear, &amonth, &aday );
+
+  // Only bother for this actual month
+  // As this event gets fired for the surrounding days too
+  if ( amonth != month )
+    return NULL;
+
+  GList *layers = vik_layers_panel_get_all_layers_of_type ( vlp, VIK_LAYER_TRW, TRUE );
+  if ( !layers )
+    return NULL;
+
+  gboolean need_to_break = FALSE;
+  VikTrwLayer *vtl = NULL;
+  GDate *gd = g_date_new();
+  while ( layers ) {
+    vtl = VIK_TRW_LAYER(layers->data);
+
+    GHashTable *trks = vik_trw_layer_get_tracks ( vtl ); 
+    GHashTableIter iter;
+    gpointer key, value;
+    // Foreach Track
+    g_hash_table_iter_init ( &iter, trks );
+    while ( g_hash_table_iter_next ( &iter, &key, &value ) ) {
+      VikTrack *trk = VIK_TRACK(value);
+      // First trackpoint
+      if ( trk->trackpoints && !isnan(VIK_TRACKPOINT(trk->trackpoints->data)->timestamp) ) {
+        // Not worried about subsecond resolution here!
+        g_date_set_time_t ( gd, (time_t)VIK_TRACKPOINT(trk->trackpoints->data)->timestamp );
+        // Is of this day?
+        if ( g_date_get_year(gd) == year &&
+             g_date_get_month(gd) == (month+1) &&
+             g_date_get_day(gd) == day ) {
+          need_to_break = TRUE;
+	  break;
+	}
+      }
+    }
+    // Fully exit
+    if ( need_to_break )
+      break;
+    
+    layers = g_list_next ( layers );
+    vtl = NULL;
+  }
+  g_date_free ( gd );
+  g_list_free ( layers );    
+
+  if ( vtl)
+    return g_strdup (vik_layer_get_name ( VIK_LAYER(vtl) ));
+
+  return NULL;
+}
+
+#define VIK_SETTINGS_CAL_MUM "layers_panel_calendar_markup_mode"
+
 static void vik_layers_panel_init ( VikLayersPanel *vlp )
 {
-  GtkWidget *hbox;
   GtkWidget *addbutton, *addimage;
   GtkWidget *removebutton, *removeimage;
   GtkWidget *upbutton, *upimage;
@@ -174,7 +429,7 @@ static void vik_layers_panel_init ( VikLayersPanel *vlp )
 
   vlp->vvp = NULL;
 
-  hbox = gtk_hbox_new ( TRUE, 2 );
+  vlp->hbox = gtk_hbox_new ( TRUE, 2 );
   vlp->vt = vik_treeview_new ( );
 
   vlp->toplayer = vik_aggregate_layer_new ();
@@ -195,57 +450,86 @@ static void vik_layers_panel_init ( VikLayersPanel *vlp )
   addbutton = gtk_button_new ( );
   gtk_container_add ( GTK_CONTAINER(addbutton), addimage );
   gtk_widget_set_tooltip_text ( GTK_WIDGET(addbutton), _("Add new layer"));
-  gtk_box_pack_start ( GTK_BOX(hbox), addbutton, TRUE, TRUE, 0 );
+  gtk_box_pack_start ( GTK_BOX(vlp->hbox), addbutton, TRUE, TRUE, 0 );
   g_signal_connect_swapped ( G_OBJECT(addbutton), "clicked", G_CALLBACK(layers_popup_cb), vlp );
   /* Remove button */
   removeimage = gtk_image_new_from_stock ( GTK_STOCK_REMOVE, GTK_ICON_SIZE_SMALL_TOOLBAR );
   removebutton = gtk_button_new ( );
   gtk_container_add ( GTK_CONTAINER(removebutton), removeimage );
   gtk_widget_set_tooltip_text ( GTK_WIDGET(removebutton), _("Remove selected layer"));
-  gtk_box_pack_start ( GTK_BOX(hbox), removebutton, TRUE, TRUE, 0 );
+  gtk_box_pack_start ( GTK_BOX(vlp->hbox), removebutton, TRUE, TRUE, 0 );
   g_signal_connect_swapped ( G_OBJECT(removebutton), "clicked", G_CALLBACK(vik_layers_panel_delete_selected), vlp );
   /* Up button */
   upimage = gtk_image_new_from_stock ( GTK_STOCK_GO_UP, GTK_ICON_SIZE_SMALL_TOOLBAR );
   upbutton = gtk_button_new ( );
   gtk_container_add ( GTK_CONTAINER(upbutton), upimage );
   gtk_widget_set_tooltip_text ( GTK_WIDGET(upbutton), _("Move selected layer up"));
-  gtk_box_pack_start ( GTK_BOX(hbox), upbutton, TRUE, TRUE, 0 );
+  gtk_box_pack_start ( GTK_BOX(vlp->hbox), upbutton, TRUE, TRUE, 0 );
   g_signal_connect_swapped ( G_OBJECT(upbutton), "clicked", G_CALLBACK(layers_move_item_up), vlp );
   /* Down button */
   downimage = gtk_image_new_from_stock ( GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_SMALL_TOOLBAR );
   downbutton = gtk_button_new ( );
   gtk_container_add ( GTK_CONTAINER(downbutton), downimage );
   gtk_widget_set_tooltip_text ( GTK_WIDGET(downbutton), _("Move selected layer down"));
-  gtk_box_pack_start ( GTK_BOX(hbox), downbutton, TRUE, TRUE, 0 );
+  gtk_box_pack_start ( GTK_BOX(vlp->hbox), downbutton, TRUE, TRUE, 0 );
   g_signal_connect_swapped ( G_OBJECT(downbutton), "clicked", G_CALLBACK(layers_move_item_down), vlp );
   /* Cut button */
   cutimage = gtk_image_new_from_stock ( GTK_STOCK_CUT, GTK_ICON_SIZE_SMALL_TOOLBAR );
   cutbutton = gtk_button_new ( );
   gtk_container_add ( GTK_CONTAINER(cutbutton), cutimage );
   gtk_widget_set_tooltip_text ( GTK_WIDGET(cutbutton), _("Cut selected layer"));
-  gtk_box_pack_start ( GTK_BOX(hbox), cutbutton, TRUE, TRUE, 0 );
+  gtk_box_pack_start ( GTK_BOX(vlp->hbox), cutbutton, TRUE, TRUE, 0 );
   g_signal_connect_swapped ( G_OBJECT(cutbutton), "clicked", G_CALLBACK(vik_layers_panel_cut_selected), vlp );
   /* Copy button */
   copyimage = gtk_image_new_from_stock ( GTK_STOCK_COPY, GTK_ICON_SIZE_SMALL_TOOLBAR );
   copybutton = gtk_button_new ( );
   gtk_container_add ( GTK_CONTAINER(copybutton), copyimage );
   gtk_widget_set_tooltip_text ( GTK_WIDGET(copybutton), _("Copy selected layer"));
-  gtk_box_pack_start ( GTK_BOX(hbox), copybutton, TRUE, TRUE, 0 );
+  gtk_box_pack_start ( GTK_BOX(vlp->hbox), copybutton, TRUE, TRUE, 0 );
   g_signal_connect_swapped ( G_OBJECT(copybutton), "clicked", G_CALLBACK(vik_layers_panel_copy_selected), vlp );
   /* Paste button */
   pasteimage = gtk_image_new_from_stock ( GTK_STOCK_PASTE, GTK_ICON_SIZE_SMALL_TOOLBAR );
   pastebutton = gtk_button_new ( );
   gtk_container_add ( GTK_CONTAINER(pastebutton),pasteimage );
   gtk_widget_set_tooltip_text ( GTK_WIDGET(pastebutton), _("Paste layer into selected container layer or otherwise above selected layer"));
-  gtk_box_pack_start ( GTK_BOX(hbox), pastebutton, TRUE, TRUE, 0 );
+  gtk_box_pack_start ( GTK_BOX(vlp->hbox), pastebutton, TRUE, TRUE, 0 );
   g_signal_connect_swapped ( G_OBJECT(pastebutton), "clicked", G_CALLBACK(vik_layers_panel_paste_selected), vlp );
 
   scrolledwindow = gtk_scrolled_window_new ( NULL, NULL );
   gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW(scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC );
   gtk_container_add ( GTK_CONTAINER(scrolledwindow), GTK_WIDGET(vlp->vt) );
+
+  int cal_markup;
+  if ( a_settings_get_integer ( VIK_SETTINGS_CAL_MUM, &cal_markup ) ) {
+    if ( cal_markup >= VLP_CAL_MU_NONE && cal_markup < VLP_CAL_MU_LAST )
+      vlp->cal_markup = cal_markup;
+    else
+      vlp->cal_markup = VLP_CAL_MU_AUTO;
+  }
+  else
+    vlp->cal_markup = VLP_CAL_MU_AUTO;
+
+  vlp->calendar = gtk_calendar_new();
+  g_signal_connect_swapped ( vlp->calendar, "month-changed", G_CALLBACK(vik_layers_panel_calendar_update), vlp);
+  // Use double click event, rather than just the day-selected event,
+  //  since changing month/year causes the day-selected event to occur
+  // (and then possibly this results in selecting a track) which would be counter intuitive
+  g_signal_connect_swapped ( vlp->calendar, "day-selected-double-click", G_CALLBACK(layers_calendar_day_selected_dc_cb), vlp);
+  g_signal_connect_swapped ( vlp->calendar, "button-press-event", G_CALLBACK(layers_calendar_button_press_cb), vlp );
+  vlp->cal_shown = TRUE;
+
+  // Ensure any detail results in a tooltip rather than embedded in the calendar display
+  GValue sd = { 0 };
+  g_value_init ( &sd, G_TYPE_BOOLEAN );
+  g_value_set_boolean ( &sd, FALSE );
+  g_object_set_property ( G_OBJECT(vlp->calendar), "show-details", &sd );
+
+  gtk_calendar_set_detail_func ( GTK_CALENDAR(vlp->calendar), calendar_detail, vlp, NULL );
+  vik_layers_panel_set_preferences ( vlp );
   
   gtk_box_pack_start ( GTK_BOX(vlp), scrolledwindow, TRUE, TRUE, 0 );
-  gtk_box_pack_start ( GTK_BOX(vlp), hbox, FALSE, FALSE, 0 );
+  gtk_box_pack_start ( GTK_BOX(vlp), vlp->calendar, FALSE, FALSE, 0 );
+  gtk_box_pack_start ( GTK_BOX(vlp), vlp->hbox, FALSE, FALSE, 0 );
 }
 
 /**
@@ -267,9 +551,9 @@ void vik_layers_panel_emit_update ( VikLayersPanel *vlp )
   // Only ever draw when there is time to do so
   if ( g_thread_self() != thread )
     // Drawing requested from another (background) thread, so handle via the gdk thread method
-    gdk_threads_add_idle ( (GSourceFunc) idle_draw_panel, vlp );
+    (void)gdk_threads_add_idle ( (GSourceFunc)idle_draw_panel, vlp );
   else
-    g_idle_add ( (GSourceFunc) idle_draw_panel, vlp );
+    (void)g_idle_add ( (GSourceFunc)idle_draw_panel, vlp );
 }
 
 static void layers_item_toggled (VikLayersPanel *vlp, GtkTreeIter *iter)
@@ -287,12 +571,15 @@ static void layers_item_toggled (VikLayersPanel *vlp, GtkTreeIter *iter)
     case VIK_TREEVIEW_TYPE_LAYER:
       visible = (VIK_LAYER(p)->visible ^= 1);
       vik_layer_emit_update_although_invisible ( VIK_LAYER(p) ); /* set trigger for half-drawn */
+      if ( !visible && VIK_LAYER(p)->type == VIK_LAYER_TRW )
+        vik_window_close_graphs ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(VIK_LAYER(p))) );
       break;
-    case VIK_TREEVIEW_TYPE_SUBLAYER:
-      visible = vik_layer_sublayer_toggle_visible ( VIK_LAYER(vik_treeview_item_get_parent(vlp->vt, iter)),
-						vik_treeview_item_get_data(vlp->vt, iter), p);
-      vik_layer_emit_update_although_invisible ( VIK_LAYER(vik_treeview_item_get_parent(vlp->vt, iter)) );
+    case VIK_TREEVIEW_TYPE_SUBLAYER: {
+      VikLayer *vl = VIK_LAYER(vik_treeview_item_get_parent ( vlp->vt, iter ));
+      visible = vik_layer_sublayer_toggle_visible ( vl, vik_treeview_item_get_data(vlp->vt, iter), p);
+      vik_layer_emit_update_although_invisible ( vl );
       break;
+    }
     default: return;
   }
 
@@ -361,7 +648,6 @@ static void layers_popup ( VikLayersPanel *vlp, GtkTreeIter *iter, gint mouse_bu
 {
   GtkMenu *menu = NULL;
 
-
   if ( iter )
   {
     if ( vik_treeview_item_get_type ( vlp->vt, iter ) == VIK_TREEVIEW_TYPE_LAYER )
@@ -372,47 +658,32 @@ static void layers_popup ( VikLayersPanel *vlp, GtkTreeIter *iter, gint mouse_bu
         menu = GTK_MENU ( layers_panel_create_popup ( vlp, TRUE ) );
       else
       {
-        GtkWidget *del, *prop;
 	VikStdLayerMenuItem menu_selection = vik_layer_get_menu_items_selection(layer);
 
         menu = GTK_MENU ( gtk_menu_new () );
 
 	if (menu_selection & VIK_MENU_ITEM_PROPERTY) {
-	  prop = gtk_image_menu_item_new_from_stock ( GTK_STOCK_PROPERTIES, NULL );
-	  g_signal_connect_swapped ( G_OBJECT(prop), "activate", G_CALLBACK(vik_layers_panel_properties), vlp );
-	  gtk_menu_shell_append (GTK_MENU_SHELL (menu), prop);
-	  gtk_widget_show ( prop );
+          (void)vu_menu_add_item ( menu, NULL, GTK_STOCK_PROPERTIES, G_CALLBACK(vik_layers_panel_properties), vlp );
 	}
 
 	if (menu_selection & VIK_MENU_ITEM_CUT) {
-	  del = gtk_image_menu_item_new_from_stock ( GTK_STOCK_CUT, NULL );
-	  g_signal_connect_swapped ( G_OBJECT(del), "activate", G_CALLBACK(vik_layers_panel_cut_selected), vlp );
-	  gtk_menu_shell_append (GTK_MENU_SHELL (menu), del);
-	  gtk_widget_show ( del );
+          (void)vu_menu_add_item ( menu, NULL, GTK_STOCK_CUT, G_CALLBACK(vik_layers_panel_cut_selected), vlp );
 	}
 
 	if (menu_selection & VIK_MENU_ITEM_COPY) {
-	  del = gtk_image_menu_item_new_from_stock ( GTK_STOCK_COPY, NULL );
-	  g_signal_connect_swapped ( G_OBJECT(del), "activate", G_CALLBACK(vik_layers_panel_copy_selected), vlp );
-	  gtk_menu_shell_append (GTK_MENU_SHELL (menu), del);
-	  gtk_widget_show ( del );
+          (void)vu_menu_add_item ( menu, NULL, GTK_STOCK_COPY, G_CALLBACK(vik_layers_panel_copy_selected), vlp );
 	}
 
 	if (menu_selection & VIK_MENU_ITEM_PASTE) {
-	  del = gtk_image_menu_item_new_from_stock ( GTK_STOCK_PASTE, NULL );
-	  g_signal_connect_swapped ( G_OBJECT(del), "activate", G_CALLBACK(vik_layers_panel_paste_selected), vlp );
-	  gtk_menu_shell_append (GTK_MENU_SHELL (menu), del);
-	  gtk_widget_show ( del );
+          (void)vu_menu_add_item ( menu, NULL, GTK_STOCK_PASTE, G_CALLBACK(vik_layers_panel_paste_selected), vlp );
 	}
 
 	if (menu_selection & VIK_MENU_ITEM_DELETE) {
-	  del = gtk_image_menu_item_new_from_stock ( GTK_STOCK_DELETE, NULL );
-	  g_signal_connect_swapped ( G_OBJECT(del), "activate", G_CALLBACK(vik_layers_panel_delete_selected), vlp );
-	  gtk_menu_shell_append (GTK_MENU_SHELL (menu), del);
-	  gtk_widget_show ( del );
+          (void)vu_menu_add_item ( menu, NULL, GTK_STOCK_DELETE, G_CALLBACK(vik_layers_panel_delete_selected), vlp );
 	}
       }
       vik_layer_add_menu_items ( layer, menu, vlp );
+      gtk_widget_show_all ( GTK_WIDGET(menu) );
     }
     else
     {
@@ -456,7 +727,7 @@ gboolean vik_layers_panel_new_layer ( VikLayersPanel *vlp, VikLayerTypeEnum type
   g_assert ( vlp->vvp );
   gboolean ask_user = FALSE;
   if ( type == VIK_LAYER_TRW )
-    a_settings_get_boolean ( VIK_SETTINGS_LAYERS_TRW_CREATE_DEFAULT, &ask_user );
+    (void)a_settings_get_boolean ( VIK_SETTINGS_LAYERS_TRW_CREATE_DEFAULT, &ask_user );
   ask_user = !ask_user;
   l = vik_layer_create ( type, vlp->vvp, ask_user );
   if ( l )
@@ -553,10 +824,8 @@ gboolean vik_layers_panel_properties ( VikLayersPanel *vlp )
 
   if ( vik_treeview_get_selected_iter ( vlp->vt, &iter ) && vik_treeview_item_get_type ( vlp->vt, &iter ) == VIK_TREEVIEW_TYPE_LAYER )
   {
-    if ( vik_treeview_item_get_data ( vlp->vt, &iter ) == VIK_LAYER_AGGREGATE )
-      a_dialog_info_msg ( VIK_GTK_WINDOW_FROM_WIDGET(vlp), _("Aggregate Layers have no settable properties.") );
     VikLayer *layer = VIK_LAYER( vik_treeview_item_get_pointer ( vlp->vt, &iter ) );
-    if (vik_layer_properties ( layer, vlp->vvp ))
+    if ( vik_layer_properties ( layer, vlp->vvp, TRUE ))
       vik_layer_emit_update ( layer );
     return TRUE;
   }
@@ -662,7 +931,7 @@ void vik_layers_panel_delete_selected ( VikLayersPanel *vlp )
         g_signal_emit ( G_OBJECT(vlp), layers_panel_signals[VLP_DELETE_LAYER_SIGNAL], 0 );
 
         if ( vik_aggregate_layer_delete ( parent, &iter ) )
-	  vik_layers_panel_emit_update ( vlp );
+	  vik_layers_panel_emit_update ( vlp );	
       }
     }
     else
@@ -675,6 +944,8 @@ void vik_layers_panel_delete_selected ( VikLayersPanel *vlp )
       vik_layer_get_interface(sel->type)->delete_item ( sel, subtype, vik_treeview_item_get_pointer(sel->vt, &iter) );
     }
   }
+  // Always attempt to update the calendar on any delete (even if not actually deleted anything)
+  vik_layers_panel_calendar_update ( vlp );
 }
 
 VikLayer *vik_layers_panel_get_selected ( VikLayersPanel *vlp )
@@ -748,11 +1019,15 @@ VikAggregateLayer *vik_layers_panel_get_top_layer ( VikLayersPanel *vlp )
   return vlp->toplayer;
 }
 
+/**
+ * Remove all layers
+ */
 void vik_layers_panel_clear ( VikLayersPanel *vlp )
 {
-  if ( (! vik_aggregate_layer_is_empty(vlp->toplayer)) && a_dialog_yes_or_no ( VIK_GTK_WINDOW_FROM_WIDGET(vlp), _("Are you sure you wish to delete all layers?"), NULL ) ) {
+  if ( ! vik_aggregate_layer_is_empty(vlp->toplayer) ) {
     g_signal_emit ( G_OBJECT(vlp), layers_panel_signals[VLP_DELETE_LAYER_SIGNAL], 0 );
     vik_aggregate_layer_clear ( vlp->toplayer ); /* simply deletes all layers */
+    gtk_calendar_clear_marks ( GTK_CALENDAR(vlp->calendar) );
   }
 }
 
@@ -772,3 +1047,54 @@ VikTreeview *vik_layers_panel_get_treeview ( VikLayersPanel *vlp )
 {
   return vlp->vt;
 }
+
+void vik_layers_panel_show_buttons ( VikLayersPanel *vlp, gboolean show )
+{
+  if ( show )
+    gtk_widget_show ( vlp->hbox );
+  else
+    gtk_widget_hide ( vlp->hbox );
+}
+
+void vik_layers_panel_show_calendar ( VikLayersPanel *vlp, gboolean show )
+{
+  if ( show ) {
+    vlp->cal_shown = TRUE;
+    vik_layers_panel_calendar_update ( vlp );
+    gtk_widget_show ( vlp->calendar );
+  }
+  else {
+    vlp->cal_shown = FALSE;
+    gtk_widget_hide ( vlp->calendar );
+  }
+}
+
+/**
+ *
+ */
+void vik_layers_panel_calendar_today ( VikLayersPanel *vlp )
+{
+  vu_calendar_set_to_today (vlp->calendar);
+}
+
+void vik_layers_panel_calendar_date ( VikLayersPanel *vlp, time_t timestamp )
+{
+  GDate *gd = g_date_new();
+  g_date_set_time_t ( gd, timestamp );
+  gtk_calendar_select_month ( GTK_CALENDAR(vlp->calendar), g_date_get_month(gd)-1, g_date_get_year(gd) );
+  gtk_calendar_select_day ( GTK_CALENDAR(vlp->calendar), g_date_get_day(gd) );
+  g_date_free ( gd );
+}
+
+/**
+ * vik_layers_panel_set_preferences:
+ *
+ * Allow reapplying preferences (i.e from the preferences dialog)
+ */
+void vik_layers_panel_set_preferences ( VikLayersPanel *vlp )
+{
+  GValue sd = { 0 };
+  g_value_init ( &sd, G_TYPE_BOOLEAN );
+  g_value_set_boolean ( &sd, a_vik_get_calendar_show_day_names() );
+  g_object_set_property ( G_OBJECT(vlp->calendar), "show-day-names", &sd );
+}
