@@ -18,19 +18,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "viking.h"
-#include "vikutils.h"
-#include <glib.h>
-#include <glib/gstdio.h>
-#include <glib/gi18n.h>
-#include <string.h>
-#include <math.h>
-#include <stdlib.h>
+#include "vikmapniklayer.h"
 #include <ctype.h>
 
 #include "map_ids.h"
@@ -38,8 +27,6 @@
 #include "mapcoord.h"
 #include "mapcache.h"
 #include "dir.h"
-#include "util.h"
-#include "ui_util.h"
 #include "preferences.h"
 #include "icons/icons.h"
 #include "mapnik_interface.h"
@@ -47,10 +34,6 @@
 
 #include "vikmapslayer.h"
 
-#if !GLIB_CHECK_VERSION(2,26,0)
-typedef struct stat GStatBuf;
-#endif
-
 struct _VikMapnikLayerClass
 {
 	VikLayerClass object_class;
@@ -101,8 +84,8 @@ enum {
   NUM_PARAMS };
 
 static const gchar* mapnik_layer_tooltip ( VikMapnikLayer *vml );
-static void mapnik_layer_marshall( VikMapnikLayer *vml, guint8 **data, gint *len );
-static VikMapnikLayer *mapnik_layer_unmarshall( guint8 *data, gint len, VikViewport *vvp );
+static void mapnik_layer_marshall( VikMapnikLayer *vml, guint8 **data, guint *len );
+static VikMapnikLayer *mapnik_layer_unmarshall( guint8 *data, guint len, VikViewport *vvp );
 static gboolean mapnik_layer_set_param ( VikMapnikLayer *vml, guint16 id, VikLayerParamData data, VikViewport *vp, gboolean is_file_operation );
 static VikLayerParamData mapnik_layer_get_param ( VikMapnikLayer *vml, guint16 id, gboolean is_file_operation );
 static VikMapnikLayer *mapnik_layer_new ( VikViewport *vvp );
@@ -123,7 +106,8 @@ static gboolean mapnik_feature_release ( VikMapnikLayer *vml, GdkEventButton *ev
 static VikToolInterface mapnik_tools[] = {
 	// Layer Info
 	// Zoom All?
-  { { "MapnikFeatures", GTK_STOCK_INFO, N_("_Mapnik Features"), NULL, N_("Mapnik Features"), 0 },
+  { NULL,
+    { "MapnikFeatures", GTK_STOCK_INFO, N_("_Mapnik Features"), NULL, N_("Mapnik Features"), 0 },
     (VikToolConstructorFunc) mapnik_feature_create,
     NULL,
     NULL,
@@ -132,6 +116,7 @@ static VikToolInterface mapnik_tools[] = {
     NULL,
     (VikToolMouseFunc) mapnik_feature_release,
     NULL,
+    NULL,
     FALSE,
     GDK_LEFT_PTR, NULL, NULL },
 };
@@ -145,7 +130,7 @@ VikLayerInterface vik_mapnik_layer_interface = {
 	&vikmapniklayer_pixbuf, // icon
 
 	mapnik_tools,
-	sizeof(mapnik_tools) / sizeof(VikToolInterface),
+	G_N_ELEMENTS(mapnik_tools),
 
 	mapnik_layer_params,
 	NUM_PARAMS,
@@ -198,6 +183,8 @@ VikLayerInterface vik_mapnik_layer_interface = {
 	(VikLayerFuncSelectMove)              NULL,
 	(VikLayerFuncSelectRelease)           NULL,
 	(VikLayerFuncSelectedViewportMenu)    NULL,
+
+	(VikLayerFuncRefresh)                 NULL,
 };
 
 struct _VikMapnikLayer {
@@ -231,8 +218,10 @@ static VikLayerParamData plugins_default ( void )
 #else
 	if ( g_file_test ( "/usr/lib/mapnik/input", G_FILE_TEST_EXISTS ) )
 		data.s = g_strdup ( "/usr/lib/mapnik/input" );
+		// Current Debian locations
+	else if ( g_file_test ( "/usr/lib/mapnik/3.0/input", G_FILE_TEST_EXISTS ) )
+		data.s = g_strdup ( "/usr/lib/mapnik/3.0/input" );
 	else if ( g_file_test ( "/usr/lib/mapnik/2.2/input", G_FILE_TEST_EXISTS ) )
-		// Current Debian location
 		data.s = g_strdup ( "/usr/lib/mapnik/2.2/input" );
 	else
 		data.s = g_strdup ( "" );
@@ -276,7 +265,7 @@ static GHashTable *requests = NULL;
  */
 void vik_mapnik_layer_init (void)
 {
-	a_preferences_register_group ( MAPNIK_PREFS_GROUP_KEY, "Mapnik" );
+	a_preferences_register_group ( MAPNIK_PREFS_GROUP_KEY, _("Mapnik") );
 
 	guint i = 0;
 	VikLayerParamData tmp = plugins_default();
@@ -333,9 +322,14 @@ void vik_mapnik_layer_uninit ()
 // NB Only performed once per program run
 static void mapnik_layer_class_init ( VikMapnikLayerClass *klass )
 {
-	mapnik_interface_initialize ( a_preferences_get (MAPNIK_PREFS_NAMESPACE"plugins_directory")->s,
-	                              a_preferences_get (MAPNIK_PREFS_NAMESPACE"fonts_directory")->s,
-	                              a_preferences_get (MAPNIK_PREFS_NAMESPACE"recurse_fonts_directory")->b );
+	VikLayerParamData *pd = a_preferences_get (MAPNIK_PREFS_NAMESPACE"plugins_directory");
+	VikLayerParamData *fd = a_preferences_get (MAPNIK_PREFS_NAMESPACE"fonts_directory");
+	VikLayerParamData *rfd = a_preferences_get (MAPNIK_PREFS_NAMESPACE"recurse_fonts_directory");
+
+	if ( pd && fd && rfd )
+		mapnik_interface_initialize ( pd->s, fd->s, rfd->b );
+	else
+		g_critical ( "Unable to initialize mapnik interface from preferences" );
 }
 
 GType vik_mapnik_layer_get_type ()
@@ -390,12 +384,12 @@ static void mapnik_layer_set_cache_dir ( VikMapnikLayer *vml, const gchar *name
 	vml->file_cache_dir = g_strdup (name);
 }
 
-static void mapnik_layer_marshall( VikMapnikLayer *vml, guint8 **data, gint *len )
+static void mapnik_layer_marshall( VikMapnikLayer *vml, guint8 **data, guint *len )
 {
 	vik_layer_marshall_params ( VIK_LAYER(vml), data, len );
 }
 
-static VikMapnikLayer *mapnik_layer_unmarshall( guint8 *data, gint len, VikViewport *vvp )
+static VikMapnikLayer *mapnik_layer_unmarshall( guint8 *data, guint len, VikViewport *vvp )
 {
 	VikMapnikLayer *rv = mapnik_layer_new ( vvp );
 	vik_layer_unmarshall_params ( VIK_LAYER(rv), data, len, vvp );
@@ -502,6 +496,7 @@ gboolean carto_load ( VikMapnikLayer *vml, VikViewport *vvp )
 		gchar *msg = g_strdup_printf ( "%s: %s", _("Running"), command );
 		vik_window_statusbar_update ( vw, msg, VIK_STATUSBAR_INFO );
 		vik_window_set_busy_cursor ( vw );
+		g_free ( msg );
 	}
 
 	gint64 tt1 = 0;
@@ -536,6 +531,7 @@ gboolean carto_load ( VikMapnikLayer *vml, VikViewport *vvp )
 				if ( !g_strcmp0 ( vml->filename_xml, vml->filename_css ) ) {
 					vml->filename_xml = g_strconcat ( vml->filename_css, ".xml", NULL );
 				}
+				g_regex_unref ( regex );
 			}
 			if ( !g_file_set_contents (vml->filename_xml, mystdout, -1, &error)  ) {
 				g_warning ("%s: %s", __FUNCTION__, error->message );
@@ -630,7 +626,8 @@ static void possibly_save_pixbuf ( VikMapnikLayer *vml, GdkPixbuf *pixbuf, MapCo
 
 			gchar *dir = g_path_get_dirname ( filename );
 			if ( !g_file_test ( filename, G_FILE_TEST_EXISTS ) )
-				g_mkdir_with_parents ( dir , 0777 );
+				if ( g_mkdir_with_parents ( dir , 0777 ) != 0 )
+					g_warning ("%s: Failed to mkdir %s", __FUNCTION__, dir );
 			g_free ( dir );
 
 			if ( !gdk_pixbuf_save (pixbuf, filename, "png", &error, NULL ) ) {
@@ -671,8 +668,9 @@ static void render ( VikMapnikLayer *vml, VikCoord *ul, VikCoord *br, MapCoord *
 
 	// NB Mapnik can apply alpha, but use our own function for now
 	if ( vml->alpha < 255 )
-		pixbuf = ui_pixbuf_set_alpha ( pixbuf, vml->alpha );
+		pixbuf = ui_pixbuf_scale_alpha ( pixbuf, vml->alpha );
 	a_mapcache_add ( pixbuf, (mapcache_extra_t){ tt }, ulm->x, ulm->y, ulm->z, MAP_ID_MAPNIK_RENDER, ulm->scale, vml->alpha, 0.0, 0.0, vml->filename_xml );
+	g_object_unref(pixbuf);
 }
 
 static void render_info_free ( RenderInfo *data )
@@ -709,7 +707,7 @@ static void render_cancel_cleanup (RenderInfo *data)
 /**
  * Thread
  */
-void thread_add (VikMapnikLayer *vml, MapCoord *mul, VikCoord *ul, VikCoord *br, gint x, gint y, gint z, gint zoom, const gchar* name )
+static void thread_add (VikMapnikLayer *vml, MapCoord *mul, VikCoord *ul, VikCoord *br, gint x, gint y, gint z, gint zoom, const gchar* name )
 {
 	// Create request
 	guint nn = name ? g_str_hash ( name ) : 0;
@@ -753,6 +751,9 @@ void thread_add (VikMapnikLayer *vml, MapCoord *mul, VikCoord *ul, VikCoord *br,
 
 /**
  * load_pixbuf:
+ *
+ * If function returns GdkPixbuf properly, reference counter to this
+ * buffer has to be decreased, when buffer is no longer needed.
  */
 static GdkPixbuf *load_pixbuf ( VikMapnikLayer *vml, MapCoord *ulm, MapCoord *brm, gboolean *rerender )
 {
@@ -785,7 +786,8 @@ static GdkPixbuf *load_pixbuf ( VikMapnikLayer *vml, MapCoord *ulm, MapCoord *br
 }
 
 /**
- *
+ * Caller has to decrease reference counter of returned
+ * GdkPixbuf, when buffer is no longer needed.
  */
 static GdkPixbuf *get_pixbuf ( VikMapnikLayer *vml, MapCoord *ulm, MapCoord *brm )
 {
@@ -872,6 +874,7 @@ static void mapnik_layer_draw ( VikMapnikLayer *vml, VikViewport *vvp )
 					map_utils_iTMS_to_vikcoord ( &ulm, &coord );
 					vik_viewport_coord_to_screen ( vvp, &coord, &xx, &yy );
 					vik_viewport_draw_pixbuf ( vvp, pixbuf, 0, 0, xx, yy, vml->tile_size_x, vml->tile_size_x );
+					g_object_unref(pixbuf);
 				}
 			}
 		}
@@ -1009,41 +1012,21 @@ static void mapnik_layer_add_menu_items ( VikMapnikLayer *vml, GtkMenu *menu, gp
 	values[MA_VML] = vml;
 	values[MA_VVP] = vik_layers_panel_get_viewport( VIK_LAYERS_PANEL(vlp) );
 
-	GtkWidget *item = gtk_menu_item_new();
-	gtk_menu_shell_append ( GTK_MENU_SHELL(menu), item );
-	gtk_widget_show ( item );
+	(void)vu_menu_add_item ( menu, NULL, NULL, NULL, NULL ); // Just a separator
 
 	// Typical users shouldn't need to use this functionality - so debug only ATM
 	if ( vik_debug ) {
-		item = gtk_image_menu_item_new_with_mnemonic ( _("_Flush Memory Cache") );
-		gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU) );
-		g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_flush_memory), values );
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		gtk_widget_show ( item );
+		(void)vu_menu_add_item ( menu, _("_Flush Memory Cache"), GTK_STOCK_REMOVE, G_CALLBACK(mapnik_layer_flush_memory), values );
 	}
 
-	item = gtk_image_menu_item_new_from_stock ( GTK_STOCK_REFRESH, NULL );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_reload), values );
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show ( item );
+	(void)vu_menu_add_item ( menu, NULL, GTK_STOCK_REFRESH, G_CALLBACK(mapnik_layer_reload), values );
 
 	if ( g_strcmp0 ("", vml->filename_css) ) {
-		item = gtk_image_menu_item_new_with_mnemonic ( _("_Run Carto Command") );
-		gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_EXECUTE, GTK_ICON_SIZE_MENU) );
-		g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_carto), values );
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		gtk_widget_show ( item );
+		(void)vu_menu_add_item ( menu, _("_Run Carto Command"), GTK_STOCK_EXECUTE, G_CALLBACK(mapnik_layer_carto), values );
 	}
 
-	item = gtk_image_menu_item_new_from_stock ( GTK_STOCK_INFO, NULL );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_information), values );
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show ( item );
-
-	item = gtk_image_menu_item_new_from_stock ( GTK_STOCK_ABOUT, NULL );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_about), values );
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show ( item );
+	(void)vu_menu_add_item ( menu, NULL, GTK_STOCK_INFO, G_CALLBACK(mapnik_layer_information), values );
+	(void)vu_menu_add_item ( menu, NULL, GTK_STOCK_ABOUT, G_CALLBACK(mapnik_layer_about), values );
 }
 
 /**
@@ -1082,7 +1065,7 @@ static void mapnik_layer_tile_info ( VikMapnikLayer *vml )
 	if ( g_file_test ( filename, G_FILE_TEST_EXISTS ) ) {
 		filemsg = g_strconcat ( "Tile File: ", filename, NULL );
 		// Get some timestamp information of the tile
-		struct stat stat_buf;
+		GStatBuf stat_buf;
 		if ( g_stat ( filename, &stat_buf ) == 0 ) {
 			gchar time_buf[64];
 			strftime ( time_buf, sizeof(time_buf), "%c", gmtime((const time_t *)&stat_buf.st_mtime) );
@@ -1126,18 +1109,10 @@ static gboolean mapnik_feature_release ( VikMapnikLayer *vml, GdkEventButton *ev
 		vml->rerender_zoom = vik_viewport_get_zoom ( vvp );
 
 		if ( ! vml->right_click_menu ) {
-			GtkWidget *item;
 			vml->right_click_menu = gtk_menu_new ();
-
-			item = gtk_image_menu_item_new_with_mnemonic ( _("_Rerender Tile") );
-			gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU) );
-			g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_rerender), vml );
-			gtk_menu_shell_append ( GTK_MENU_SHELL(vml->right_click_menu), item );
-
-			item = gtk_image_menu_item_new_with_mnemonic ( _("_Info") );
-			gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_INFO, GTK_ICON_SIZE_MENU) );
-			g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(mapnik_layer_tile_info), vml );
-			gtk_menu_shell_append ( GTK_MENU_SHELL(vml->right_click_menu), item );
+			GtkMenu *menu = GTK_MENU(vml->right_click_menu); // local rename
+			(void)vu_menu_add_item ( menu, _("_Rerender Tile"), GTK_STOCK_REFRESH, G_CALLBACK(mapnik_layer_rerender), vml );
+			(void)vu_menu_add_item ( menu, _("_Info"), GTK_STOCK_INFO, G_CALLBACK(mapnik_layer_tile_info), vml );
 		}
 
 		gtk_menu_popup ( GTK_MENU(vml->right_click_menu), NULL, NULL, NULL, NULL, event->button, event->time );
