@@ -29,6 +29,7 @@
 #include "vikmapsourcedefault.h"
 #include "vikenumtypes.h"
 #include "download.h"
+#include <string.h>
 
 static void map_source_get_copyright (VikMapSource *self, LatLonBBox bbox, gdouble zoom, void (*fct)(VikViewport*,const gchar*), void *data);
 static const gchar *map_source_get_license (VikMapSource *self);
@@ -40,6 +41,7 @@ static guint16 map_source_get_uniq_id (VikMapSource *self);
 static const gchar *map_source_get_label (VikMapSource *self);
 static guint16 map_source_get_tilesize_x (VikMapSource *self);
 static guint16 map_source_get_tilesize_y (VikMapSource *self);
+static gdouble map_source_get_scale (VikMapSource *self);
 static VikViewportDrawMode map_source_get_drawmode (VikMapSource *self);
 static const gchar *map_source_get_file_extension (VikMapSource *self);
 
@@ -61,6 +63,7 @@ struct _VikMapSourceDefaultPrivate
 	gchar *label;
 	guint16 tilesize_x;
 	guint16 tilesize_y;
+	gdouble scale;
 	VikViewportDrawMode drawmode;
 	gchar *file_extension;
 };
@@ -77,6 +80,7 @@ enum
   PROP_LABEL,
   PROP_TILESIZE_X,
   PROP_TILESIZE_Y,
+  PROP_SCALE,
   PROP_DRAWMODE,
   PROP_COPYRIGHT,
   PROP_LICENSE,
@@ -161,6 +165,10 @@ vik_map_source_default_set_property (GObject      *object,
       priv->tilesize_y = g_value_get_uint (value);
       break;
 
+    case PROP_SCALE:
+      priv->scale = g_value_get_double (value);
+      break;
+
     case PROP_DRAWMODE:
       priv->drawmode = g_value_get_enum(value);
       break;
@@ -223,6 +231,10 @@ vik_map_source_default_get_property (GObject    *object,
       g_value_set_uint (value, priv->tilesize_y);
       break;
 
+    case PROP_SCALE:
+      g_value_set_double (value, priv->scale);
+      break;
+
     case PROP_DRAWMODE:
       g_value_set_enum (value, priv->drawmode);
       break;
@@ -270,6 +282,7 @@ vik_map_source_default_class_init (VikMapSourceDefaultClass *klass)
 	parent_class->get_label =      map_source_get_label;
 	parent_class->get_tilesize_x = map_source_get_tilesize_x;
 	parent_class->get_tilesize_y = map_source_get_tilesize_y;
+	parent_class->get_scale =      map_source_get_scale;
 	parent_class->get_drawmode =   map_source_get_drawmode;
 	parent_class->get_file_extension = map_source_get_file_extension;
 	parent_class->download =                 _download;
@@ -322,6 +335,15 @@ vik_map_source_default_class_init (VikMapSourceDefaultClass *klass)
                                G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_TILESIZE_Y, pspec);
 
+	pspec = g_param_spec_double ("scale",
+	                             "Scale",
+                                 "Scale of a tile",
+                                 0.0, // minimum value
+                                 99999.0, // maximum value
+                                 1.0, // default value
+                                 G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_SCALE, pspec);
+
 	pspec = g_param_spec_enum("drawmode",
                               "Drawmode",
                               "The mode used to draw map",
@@ -452,6 +474,16 @@ map_source_get_tilesize_y (VikMapSource *self)
 	return priv->tilesize_y;
 }
 
+static double
+map_source_get_scale (VikMapSource *self)
+{
+	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT(self), (gdouble)1.0);
+
+    VikMapSourceDefaultPrivate *priv = VIK_MAP_SOURCE_DEFAULT_PRIVATE(self);
+
+	return priv->scale;
+}
+
 static VikViewportDrawMode
 map_source_get_drawmode (VikMapSource *self)
 {
@@ -467,7 +499,7 @@ _download ( VikMapSource *self, MapCoord *src, const gchar *dest_fn, void *handl
 {
    gchar *uri = vik_map_source_default_get_uri(VIK_MAP_SOURCE_DEFAULT(self), src);
    gchar *host = vik_map_source_default_get_hostname(VIK_MAP_SOURCE_DEFAULT(self));
-   DownloadMapOptions *options = vik_map_source_default_get_download_options(VIK_MAP_SOURCE_DEFAULT(self));
+   DownloadFileOptions *options = vik_map_source_default_get_download_options(VIK_MAP_SOURCE_DEFAULT(self));
    DownloadResult_t res = a_http_download_get_url ( host, uri, dest_fn, options, handle );
    g_free ( uri );
    g_free ( host );
@@ -521,7 +553,7 @@ vik_map_source_default_get_hostname( VikMapSourceDefault *self )
 	return (*klass->get_hostname)(self);
 }
 
-DownloadMapOptions *
+DownloadFileOptions *
 vik_map_source_default_get_download_options( VikMapSourceDefault *self )
 {
 	VikMapSourceDefaultClass *klass;
@@ -533,3 +565,40 @@ vik_map_source_default_get_download_options( VikMapSourceDefault *self )
 
 	return (*klass->get_download_options)(self);
 }
+
+gchar *
+vik_map_source_default_get_url_display( VikMapSourceDefault *self, MapCoord *src )
+{
+	VikMapSourceDefaultClass *klass;
+	g_return_val_if_fail (self != NULL, 0);
+	g_return_val_if_fail (VIK_IS_MAP_SOURCE_DEFAULT (self), 0);
+	klass = VIK_MAP_SOURCE_DEFAULT_GET_CLASS(self);
+
+	g_return_val_if_fail (klass->get_uri != NULL, 0);
+	g_return_val_if_fail (klass->get_hostname != NULL, 0);
+
+	gchar *newstr = NULL;
+	gchar *hostname = (*klass->get_hostname)(self);
+	gchar *url = (*klass->get_uri)(self, src);
+	if ( hostname && strlen(hostname)>1 ) {
+		if ( strstr (hostname, "://") == NULL ) {
+			//prepend http
+			newstr = g_strdup_printf ( "http://%s", hostname );
+		}
+		else {
+			newstr = g_strdup ( hostname );
+		}
+	}
+	if ( url && strlen(url)>1 ) {
+		if ( newstr ) {
+			gchar *tmp = g_strdup ( newstr );
+			newstr = g_strdup_printf ( "%s%s", newstr, url );
+			g_free ( tmp );
+		}
+		else {
+			newstr = g_strdup ( url );
+		}
+	}
+
+	return newstr;
+}
