@@ -24,7 +24,6 @@
 #endif
 
 #include <glib.h>
-#include <time.h>
 #include <stdlib.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
@@ -105,6 +104,28 @@ void vik_track_set_description(VikTrack *tr, const gchar *description)
     tr->description = NULL;
 }
 
+void vik_track_set_source(VikTrack *tr, const gchar *source)
+{
+  if ( tr->source )
+    g_free ( tr->source );
+
+  if ( source && source[0] != '\0' )
+    tr->source = g_strdup(source);
+  else
+    tr->source = NULL;
+}
+
+void vik_track_set_type(VikTrack *tr, const gchar *type)
+{
+  if ( tr->type )
+    g_free ( tr->type );
+
+  if ( type && type[0] != '\0' )
+    tr->type = g_strdup(type);
+  else
+    tr->type = NULL;
+}
+
 void vik_track_ref(VikTrack *tr)
 {
   tr->ref_count++;
@@ -132,6 +153,10 @@ void vik_track_free(VikTrack *tr)
     g_free ( tr->comment );
   if ( tr->description )
     g_free ( tr->description );
+  if ( tr->source )
+    g_free ( tr->source );
+  if ( tr->type )
+    g_free ( tr->type );
   g_list_foreach ( tr->trackpoints, (GFunc) vik_trackpoint_free, NULL );
   g_list_free( tr->trackpoints );
   if (tr->property_dialog)
@@ -153,7 +178,6 @@ void vik_track_free(VikTrack *tr)
 VikTrack *vik_track_copy ( const VikTrack *tr, gboolean copy_points )
 {
   VikTrack *new_tr = vik_track_new();
-  new_tr->name = g_strdup(tr->name);
   new_tr->visible = tr->visible;
   new_tr->is_route = tr->is_route;
   new_tr->draw_name_mode = tr->draw_name_mode;
@@ -177,18 +201,20 @@ VikTrack *vik_track_copy ( const VikTrack *tr, gboolean copy_points )
   vik_track_set_name(new_tr,tr->name);
   vik_track_set_comment(new_tr,tr->comment);
   vik_track_set_description(new_tr,tr->description);
+  vik_track_set_source(new_tr,tr->source);
   return new_tr;
 }
 
 VikTrackpoint *vik_trackpoint_new()
 {
   VikTrackpoint *tp = g_malloc0(sizeof(VikTrackpoint));
+  tp->timestamp = NAN;
   tp->speed = NAN;
   tp->course = NAN;
-  tp->altitude = VIK_DEFAULT_ALTITUDE;
-  tp->hdop = VIK_DEFAULT_DOP;
-  tp->vdop = VIK_DEFAULT_DOP;
-  tp->pdop = VIK_DEFAULT_DOP;
+  tp->altitude = NAN;
+  tp->hdop = NAN;
+  tp->vdop = NAN;
+  tp->pdop = NAN;
   return tp;
 }
 
@@ -391,8 +417,8 @@ gulong vik_track_get_same_time_point_count ( const VikTrack *tr )
   GList *iter = tr->trackpoints;
   while ( iter ) {
     if ( iter->next &&
-	 ( VIK_TRACKPOINT(iter->data)->has_timestamp &&
-           VIK_TRACKPOINT(iter->next->data)->has_timestamp ) &&
+	 ( !isnan(VIK_TRACKPOINT(iter->data)->timestamp) &&
+           !isnan(VIK_TRACKPOINT(iter->next->data)->timestamp) ) &&
          ( VIK_TRACKPOINT(iter->data)->timestamp ==
            VIK_TRACKPOINT(iter->next->data)->timestamp) )
       num++;
@@ -411,8 +437,8 @@ gulong vik_track_remove_same_time_points ( VikTrack *tr )
   GList *iter = tr->trackpoints;
   while ( iter ) {
     if ( iter->next &&
-	 ( VIK_TRACKPOINT(iter->data)->has_timestamp &&
-           VIK_TRACKPOINT(iter->next->data)->has_timestamp ) &&
+	 ( !isnan(VIK_TRACKPOINT(iter->data)->timestamp) &&
+           !isnan(VIK_TRACKPOINT(iter->next->data)->timestamp) ) &&
          ( VIK_TRACKPOINT(iter->data)->timestamp ==
            VIK_TRACKPOINT(iter->next->data)->timestamp) ) {
 
@@ -434,6 +460,54 @@ gulong vik_track_remove_same_time_points ( VikTrack *tr )
   return num;
 }
 
+/**
+ * vik_track_remove_dodgy_first_point:
+ * @vt:            The track
+ * @speed:         Maximum speed in m/s between points allowed
+ * @recalc_bounds: Whether track bounds should be recalculated
+ *           (i.e. can be skipped if bounds will get calculated later on)
+ *
+ * Returns: Whether the first point was removed
+ *
+ * ATM Primarily to remove dodgy first point inserted back at previous
+ *  location by some Garmin GPS Etrex units after being significantly moved
+ * e.g. you've driven somewhere else and start recording a new cycle/walk etc...
+ *
+ * NB This function is limited to just handling first point issues,
+ *  rather than a more comprehensive attempt to remove any suspicious points
+ *  through-out the track.
+ */
+gboolean vik_track_remove_dodgy_first_point ( VikTrack *vt, guint speed, gboolean recalc_bounds )
+{
+  gboolean deleted = FALSE;
+
+  if ( vt->trackpoints ) {
+    GList *iter = g_list_first ( vt->trackpoints );
+    VikTrackpoint *tp1 = VIK_TRACKPOINT(iter->data);
+
+    if ( !isnan(tp1->timestamp) ) {
+      if ( iter->next ) {
+        VikTrackpoint *tp2 = VIK_TRACKPOINT(iter->next->data);
+        if ( !isnan(tp2->timestamp) ) {
+          gdouble dist_diff = vik_coord_diff ( &tp1->coord, &tp2->coord );
+          gdouble time_diff = tp2->timestamp - tp1->timestamp;
+
+	  gdouble spd = fabs(dist_diff / time_diff);
+          if ( spd > speed ) {
+            deleted = TRUE;
+            vik_trackpoint_free ( tp1 );
+            vt->trackpoints = g_list_delete_link ( vt->trackpoints, iter );
+            if ( recalc_bounds )
+              vik_track_calculate_bounds ( vt );
+	  }
+	}
+      }
+    }
+  }
+
+  return deleted;
+}
+
 /*
  * Deletes all 'extra' trackpoint information
  *  such as time stamps, speed, course etc...
@@ -445,13 +519,12 @@ void vik_track_to_routepoints ( VikTrack *tr )
 
     // c.f. with vik_trackpoint_new()
 
-    VIK_TRACKPOINT(iter->data)->has_timestamp = FALSE;
-    VIK_TRACKPOINT(iter->data)->timestamp = 0;
+    VIK_TRACKPOINT(iter->data)->timestamp = NAN;
     VIK_TRACKPOINT(iter->data)->speed = NAN;
     VIK_TRACKPOINT(iter->data)->course = NAN;
-    VIK_TRACKPOINT(iter->data)->hdop = VIK_DEFAULT_DOP;
-    VIK_TRACKPOINT(iter->data)->vdop = VIK_DEFAULT_DOP;
-    VIK_TRACKPOINT(iter->data)->pdop = VIK_DEFAULT_DOP;
+    VIK_TRACKPOINT(iter->data)->hdop = NAN;
+    VIK_TRACKPOINT(iter->data)->vdop = NAN;
+    VIK_TRACKPOINT(iter->data)->pdop = NAN;
     VIK_TRACKPOINT(iter->data)->nsats = 0;
     VIK_TRACKPOINT(iter->data)->fix_mode = VIK_GPS_MODE_NOT_SEEN;
 
@@ -562,22 +635,38 @@ void vik_track_reverse ( VikTrack *tr )
 /**
  * vik_track_get_duration:
  * @trk: The track
+ * @segment_gaps: Whether the duration should include gaps between segments
  *
- * Returns: The time in seconds that covers the whole track including gaps
+ * Returns: The time in seconds
  *  NB this may be negative particularly if the track has been reversed
  */
-time_t vik_track_get_duration(const VikTrack *trk)
+gdouble vik_track_get_duration(const VikTrack *trk, gboolean segment_gaps)
 {
-  time_t duration = 0;
+  gdouble duration = 0;
   if ( trk->trackpoints ) {
     // Ensure times are available
-    if ( vik_track_get_tp_first(trk)->has_timestamp ) {
+    if ( !isnan(vik_track_get_tp_first(trk)->timestamp) ) {
       // Get trkpt only once - as using vik_track_get_tp_last() iterates whole track each time
-      VikTrackpoint *trkpt_last = vik_track_get_tp_last(trk);
-      if ( trkpt_last->has_timestamp ) {
-        time_t t1 = vik_track_get_tp_first(trk)->timestamp;
-        time_t t2 = trkpt_last->timestamp;
-        duration = t2 - t1;
+      if (segment_gaps) {
+        // Simple duration
+        VikTrackpoint *trkpt_last = vik_track_get_tp_last(trk);
+        if ( !isnan(trkpt_last->timestamp) ) {
+          gdouble t1 = vik_track_get_tp_first(trk)->timestamp;
+          gdouble t2 = trkpt_last->timestamp;
+          duration = t2 - t1;
+        }
+      }
+      else {
+        // Total within segments
+        GList *iter = trk->trackpoints->next;
+        while (iter) {
+          if ( !isnan(VIK_TRACKPOINT(iter->data)->timestamp) &&
+               !isnan(VIK_TRACKPOINT(iter->prev->data)->timestamp) &&
+              (!VIK_TRACKPOINT(iter->data)->newsegment) ) {
+            duration += ABS(VIK_TRACKPOINT(iter->data)->timestamp - VIK_TRACKPOINT(iter->prev->data)->timestamp);
+          }
+          iter = iter->next;
+        }
       }
     }
   }
@@ -587,14 +676,14 @@ time_t vik_track_get_duration(const VikTrack *trk)
 gdouble vik_track_get_average_speed(const VikTrack *tr)
 {
   gdouble len = 0.0;
-  guint32 time = 0;
+  gdouble time = 0;
   if ( tr->trackpoints )
   {
     GList *iter = tr->trackpoints->next;
     while (iter)
     {
-      if ( VIK_TRACKPOINT(iter->data)->has_timestamp && 
-          VIK_TRACKPOINT(iter->prev->data)->has_timestamp &&
+      if ( !isnan(VIK_TRACKPOINT(iter->data)->timestamp) &&
+           !isnan(VIK_TRACKPOINT(iter->prev->data)->timestamp) &&
           (! VIK_TRACKPOINT(iter->data)->newsegment) )
       {
         len += vik_coord_diff ( &(VIK_TRACKPOINT(iter->data)->coord),
@@ -620,14 +709,14 @@ gdouble vik_track_get_average_speed(const VikTrack *tr)
 gdouble vik_track_get_average_speed_moving (const VikTrack *tr, int stop_length_seconds)
 {
   gdouble len = 0.0;
-  guint32 time = 0;
+  gdouble time = 0;
   if ( tr->trackpoints )
   {
     GList *iter = tr->trackpoints->next;
     while (iter)
     {
-      if ( VIK_TRACKPOINT(iter->data)->has_timestamp &&
-          VIK_TRACKPOINT(iter->prev->data)->has_timestamp &&
+      if ( !isnan(VIK_TRACKPOINT(iter->data)->timestamp) &&
+           !isnan(VIK_TRACKPOINT(iter->prev->data)->timestamp) &&
           (! VIK_TRACKPOINT(iter->data)->newsegment) )
       {
 	if ( ( VIK_TRACKPOINT(iter->data)->timestamp - VIK_TRACKPOINT(iter->prev->data)->timestamp ) < stop_length_seconds ) {
@@ -651,8 +740,8 @@ gdouble vik_track_get_max_speed(const VikTrack *tr)
     GList *iter = tr->trackpoints->next;
     while (iter)
     {
-      if ( VIK_TRACKPOINT(iter->data)->has_timestamp && 
-          VIK_TRACKPOINT(iter->prev->data)->has_timestamp &&
+      if ( !isnan(VIK_TRACKPOINT(iter->data)->timestamp) &&
+           !isnan(VIK_TRACKPOINT(iter->prev->data)->timestamp) &&
           (! VIK_TRACKPOINT(iter->data)->newsegment) )
       {
         speed =  vik_coord_diff ( &(VIK_TRACKPOINT(iter->data)->coord), &(VIK_TRACKPOINT(iter->prev->data)->coord) )
@@ -699,7 +788,7 @@ gdouble *vik_track_make_elevation_map ( const VikTrack *tr, guint16 num_chunks )
       // Since when is 9.9999e+24 a valid elevation!!
       // This can happen when a track (with no elevations) is uploaded to a GPS device and then redownloaded (e.g. using a Garmin Legend EtrexHCx)
       // Some protection against trying to work with crazily massive numbers (otherwise get SIGFPE, Arithmetic exception)
-      if ( VIK_TRACKPOINT(iter->data)->altitude != VIK_DEFAULT_ALTITUDE &&
+      if ( !isnan(VIK_TRACKPOINT(iter->data)->altitude) &&
 	   VIK_TRACKPOINT(iter->data)->altitude < 1E9 ) {
         okay = TRUE; break;
       }
@@ -807,25 +896,32 @@ gdouble *vik_track_make_elevation_map ( const VikTrack *tr, guint16 num_chunks )
   return pts;
 }
 
-
+/**
+ * vik_track_get_total_elevation_gain:
+ *
+ * elevation gains and losses may be NAN if no elevations are available
+ *
+ */
 void vik_track_get_total_elevation_gain(const VikTrack *tr, gdouble *up, gdouble *down)
 {
   gdouble diff;
   *up = *down = 0;
-  if ( tr->trackpoints && VIK_TRACKPOINT(tr->trackpoints->data)->altitude != VIK_DEFAULT_ALTITUDE )
-  {
+  if ( tr->trackpoints ) {
     GList *iter = tr->trackpoints->next;
-    while (iter)
-    {
-      diff = VIK_TRACKPOINT(iter->data)->altitude - VIK_TRACKPOINT(iter->prev->data)->altitude;
-      if ( diff > 0 )
-        *up += diff;
-      else
-        *down -= diff;
+    while (iter) {
+      VikTrackpoint *tp1 = VIK_TRACKPOINT(iter->data);
+      VikTrackpoint *tp2 = VIK_TRACKPOINT(iter->prev->data);
+      if ( !isnan(tp1->altitude) && !isnan(tp2->altitude) ) {
+        diff = tp1->altitude - tp2->altitude;
+        if ( diff > 0 )
+          *up += diff;
+        else
+          *down -= diff;
+      }
       iter = iter->next;
     }
   } else
-    *up = *down = VIK_DEFAULT_ALTITUDE;
+    *up = *down = NAN;
 }
 
 gdouble *vik_track_make_gradient_map ( const VikTrack *tr, guint16 num_chunks )
@@ -873,7 +969,6 @@ gdouble *vik_track_make_speed_map ( const VikTrack *tr, guint16 num_chunks )
 {
   gdouble *v, *s, *t;
   gdouble duration, chunk_dur;
-  time_t t1, t2;
   int i, pt_count, numpts, index;
   GList *iter;
 
@@ -882,11 +977,11 @@ gdouble *vik_track_make_speed_map ( const VikTrack *tr, guint16 num_chunks )
 
   g_assert ( num_chunks < 16000 );
 
-  t1 = VIK_TRACKPOINT(tr->trackpoints->data)->timestamp;
-  t2 = VIK_TRACKPOINT(g_list_last(tr->trackpoints)->data)->timestamp;
+  gdouble t1 = VIK_TRACKPOINT(tr->trackpoints->data)->timestamp;
+  gdouble t2 = VIK_TRACKPOINT(g_list_last(tr->trackpoints)->data)->timestamp;
   duration = t2 - t1;
 
-  if ( !t1 || !t2 || !duration )
+  if ( isnan(t1) || isnan(t2) || !duration )
     return NULL;
 
   if (duration < 0) {
@@ -949,18 +1044,17 @@ gdouble *vik_track_make_distance_map ( const VikTrack *tr, guint16 num_chunks )
 {
   gdouble *v, *s, *t;
   gdouble duration, chunk_dur;
-  time_t t1, t2;
   int i, pt_count, numpts, index;
   GList *iter;
 
   if ( ! tr->trackpoints )
     return NULL;
 
-  t1 = VIK_TRACKPOINT(tr->trackpoints->data)->timestamp;
-  t2 = VIK_TRACKPOINT(g_list_last(tr->trackpoints)->data)->timestamp;
+  gdouble t1 = VIK_TRACKPOINT(tr->trackpoints->data)->timestamp;
+  gdouble t2 = VIK_TRACKPOINT(g_list_last(tr->trackpoints)->data)->timestamp;
   duration = t2 - t1;
 
-  if ( !t1 || !t2 || !duration )
+  if ( isnan(t1) || isnan(t2) || !duration )
     return NULL;
 
   if (duration < 0) {
@@ -1024,7 +1118,6 @@ gdouble *vik_track_make_distance_map ( const VikTrack *tr, guint16 num_chunks )
  */
 gdouble *vik_track_make_elevation_time_map ( const VikTrack *tr, guint16 num_chunks )
 {
-  time_t t1, t2;
   gdouble duration, chunk_dur;
   GList *iter = tr->trackpoints;
 
@@ -1034,7 +1127,7 @@ gdouble *vik_track_make_elevation_time_map ( const VikTrack *tr, guint16 num_chu
   /* test if there's anything worth calculating */
   gboolean okay = FALSE;
   while ( iter ) {
-    if ( VIK_TRACKPOINT(iter->data)->altitude != VIK_DEFAULT_ALTITUDE ) {
+    if ( !isnan(VIK_TRACKPOINT(iter->data)->altitude) ) {
       okay = TRUE;
       break;
     }
@@ -1043,11 +1136,11 @@ gdouble *vik_track_make_elevation_time_map ( const VikTrack *tr, guint16 num_chu
   if ( ! okay )
     return NULL;
 
-  t1 = VIK_TRACKPOINT(tr->trackpoints->data)->timestamp;
-  t2 = VIK_TRACKPOINT(g_list_last(tr->trackpoints)->data)->timestamp;
+  gdouble t1 = VIK_TRACKPOINT(tr->trackpoints->data)->timestamp;
+  gdouble t2 = VIK_TRACKPOINT(g_list_last(tr->trackpoints)->data)->timestamp;
   duration = t2 - t1;
 
-  if ( !t1 || !t2 || !duration )
+  if ( isnan(t1) || isnan(t2) || !duration )
     return NULL;
 
   if (duration < 0) {
@@ -1112,7 +1205,6 @@ gdouble *vik_track_make_elevation_time_map ( const VikTrack *tr, guint16 num_chu
 gdouble *vik_track_make_speed_dist_map ( const VikTrack *tr, guint16 num_chunks )
 {
   gdouble *v, *s, *t;
-  time_t t1, t2;
   gint i, pt_count, numpts, index;
   GList *iter;
   gdouble duration, total_length, chunk_length;
@@ -1120,11 +1212,11 @@ gdouble *vik_track_make_speed_dist_map ( const VikTrack *tr, guint16 num_chunks
   if ( ! tr->trackpoints )
     return NULL;
 
-  t1 = VIK_TRACKPOINT(tr->trackpoints->data)->timestamp;
-  t2 = VIK_TRACKPOINT(g_list_last(tr->trackpoints)->data)->timestamp;
+  gdouble t1 = VIK_TRACKPOINT(tr->trackpoints->data)->timestamp;
+  gdouble t2 = VIK_TRACKPOINT(g_list_last(tr->trackpoints)->data)->timestamp;
   duration = t2 - t1;
 
-  if ( !t1 || !t2 || !duration )
+  if ( isnan(t1) || isnan(t2) || !duration )
     return NULL;
 
   if (duration < 0) {
@@ -1280,18 +1372,18 @@ VikTrackpoint *vik_track_get_closest_tp_by_percentage_dist ( VikTrack *tr, gdoub
   return NULL;
 }
 
-VikTrackpoint *vik_track_get_closest_tp_by_percentage_time ( VikTrack *tr, gdouble reltime, time_t *seconds_from_start )
+VikTrackpoint *vik_track_get_closest_tp_by_percentage_time ( VikTrack *tr, gdouble reltime, gdouble *seconds_from_start )
 {
-  time_t t_pos, t_start, t_end, t_total;
+  if ( !tr->trackpoints )
+    return NULL;
+
+  gdouble t_pos, t_start, t_end, t_total;
   t_start = VIK_TRACKPOINT(tr->trackpoints->data)->timestamp;
   t_end = VIK_TRACKPOINT(g_list_last(tr->trackpoints)->data)->timestamp;
   t_total = t_end - t_start;
 
   t_pos = t_start + t_total * reltime;
 
-  if ( !tr->trackpoints )
-    return NULL;
-
   GList *iter = tr->trackpoints;
 
   while (iter) {
@@ -1300,8 +1392,8 @@ VikTrackpoint *vik_track_get_closest_tp_by_percentage_time ( VikTrack *tr, gdoub
     if (VIK_TRACKPOINT(iter->data)->timestamp > t_pos) {
       if (iter->prev == NULL)  /* first trackpoint */
         break;
-      time_t t_before = t_pos - VIK_TRACKPOINT(iter->prev->data)->timestamp;
-      time_t t_after = VIK_TRACKPOINT(iter->data)->timestamp - t_pos;
+      gdouble t_before = t_pos - VIK_TRACKPOINT(iter->prev->data)->timestamp;
+      gdouble t_after = VIK_TRACKPOINT(iter->data)->timestamp - t_pos;
       if (t_before <= t_after)
         iter = iter->prev;
       break;
@@ -1330,8 +1422,8 @@ VikTrackpoint* vik_track_get_tp_by_max_speed ( const VikTrack *tr )
 
   while (iter) {
     if (iter->prev) {
-      if ( VIK_TRACKPOINT(iter->data)->has_timestamp &&
-	   VIK_TRACKPOINT(iter->prev->data)->has_timestamp &&
+      if ( !isnan(VIK_TRACKPOINT(iter->data)->timestamp) &&
+           !isnan(VIK_TRACKPOINT(iter->prev->data)->timestamp) &&
 	   (! VIK_TRACKPOINT(iter->data)->newsegment) ) {
 	speed =  vik_coord_diff ( &(VIK_TRACKPOINT(iter->data)->coord), &(VIK_TRACKPOINT(iter->prev->data)->coord) )
 	  / ABS(VIK_TRACKPOINT(iter->data)->timestamp - VIK_TRACKPOINT(iter->prev->data)->timestamp);
@@ -1344,9 +1436,6 @@ VikTrackpoint* vik_track_get_tp_by_max_speed ( const VikTrack *tr )
     iter = iter->next;
   }
   
-  if (!max_speed_tp)
-    return NULL;
-
   return max_speed_tp;
 }
 
@@ -1433,23 +1522,32 @@ VikTrackpoint *vik_track_get_tp_prev ( const VikTrack *tr, VikTrackpoint *tp )
   return tp_prev;
 }
 
+/**
+ * vik_track_get_minmax_alt:
+ *
+ * Finds the minimum and maximum altitudes in the specified track
+ *
+ * Returns: Whether any altitudes where found
+ */
 gboolean vik_track_get_minmax_alt ( const VikTrack *tr, gdouble *min_alt, gdouble *max_alt )
 {
   *min_alt = 25000;
   *max_alt = -5000;
-  if ( tr && tr->trackpoints && tr->trackpoints->data && (VIK_TRACKPOINT(tr->trackpoints->data)->altitude != VIK_DEFAULT_ALTITUDE) ) {
-    GList *iter = tr->trackpoints->next;
+  if ( tr && tr->trackpoints ) {
+    GList *iter = tr->trackpoints;
     gdouble tmp_alt;
-    while (iter)
-    {
-      tmp_alt = VIK_TRACKPOINT(iter->data)->altitude;
-      if ( tmp_alt > *max_alt )
-        *max_alt = tmp_alt;
-      if ( tmp_alt < *min_alt )
-        *min_alt = tmp_alt;
+    while (iter) {
+      VikTrackpoint *tp = VIK_TRACKPOINT(iter->data);
+      if ( !isnan(tp->altitude) ) {
+	tmp_alt = tp->altitude;
+	if ( tmp_alt > *max_alt )
+	  *max_alt = tmp_alt;
+	if ( tmp_alt < *min_alt )
+	  *min_alt = tmp_alt;
+      }
       iter = iter->next;
     }
-    return TRUE;
+    return (*min_alt != 25000);
   }
   return FALSE;
 }
@@ -1487,6 +1585,7 @@ void vik_track_marshall ( VikTrack *tr, guint8 **data, guint *datalen)
   vtm_append(tr->name);
   vtm_append(tr->comment);
   vtm_append(tr->description);
+  vtm_append(tr->source);
 
   *data = b->data;
   *datalen = b->len;
@@ -1496,13 +1595,14 @@ void vik_track_marshall ( VikTrack *tr, guint8 **data, guint *datalen)
 /*
  * Take a byte array and convert it into a Track
  */
-VikTrack *vik_track_unmarshall (guint8 *data, guint datalen)
+VikTrack *vik_track_unmarshall (const guint8 *data_in, guint datalen)
 {
   guint len;
   VikTrack *new_tr = vik_track_new();
   VikTrackpoint *new_tp;
   guint ntp;
   gint i;
+  guint8 *data = (guint8*)data_in;
 
   /* basic properties: */
   new_tr->visible = ((VikTrack *)data)->visible;
@@ -1541,6 +1641,7 @@ VikTrack *vik_track_unmarshall (guint8 *data, guint datalen)
   vtu_get(new_tr->name);
   vtu_get(new_tr->comment);
   vtu_get(new_tr->description);
+  vtu_get(new_tr->source);
 
   return new_tr;
 }
@@ -1592,16 +1693,20 @@ void vik_track_calculate_bounds ( VikTrack *trk )
 void vik_track_anonymize_times ( VikTrack *tr )
 {
   GTimeVal gtv;
-  g_time_val_from_iso8601 ( "1901-01-01T00:00:00Z", &gtv );
+  // Check result just to please Coverity - even though it shouldn't fail as it's a hard coded value here!
+  if ( !g_time_val_from_iso8601 ( "1901-01-01T00:00:00Z", &gtv ) ) {
+    g_critical ( "Calendar time value failure" );
+    return;
+  }
 
-  time_t anon_timestamp = gtv.tv_sec;
-  time_t offset = 0;
+  gdouble anon_timestamp = gtv.tv_sec;
+  gdouble offset = 0;
 
   GList *tp_iter;
   tp_iter = tr->trackpoints;
   while ( tp_iter ) {
     VikTrackpoint *tp = VIK_TRACKPOINT(tp_iter->data);
-    if ( tp->has_timestamp ) {
+    if ( !isnan(tp->timestamp) ) {
       // Calculate an offset in time using the first available timestamp
       if ( offset == 0 )
 	offset = tp->timestamp - anon_timestamp;
@@ -1626,13 +1731,13 @@ void vik_track_anonymize_times ( VikTrack *tr )
 void vik_track_interpolate_times ( VikTrack *tr )
 {
   gdouble tr_dist, cur_dist;
-  time_t tsdiff, tsfirst;
+  gdouble tsdiff, tsfirst;
 
   GList *iter;
   iter = tr->trackpoints;
 
   VikTrackpoint *tp = VIK_TRACKPOINT(iter->data);
-  if ( tp->has_timestamp ) {
+  if ( !isnan(tp->timestamp) ) {
     tsfirst = tp->timestamp;
 
     // Find the end of the track and the last timestamp
@@ -1640,7 +1745,7 @@ void vik_track_interpolate_times ( VikTrack *tr )
       iter = iter->next;
     }
     tp = VIK_TRACKPOINT(iter->data);
-    if ( tp->has_timestamp ) {
+    if ( !isnan(tp->timestamp) ) {
       tsdiff = tp->timestamp - tsfirst;
 
       tr_dist = vik_track_get_length_including_gaps ( tr );
@@ -1655,7 +1760,6 @@ void vik_track_interpolate_times ( VikTrack *tr )
           cur_dist += vik_coord_diff ( &(tp->coord), &(VIK_TRACKPOINT(iter->prev->data)->coord) );
 
           tp->timestamp = (cur_dist / tr_dist) * tsdiff + tsfirst;
-          tp->has_timestamp = TRUE;
         }
         // Some points may now have the same time so remove them.
         vik_track_remove_same_time_points ( tr );
@@ -1678,7 +1782,7 @@ gulong vik_track_apply_dem_data ( VikTrack *tr, gboolean skip_existing )
   tp_iter = tr->trackpoints;
   while ( tp_iter ) {
     // Don't apply if the point already has a value and the overwrite is off
-    if ( !(skip_existing && VIK_TRACKPOINT(tp_iter->data)->altitude != VIK_DEFAULT_ALTITUDE) ) {
+    if ( !(skip_existing && !isnan(VIK_TRACKPOINT(tp_iter->data)->altitude)) ) {
       /* TODO: of the 4 possible choices we have for choosing an elevation
        * (trackpoint in between samples), choose the one with the least elevation change
        * as the last */
@@ -1694,21 +1798,39 @@ gulong vik_track_apply_dem_data ( VikTrack *tr, gboolean skip_existing )
   return num;
 }
 
+/**
+ * vik_trackpoint_apply_dem_data:
+ * Apply DEM data (if available) to the trackpoint
+ * NB This will overwrite whatever was in the trackpoint before
+ */
+gboolean vik_trackpoint_apply_dem_data ( VikTrackpoint *tp )
+{
+  if ( tp ) {
+    gint16 elev = a_dems_get_elev_by_coord ( &(tp->coord), VIK_DEM_INTERPOL_BEST );
+    if ( elev != VIK_DEM_INVALID_ELEVATION ) {
+      tp->altitude = elev;
+      return TRUE;
+    }
+  }
+  return FALSE;
+}
+
 /**
  * vik_track_apply_dem_data_last_trackpoint:
  * Apply DEM data (if available) - to only the last trackpoint
  */
+/*
 void vik_track_apply_dem_data_last_trackpoint ( VikTrack *tr )
 {
   gint16 elev;
   if ( tr->trackpoints ) {
-    /* As in vik_track_apply_dem_data above - use 'best' interpolation method */
+    // As in vik_track_apply_dem_data above - use 'best' interpolation method
     elev = a_dems_get_elev_by_coord ( &(VIK_TRACKPOINT(g_list_last(tr->trackpoints)->data)->coord), VIK_DEM_INTERPOL_BEST );
     if ( elev != VIK_DEM_INVALID_ELEVATION )
       VIK_TRACKPOINT(g_list_last(tr->trackpoints)->data)->altitude = elev;
   }
 }
-
+*/
 
 /**
  * smoothie:
@@ -1749,7 +1871,7 @@ gulong vik_track_smooth_missing_elevation_data ( VikTrack *tr, gboolean flat )
   gulong num = 0;
 
   GList *tp_iter;
-  gdouble elev = VIK_DEFAULT_ALTITUDE;
+  gdouble elev = NAN;
 
   VikTrackpoint *tp_missing = NULL;
   GList *iter_first = NULL;
@@ -1759,10 +1881,10 @@ gulong vik_track_smooth_missing_elevation_data ( VikTrack *tr, gboolean flat )
   while ( tp_iter ) {
     VikTrackpoint *tp = VIK_TRACKPOINT(tp_iter->data);
 
-    if ( VIK_DEFAULT_ALTITUDE == tp->altitude ) {
+    if ( isnan(tp->altitude) ) {
       if ( flat ) {
         // Simply assign to last known value
-	if ( elev != VIK_DEFAULT_ALTITUDE ) {
+	if ( !isnan(elev) ) {
           tp->altitude = elev;
           num++;
 	}
@@ -1784,7 +1906,7 @@ gulong vik_track_smooth_missing_elevation_data ( VikTrack *tr, gboolean flat )
       // Altitude available (maybe again!)
       // If this marks the end of a section of altitude-less points
       //  then apply smoothing for that section of points
-      if ( points > 0 && elev != VIK_DEFAULT_ALTITUDE )
+      if ( points > 0 && !isnan(elev) )
         if ( !flat ) {
           smoothie ( iter_first, tp_iter, elev, tp->altitude, points );
           num = num + points;
@@ -1872,6 +1994,7 @@ VikCoord *vik_track_cut_back_to_double_point ( VikTrack *tr )
 
 /**
  * Function to compare two tracks by their first timestamp
+ * If same timestamp, fall back on name
  **/
 int vik_track_compare_timestamp (const void *x, const void *y)
 {
@@ -1900,5 +2023,145 @@ int vik_track_compare_timestamp (const void *x, const void *y)
   if ( !tpa && tpb )
     return -1;
 
-  return 0;
+  return strcmp ( a->name, b->name );
+}
+
+/**
+ * Scale is the relative distance between trackpoints (0..1)
+ * ATM calculate position, altitude and time
+ *  i.e. not bothered with course and speed etc...
+ */
+static void vik_trackpoint_interpolate ( const VikTrackpoint *tp1, const VikTrackpoint *tp2, gdouble scale, VikTrackpoint *tp )
+{
+  struct LatLon ll, ll1, ll2;
+  vik_coord_to_latlon ( &(tp1->coord), &ll1 );
+  vik_coord_to_latlon ( &(tp2->coord), &ll2 );
+  ll.lat = ll1.lat + ((ll2.lat - ll1.lat) * scale);
+  // NB This won't cope with going over the 180 degrees longitude boundary
+  ll.lon = ll1.lon + ((ll2.lon - ll1.lon) * scale);
+
+  vik_coord_load_from_latlon ( &tp->coord, tp1->coord.mode, &ll );
+
+  if ( !isnan(tp2->altitude) ) {
+    if ( !isnan(tp1->altitude) ) {
+      tp->altitude = tp1->altitude + ((tp2->altitude - tp1->altitude) * scale);
+    } else {
+      tp->altitude = tp2->altitude;
+    }
+  } else {
+    if ( !isnan(tp1->altitude) ) {
+      tp->altitude = tp1->altitude;
+    } else {
+      tp->altitude = NAN;
+    }
+  }
+
+  if ( !isnan(tp1->timestamp) && !isnan(tp2->timestamp) ) {
+    tp->timestamp = tp1->timestamp + ((tp2->timestamp - tp1->timestamp) * scale);
+  } else {
+    tp->timestamp = NAN;
+  }
+}
+
+/**
+ * vik_track_speed_splits:
+ * Free the returned contents of the array & the array itself after use
+ */
+GArray *vik_track_speed_splits (const VikTrack *tr, gdouble split_length )
+{
+  GArray *ga = g_array_new ( FALSE, FALSE, sizeof(VikTrackSpeedSplits_t) );
+
+  if ( !tr->trackpoints ) {
+    return ga;
+  }
+
+  VikTrackpoint *vtp = vik_trackpoint_new(); // 'Virtual' trackpoint
+
+  gdouble len = 0.0;
+  gdouble time = 0;
+  gdouble up = 0.0, down = 0.0;
+  gdouble cur_len = 0.0;
+
+  GList *iter = tr->trackpoints->next;
+  while (iter) {
+    VikTrackpoint *tp1 = VIK_TRACKPOINT(iter->data);
+    VikTrackpoint *tp2 = VIK_TRACKPOINT(iter->prev->data);
+    if ( !isnan(tp1->timestamp) &&
+         !isnan(tp2->timestamp) &&
+         !tp1->newsegment ) {
+
+      gdouble diff;
+      len += vik_coord_diff ( &(tp2->coord), &(tp1->coord) );
+      time += ABS(tp1->timestamp - tp2->timestamp);
+
+      if ( !isnan(tp1->altitude) && !isnan(tp2->altitude) ) {
+        diff = tp1->altitude - tp2->altitude;
+        if ( diff > 0 )
+          up += diff;
+        else
+          down -= diff;
+      }
+
+      if ( len > split_length ) {
+        // Create split info
+        // Normally distance between crossover points is very small compared to the split length
+        // Which is typically for tracks recorded every second or two.
+        // However to cater for trackpoints seperated in both large time & distance gaps,
+        //  need to interpolate back to the split point to get proper split metrics
+
+        // This bit assumes individual distances between track points won't ever be bigger than the split distances
+        gdouble tmp_len = vik_coord_diff ( &(tp1->coord), &tp2->coord);
+        gdouble over_dist = len - split_length;
+        // Ratio of the distance to the virtual split point, compared to point that is over the split point
+        gdouble scale = (split_length - (len - tmp_len)) / tmp_len;
+
+        vik_trackpoint_interpolate ( tp2, tp1, scale, vtp );
+
+        VikTrackSpeedSplits_t vtss;
+        // Remove the extra bit added by going over distance
+        cur_len = cur_len + len - over_dist;
+        vtss.length = cur_len;
+        vtss.time = time - (tp1->timestamp - vtp->timestamp);
+        vtss.speed = ABS((len-over_dist)/vtss.time);
+        // Elevation change is only applied to one of them
+        gdouble alt_diff = 0.0;
+        if ( !isnan(vtp->altitude) && !isnan(tp1->altitude) ) {
+          alt_diff = tp1->altitude - vtp->altitude;
+        }
+        if ( alt_diff > 0 ) {
+          vtss.elev_up = up - alt_diff;
+          vtss.elev_down = down;
+        } else {
+          vtss.elev_up = up;
+          vtss.elev_down = down - fabs ( alt_diff );
+        }
+        g_array_append_val ( ga, vtss );
+
+        // Reset: starting from the virtual trackpoint
+        if ( alt_diff > 0 ) {
+          up = alt_diff;
+          down = 0.0;
+        } else {
+          up = 0.0;
+          down = fabs ( alt_diff );
+        }
+        len = over_dist;
+        time = ABS(tp1->timestamp - vtp->timestamp);
+      }
+    }
+    iter = iter->next;
+  }
+
+  // Values for trackpoints in the final segment
+  // Stick in whatever is left.
+  VikTrackSpeedSplits_t vtss;
+  vtss.length = cur_len + len;
+  vtss.time = time;
+  vtss.speed = time ? ABS(len/time) : 0;
+  vtss.elev_up = up;
+  vtss.elev_down = down;
+  g_array_append_val ( ga, vtss );
+
+  vik_trackpoint_free ( vtp );
+  return ga;
 }
