@@ -19,18 +19,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <gtk/gtk.h>
-#include <glib/gi18n.h>
-
-#include <string.h>
-
 #include "viking.h"
-#include "config.h"
 
 #define TREEVIEW_GET(model,iter,what,dest) gtk_tree_model_get(GTK_TREE_MODEL(model),(iter),(what),(dest),-1)
 
@@ -194,8 +183,7 @@ vik_treeview_tooltip_cb (GtkWidget  *widget,
   GtkTreeView *tree_view = GTK_TREE_VIEW (widget);
   GtkTreeModel *model = gtk_tree_view_get_model (tree_view);
   GtkTreePath *path = NULL;
-
-  char buffer[256];
+  const gchar *tiptext = NULL;
 
   if (!gtk_tree_view_get_tooltip_context (tree_view, &x, &y,
 					  keyboard_tip,
@@ -215,12 +203,12 @@ vik_treeview_tooltip_cb (GtkWidget  *widget,
     gpointer parent;
     gtk_tree_model_get (model, &iter, ITEM_PARENT_COLUMN, &parent, -1);
 
-    g_snprintf (buffer, sizeof(buffer), "%s", vik_layer_sublayer_tooltip (VIK_LAYER(parent), rv, sublayer));
+    tiptext = vik_layer_sublayer_tooltip (VIK_LAYER(parent), rv, sublayer);
   }
   else if ( rv == VIK_TREEVIEW_TYPE_LAYER ) {
     gpointer layer;
     gtk_tree_model_get (model, &iter, ITEM_POINTER_COLUMN, &layer, -1);
-    g_snprintf (buffer, sizeof(buffer), "%s", vik_layer_layer_tooltip (VIK_LAYER(layer)));
+    tiptext = vik_layer_layer_tooltip (VIK_LAYER(layer));
   }
   else {
     gtk_tree_path_free (path);
@@ -228,7 +216,7 @@ vik_treeview_tooltip_cb (GtkWidget  *widget,
   }
 
   // Don't display null strings :)
-  if ( strncmp (buffer, "(null)", 6) == 0 ) {
+  if ( !tiptext ) {
     gtk_tree_path_free (path);
     return FALSE;
   }
@@ -236,7 +224,7 @@ vik_treeview_tooltip_cb (GtkWidget  *widget,
     // No point in using (Pango) markup verson - gtk_tooltip_set_markup()
     //  especially as waypoint comments may well contain HTML markup which confuses the pango markup parser
     // This plain text is probably faster too.
-    gtk_tooltip_set_text (tooltip, buffer);
+    gtk_tooltip_set_text (tooltip, tiptext);
   }
 
   gtk_tree_view_set_tooltip_row (tree_view, tooltip, path);
@@ -284,7 +272,7 @@ void vik_treeview_item_set_pointer ( VikTreeview *vt, GtkTreeIter *iter, gpointe
   gtk_tree_store_set ( GTK_TREE_STORE(vt->model), iter, ITEM_POINTER_COLUMN, pointer, -1 );
 }
 
-void vik_treeview_item_set_timestamp ( VikTreeview *vt, GtkTreeIter *iter, time_t timestamp )
+void vik_treeview_item_set_timestamp ( VikTreeview *vt, GtkTreeIter *iter, gdouble timestamp )
 {
   gtk_tree_store_set ( GTK_TREE_STORE(vt->model), iter, ITEM_TIMESTAMP_COLUMN, timestamp, -1 );
 }
@@ -327,6 +315,43 @@ gboolean vik_treeview_item_get_visible_tree ( VikTreeview *vt, GtkTreeIter *iter
   return ans;
 }
 
+/**
+ * Ensure visibility of an item considering visibility of all parents
+ *  i.e. if any parent is off then they will be turned on
+ * NB This also sets the value of the associated layer
+ *  to ensure the Tree <-> Layer visibility value is kept in sync
+ */
+void vik_treeview_item_set_visible_tree ( VikTreeview *vt, GtkTreeIter *iter )
+{
+  VikLayer *vl;
+  gboolean vis;
+
+  // This layer
+  TREEVIEW_GET ( vt->model, iter, VISIBLE_COLUMN, &vis );
+  if ( !vis ) {
+    gtk_tree_store_set ( GTK_TREE_STORE(vt->model), iter, VISIBLE_COLUMN, TRUE, -1 );
+    vl = VIK_LAYER(vik_treeview_item_get_pointer ( vt, iter ));
+    if ( vl )
+      vl->visible = TRUE;
+  }
+
+  // All parent layers
+  GtkTreeIter parent;
+  GtkTreeIter child = *iter;
+  while ( gtk_tree_model_iter_parent (vt->model, &parent, &child) ) {
+    // Visibility of this parent
+    TREEVIEW_GET ( vt->model, &parent, VISIBLE_COLUMN, &vis );
+    if ( !vis ) {
+      gtk_tree_store_set ( GTK_TREE_STORE(vt->model), &parent, VISIBLE_COLUMN, TRUE, -1 );
+      vl = VIK_LAYER(vik_treeview_item_get_pointer ( vt, &parent ));
+      if ( vl )
+        vl->visible = TRUE;
+    }
+    child = parent;
+  }
+}
+
+
 static void vik_treeview_add_columns ( VikTreeview *vt )
 {
   gint col_offset;
@@ -470,7 +495,6 @@ void vik_treeview_init ( VikTreeview *vt )
 
   // ATM The dates are stored on initial creation and updated when items are deleted
   //  this should be good enough for most purposes, although it may get inaccurate if items are edited in a particular manner
-  // NB implicit conversion of time_t to gint64
   vt->model = GTK_TREE_MODEL(gtk_tree_store_new ( NUM_COLUMNS,
                                                   G_TYPE_STRING,  // Name
                                                   G_TYPE_BOOLEAN, // Visibility
@@ -480,7 +504,7 @@ void vik_treeview_init ( VikTreeview *vt )
                                                   G_TYPE_POINTER, // pointer to the layer or sublayer
                                                   G_TYPE_INT,     // type of the sublayer
                                                   G_TYPE_BOOLEAN, // Editable
-                                                  G_TYPE_INT64 )); // Timestamp
+                                                  G_TYPE_DOUBLE )); // Timestamp
 
   /* create tree view */
   gtk_tree_selection_set_select_function(gtk_tree_view_get_selection (GTK_TREE_VIEW(vt)), vik_treeview_selection_filter, vt, NULL);
@@ -561,7 +585,7 @@ gboolean vik_treeview_move_item ( VikTreeview *vt, GtkTreeIter *iter, gboolean u
 gboolean vik_treeview_get_iter_at_pos ( VikTreeview *vt, GtkTreeIter *iter, gint x, gint y )
 {
   GtkTreePath *path;
-  gtk_tree_view_get_path_at_pos ( GTK_TREE_VIEW(vt), x, y, &path, NULL, NULL, NULL );
+  (void)gtk_tree_view_get_path_at_pos ( GTK_TREE_VIEW(vt), x, y, &path, NULL, NULL, NULL );
   if ( ! path )
     return FALSE;
 
@@ -659,7 +683,7 @@ void vik_treeview_item_unselect ( VikTreeview *vt, GtkTreeIter *iter )
 }
 
 void vik_treeview_add_layer ( VikTreeview *vt, GtkTreeIter *parent_iter, GtkTreeIter *iter, const gchar *name, gpointer parent, gboolean above,
-                              gpointer item, gint data, VikLayerTypeEnum layer_type, time_t timestamp )
+                              gpointer item, gint data, VikLayerTypeEnum layer_type, gdouble timestamp )
 {
   g_assert ( iter != NULL );
   if ( above )
@@ -680,7 +704,7 @@ void vik_treeview_add_layer ( VikTreeview *vt, GtkTreeIter *parent_iter, GtkTree
 }
 
 void vik_treeview_insert_layer ( VikTreeview *vt, GtkTreeIter *parent_iter, GtkTreeIter *iter, const gchar *name, gpointer parent, gboolean above,
-                              gpointer item, gint data, VikLayerTypeEnum layer_type, GtkTreeIter *sibling, time_t timestamp )
+                              gpointer item, gint data, VikLayerTypeEnum layer_type, GtkTreeIter *sibling, gdouble timestamp )
 {
   g_assert ( iter != NULL );
   if (sibling) {
@@ -709,7 +733,7 @@ void vik_treeview_insert_layer ( VikTreeview *vt, GtkTreeIter *parent_iter, GtkT
 }
 
 void vik_treeview_add_sublayer ( VikTreeview *vt, GtkTreeIter *parent_iter, GtkTreeIter *iter, const gchar *name, gpointer parent, gpointer item,
-                                 gint data, GdkPixbuf *icon, gboolean editable, time_t timestamp )
+                                 gint data, GdkPixbuf *icon, gboolean editable, gdouble timestamp )
 {
   g_assert ( iter != NULL );
 
@@ -732,7 +756,7 @@ typedef struct _SortTuple
 {
   gint offset;
   gchar *name;
-  time_t timestamp;
+  gdouble timestamp;
 } SortTuple;
 
 /**
@@ -743,7 +767,7 @@ static gint sort_tuple_compare ( gconstpointer a, gconstpointer b, gpointer orde
   SortTuple *sa = (SortTuple *)a;
   SortTuple *sb = (SortTuple *)b;
 
-  gint answer = 0;
+  gint answer = -1;
   if ( GPOINTER_TO_INT(order) < VL_SO_DATE_ASCENDING ) {
     // Alphabetical comparison
     // Default ascending order
@@ -754,10 +778,12 @@ static gint sort_tuple_compare ( gconstpointer a, gconstpointer b, gpointer orde
   }
   else {
     // Date comparison
-    answer = ( sa->timestamp > sb->timestamp );
+    gboolean ans = ( sa->timestamp > sb->timestamp );
+    if ( ans )
+      answer = 1;
     // Invert sort order for descending order
     if ( GPOINTER_TO_INT(order) == VL_SO_DATE_DESCENDING )
-      answer = !answer;
+      answer = -answer;
   }
   return answer;
 }
@@ -815,7 +841,7 @@ void vik_treeview_sort_children ( VikTreeview *vt, GtkTreeIter *parent, vik_laye
                      GINT_TO_POINTER(order));
 
   // As the sorted list contains the reordered position offsets, extract this and then apply to the treeview
-  gint *positions = g_malloc ( sizeof(gdouble) * length );
+  gint *positions = g_malloc ( sizeof(gint) * length );
   for ( ii = 0; ii < length; ii++ ) {
     positions[ii] = sort_array[ii].offset;
     g_free ( sort_array[ii].name );
@@ -892,8 +918,8 @@ static gboolean vik_treeview_drag_data_received (GtkTreeDragDest *drag_dest, Gtk
 	       vik_treeview_item_get_type(vt, &dest_parent) != VIK_TREEVIEW_TYPE_LAYER);
 
       
-      g_assert ( vik_treeview_item_get_parent(vt, &src_iter) );
       vl_src = vik_treeview_item_get_parent(vt, &src_iter);
+      g_assert ( vl_src );
       vl_dest = vik_treeview_item_get_pointer(vt, &dest_parent);
 
       /* TODO: might want to allow different types, and let the clients handle how they want */
