@@ -23,15 +23,6 @@
  *  Similar to the track and trackpoint properties dialogs,
  *   this is made a separate file for ease of grouping related stuff together
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-#include <math.h>
-#include <time.h>
-#include <string.h>
-#include <stdlib.h>
-#include <gtk/gtk.h>
-#include <glib/gi18n.h>
 #include "viking.h"
 #include "vikfilelist.h"
 #include "geotag_exif.h"
@@ -43,7 +34,7 @@
 
 #define EXIF_DATE_FORMAT "%d:%d:%d %d:%d:%d"
 
-time_t ConvertToUnixTime(char* StringTime, char* Format, int TZOffsetHours, int TZOffsetMinutes)
+static time_t ConvertToUnixTime(char* StringTime, char* Format, int TZOffsetHours, int TZOffsetMinutes, gboolean time_is_local)
 {
 	/* Read the time using the specified format.
 	 * The format and string being read from must
@@ -61,28 +52,34 @@ time_t ConvertToUnixTime(char* StringTime, char* Format, int TZOffsetHours, int
 	struct tm Time;
 	Time.tm_wday = 0;
 	Time.tm_yday = 0;
-	Time.tm_isdst = -1;
 
 	/* Read out the time from the string using our format. */
 	sscanf(StringTime, Format, &Time.tm_year, &Time.tm_mon,
 			&Time.tm_mday, &Time.tm_hour,
 			&Time.tm_min, &Time.tm_sec);
 
-	/* Adjust the years for the mktime function to work. */
+	/* Adjust the years & months for valid tm struct values. */
 	Time.tm_year -= 1900;
 	Time.tm_mon  -= 1;
 
-	/* Add our timezone offset to the time.
-	 * We don't check to see if it overflowed anything;
-	 * mktime does this and fixes it for us. */
-	/* Note also that we SUBTRACT these times. We want the
-	 * result to be in UTC. */
+	/* Calculate the unix time. */
+	time_t thetime;
+	if ( time_is_local ) {
+		Time.tm_isdst = -1;
+		// Time is apparently in local time, thus use mktime() which is locale dependent.
+		// Hence this assumes the current timezone is the same as the timezone the image was taken in.
+		thetime = mktime ( &Time );
+	}
+	else {
+		Time.tm_isdst = 0; // there is no DST in UTC
+		thetime = util_timegm ( &Time );
+	}
+	/* Apply specified timezone offset to the time. */
+	/* Note also that we SUBTRACT these times */
+	thetime -= TZOffsetHours * 60 * 60;
+	thetime -= TZOffsetMinutes * 60;
 
-	Time.tm_hour -= TZOffsetHours;
-	Time.tm_min  -= TZOffsetMinutes;
-
-	/* Calculate and return the unix time. */
-	return mktime(&Time);
+	return thetime;
 }
 
 // GPSCorrelate END
@@ -100,8 +97,10 @@ typedef struct {
 	GtkLabel *overwrite_gps_exif_l; // Referenced so the sensitivity can be changed
 	GtkCheckButton *overwrite_gps_exif_b;
 	GtkLabel *no_change_mtime_l; // Referenced so the sensitivity can be changed
+	GtkCheckButton *auto_image_direction_b;
 	GtkCheckButton *no_change_mtime_b;
 	GtkCheckButton *interpolate_segments_b;
+	GtkCheckButton *time_is_local_b;
 	GtkEntry *time_zone_b; // TODO consider a more user friendly tz widget eg libtimezonemap or similar
 	GtkEntry *time_offset_b;
 } GeoTagWidgets;
@@ -123,9 +122,11 @@ typedef struct {
 	gboolean overwrite_waypoints;
 	gboolean write_exif;
 	gboolean overwrite_gps_exif;
+	gboolean auto_image_direction;
 	gboolean no_change_mtime;
 	gboolean interpolate_segments;
 	gint time_offset;
+	gboolean time_is_local;
 	gint TimeZoneHours;
 	gint TimeZoneMins;
 } option_values_t;
@@ -143,6 +144,7 @@ typedef struct {
 	gboolean found_match;
 	VikCoord coord;
 	gdouble altitude;
+	gdouble image_direction;
 	// If anything has changed
 	gboolean redraw;
 } geotag_options_t;
@@ -151,11 +153,13 @@ typedef struct {
 #define VIK_SETTINGS_GEOTAG_OVERWRITE_WAYPOINTS  "geotag_overwrite_waypoints"
 #define VIK_SETTINGS_GEOTAG_WRITE_EXIF           "geotag_write_exif"
 #define VIK_SETTINGS_GEOTAG_OVERWRITE_GPS_EXIF   "geotag_overwrite_gps"
+#define VIK_SETTINGS_GEOTAG_AUTO_IMAGE_DIRECTION "geotag_auto_image_direction"
 #define VIK_SETTINGS_GEOTAG_NO_CHANGE_MTIME      "geotag_no_change_mtime"
 #define VIK_SETTINGS_GEOTAG_INTERPOLATE_SEGMENTS "geotag_interpolate_segments"
 #define VIK_SETTINGS_GEOTAG_TIME_OFFSET          "geotag_time_offset"
 #define VIK_SETTINGS_GEOTAG_TIME_OFFSET_HOURS    "geotag_time_offset_hours"
 #define VIK_SETTINGS_GEOTAG_TIME_OFFSET_MINS     "geotag_time_offset_mins"
+#define VIK_SETTINGS_GEOTAG_TIME_IS_LOCAL        "geotag_time_is_local"
 
 static void save_default_values ( option_values_t default_values )
 {
@@ -163,9 +167,11 @@ static void save_default_values ( option_values_t default_values )
 	a_settings_set_boolean ( VIK_SETTINGS_GEOTAG_OVERWRITE_WAYPOINTS, default_values.overwrite_waypoints );
 	a_settings_set_boolean ( VIK_SETTINGS_GEOTAG_WRITE_EXIF, default_values.write_exif );
 	a_settings_set_boolean ( VIK_SETTINGS_GEOTAG_OVERWRITE_GPS_EXIF, default_values.overwrite_gps_exif );
+	a_settings_set_boolean ( VIK_SETTINGS_GEOTAG_AUTO_IMAGE_DIRECTION, default_values.auto_image_direction );
 	a_settings_set_boolean ( VIK_SETTINGS_GEOTAG_NO_CHANGE_MTIME, default_values.no_change_mtime );
 	a_settings_set_boolean ( VIK_SETTINGS_GEOTAG_INTERPOLATE_SEGMENTS, default_values.interpolate_segments );
 	a_settings_set_integer ( VIK_SETTINGS_GEOTAG_TIME_OFFSET, default_values.time_offset );
+	a_settings_set_boolean ( VIK_SETTINGS_GEOTAG_TIME_IS_LOCAL, default_values.time_is_local );
 	a_settings_set_integer ( VIK_SETTINGS_GEOTAG_TIME_OFFSET_HOURS, default_values.TimeZoneHours );
 	a_settings_set_integer ( VIK_SETTINGS_GEOTAG_TIME_OFFSET_MINS, default_values.TimeZoneMins );
 }
@@ -181,12 +187,16 @@ static option_values_t get_default_values ( )
 		default_values.write_exif = TRUE;
 	if ( ! a_settings_get_boolean ( VIK_SETTINGS_GEOTAG_OVERWRITE_GPS_EXIF, &default_values.overwrite_gps_exif ) )
 		default_values.overwrite_gps_exif = FALSE;
+	if ( ! a_settings_get_boolean ( VIK_SETTINGS_GEOTAG_AUTO_IMAGE_DIRECTION, &default_values.auto_image_direction ) )
+		default_values.auto_image_direction = FALSE;
 	if ( ! a_settings_get_boolean ( VIK_SETTINGS_GEOTAG_NO_CHANGE_MTIME, &default_values.no_change_mtime ) )
 		default_values.no_change_mtime = TRUE;
 	if ( ! a_settings_get_boolean ( VIK_SETTINGS_GEOTAG_INTERPOLATE_SEGMENTS, &default_values.interpolate_segments ) )
 		default_values.interpolate_segments = TRUE;
 	if ( ! a_settings_get_integer ( VIK_SETTINGS_GEOTAG_TIME_OFFSET, &default_values.time_offset ) )
 		default_values.time_offset = 0;
+	if ( ! a_settings_get_boolean ( VIK_SETTINGS_GEOTAG_TIME_IS_LOCAL, &default_values.time_is_local ) )
+		default_values.time_is_local = FALSE;
 	if ( ! a_settings_get_integer ( VIK_SETTINGS_GEOTAG_TIME_OFFSET_HOURS, &default_values.TimeZoneHours ) )
 		default_values.TimeZoneHours = 0;
 	if ( ! a_settings_get_integer ( VIK_SETTINGS_GEOTAG_TIME_OFFSET_MINS, &default_values.TimeZoneMins ) )
@@ -194,6 +204,24 @@ static option_values_t get_default_values ( )
 	return default_values;
 }
 
+/**
+ * Get a heading from a single trkpoint
+ *
+ * Try using the adjacent trkpoints to get a direction
+ */
+static gdouble get_heading_from_trackpoint ( GList *mytrkpt )
+{
+	VikTrackpoint *trkpt = VIK_TRACKPOINT(mytrkpt->data);
+
+	if ( mytrkpt->prev )
+		return vik_coord_angle ( &VIK_TRACKPOINT(mytrkpt->prev->data)->coord, &trkpt->coord );
+	else if ( mytrkpt->next )
+		return vik_coord_angle ( &trkpt->coord, &VIK_TRACKPOINT(mytrkpt->prev->data)->coord );
+
+	// In the unlikely event of a single trackpoint track - can't guess a direction
+	return NAN;
+}
+
 /**
  * Correlate the image against the specified track
  */
@@ -213,11 +241,17 @@ static void trw_layer_geotag_track ( const gpointer id, VikTrack *track, geotag_
 
 		trkpt = VIK_TRACKPOINT(mytrkpt->data);
 
+		if ( isnan(trkpt->timestamp) ) {
+			continue;
+		}
+
 		// is it exactly this point?
 		if ( options->PhotoTime == trkpt->timestamp ) {
 			options->coord = trkpt->coord;
 			options->altitude = trkpt->altitude;
 			options->found_match = TRUE;
+			if ( options->ov.auto_image_direction )
+				options->image_direction = get_heading_from_trackpoint ( mytrkpt );
 			break;
 		}
 
@@ -225,7 +259,7 @@ static void trw_layer_geotag_track ( const gpointer id, VikTrack *track, geotag_
 		if ( !mytrkpt->next ) break;
 		trkpt_next = VIK_TRACKPOINT(mytrkpt->next->data);
 
-		// TODO need to use 'has_timestamp' property
+		if ( isnan(trkpt_next->timestamp) ) continue;
 		if ( trkpt->timestamp == trkpt_next->timestamp ) continue;
 		if ( trkpt->timestamp > trkpt_next->timestamp ) continue;
 
@@ -247,8 +281,10 @@ static void trw_layer_geotag_track ( const gpointer id, VikTrack *track, geotag_
 			 * in time between the two points. Ie, a number between 0 and 1 -
 			 * 0 is the first point, 1 is the next point, and 0.5 would be
 			 * half way. */
-			gdouble scale = (gdouble)trkpt_next->timestamp - (gdouble)trkpt->timestamp;
-			scale = ((gdouble)options->PhotoTime - (gdouble)trkpt->timestamp) / scale;
+			gdouble tdiff = (gdouble)trkpt_next->timestamp - (gdouble)trkpt->timestamp;
+			gdouble scale = ((gdouble)options->PhotoTime - (gdouble)trkpt->timestamp) / tdiff;
+
+			options->PhotoTime = options->PhotoTime + (time_t)(tdiff * scale);
 
 			struct LatLon ll_result, ll1, ll2;
 
@@ -265,6 +301,10 @@ static void trw_layer_geotag_track ( const gpointer id, VikTrack *track, geotag_
 
 			// Interpolate elevation
 			options->altitude = trkpt->altitude + ((trkpt_next->altitude - trkpt->altitude) * scale);
+
+			if ( options->ov.auto_image_direction )
+				options->image_direction = vik_coord_angle ( &trkpt->coord, &trkpt_next->coord );
+
 			break;
 		}
 	}
@@ -281,7 +321,9 @@ static void trw_layer_geotag_waypoint ( geotag_options_t *options )
 		gchar* datetime = a_geotag_get_exif_date_from_file ( options->image, &has_gps_exif );
 		// If image already has gps info - don't attempt to change it unless forced
 		if ( options->ov.overwrite_gps_exif || !has_gps_exif ) {
-			gint ans = a_geotag_write_exif_gps ( options->image, options->wpt->coord, options->wpt->altitude, options->ov.no_change_mtime );
+			gint ans = a_geotag_write_exif_gps ( options->image, options->wpt->coord, options->wpt->altitude,
+			                                     options->wpt->image_direction, options->wpt->image_direction_ref,
+			                                     options->ov.no_change_mtime );
 			if ( ans != 0 ) {
 				gchar *message = g_strdup_printf ( _("Failed updating EXIF on %s"), options->image );
 				vik_window_statusbar_update ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(options->vtl)), message, VIK_STATUSBAR_INFO );
@@ -333,7 +375,7 @@ static void trw_layer_geotag_process ( geotag_options_t *options )
 					VikWaypoint *current_wp = vik_trw_layer_get_waypoint ( options->vtl, name );
 					if ( current_wp ) {
 						// Existing wp found, so set new position, comment and image
-						current_wp = a_geotag_waypoint_positioned ( options->image, wp->coord, wp->altitude, &name, current_wp );
+						(void)a_geotag_waypoint_positioned ( options->image, wp->coord, wp->altitude, &name, current_wp );
 						updated_waypoint = TRUE;
 					}
 				}
@@ -351,13 +393,14 @@ static void trw_layer_geotag_process ( geotag_options_t *options )
 			return;
 		}
 
-		options->PhotoTime = ConvertToUnixTime ( datetime, EXIF_DATE_FORMAT, options->ov.TimeZoneHours, options->ov.TimeZoneMins);
+		options->PhotoTime = ConvertToUnixTime ( datetime, EXIF_DATE_FORMAT, options->ov.TimeZoneHours, options->ov.TimeZoneMins, options->ov.time_is_local );
 		g_free ( datetime );
 		
 		// Apply any offset
 		options->PhotoTime = options->PhotoTime + options->ov.time_offset;
 
 		options->found_match = FALSE;
+		options->image_direction = NAN;
 
 		if ( options->track ) {
 			// Single specified track
@@ -388,7 +431,10 @@ static void trw_layer_geotag_process ( geotag_options_t *options )
 					VikWaypoint *wp = vik_trw_layer_get_waypoint ( options->vtl, name );
 					if ( wp ) {
 						// Found, so set new position, comment and image
-						wp = a_geotag_waypoint_positioned ( options->image, options->coord, options->altitude, &name, wp );
+						(void)a_geotag_waypoint_positioned ( options->image, options->coord, options->altitude, &name, wp );
+						wp->image_direction_ref = WP_IMAGE_DIRECTION_REF_TRUE;
+						wp->image_direction = options->image_direction;
+						wp->timestamp = options->PhotoTime;
 						updated_waypoint = TRUE;
 					}
 					g_free ( name );
@@ -400,6 +446,9 @@ static void trw_layer_geotag_process ( geotag_options_t *options )
 					VikWaypoint *wp = a_geotag_waypoint_positioned ( options->image, options->coord, options->altitude, &name, NULL );
 					if ( !name )
 						name = g_strdup ( a_file_basename ( options->image ) );
+					wp->image_direction_ref = WP_IMAGE_DIRECTION_REF_TRUE;
+					wp->image_direction = options->image_direction;
+					wp->timestamp = options->PhotoTime;
 					vik_trw_layer_filein_add_waypoint ( options->vtl, name, wp );
 					g_free ( name );
 				}
@@ -410,7 +459,9 @@ static void trw_layer_geotag_process ( geotag_options_t *options )
 
 			// Write EXIF if specified
 			if ( options->ov.write_exif ) {
-				gint ans = a_geotag_write_exif_gps ( options->image, options->coord, options->altitude, options->ov.no_change_mtime );
+				gint ans = a_geotag_write_exif_gps ( options->image, options->coord, options->altitude,
+				                                     options->image_direction, WP_IMAGE_DIRECTION_REF_TRUE,
+				                                     options->ov.no_change_mtime );
 				if ( ans != 0 ) {
 					gchar *message = g_strdup_printf ( _("Failed updating EXIF on %s"), options->image );
 					vik_window_statusbar_update ( VIK_WINDOW(VIK_GTK_WINDOW_FROM_LAYER(options->vtl)), message, VIK_STATUSBAR_INFO );
@@ -456,7 +507,7 @@ static int trw_layer_geotag_thread ( geotag_options_t *options, gpointer threadd
 		if ( IS_VIK_LAYER(options->vtl) ) {
 			trw_layer_calculate_bounds_waypoints ( options->vtl );
 			// Ensure any new images get shown
-			trw_layer_verify_thumbnails ( options->vtl, NULL ); // NB second parameter not used ATM
+			trw_layer_verify_thumbnails ( options->vtl );
 			// Force redraw as verify only redraws if there are new thumbnails (they may already exist)
 			vik_layer_emit_update ( VIK_LAYER(options->vtl) ); // NB Update from background
 		}
@@ -486,8 +537,10 @@ static void trw_layer_geotag_response_cb ( GtkDialog *dialog, gint resp, GeoTagW
 		options->ov.overwrite_waypoints = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(widgets->overwrite_waypoints_b) );
 		options->ov.write_exif = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(widgets->write_exif_b) );
 		options->ov.overwrite_gps_exif = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(widgets->overwrite_gps_exif_b) );
+		options->ov.auto_image_direction = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(widgets->auto_image_direction_b) );
 		options->ov.no_change_mtime = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(widgets->no_change_mtime_b) );
 		options->ov.interpolate_segments = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(widgets->interpolate_segments_b) );
+		options->ov.time_is_local = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(widgets->time_is_local_b) );
 		options->ov.TimeZoneHours = 0;
 		options->ov.TimeZoneMins = 0;
 		const gchar* TZString = gtk_entry_get_text(GTK_ENTRY(widgets->time_zone_b));
@@ -603,9 +656,11 @@ void trw_layer_geotag_dialog ( GtkWindow *parent,
 	widgets->overwrite_gps_exif_b = GTK_CHECK_BUTTON ( gtk_check_button_new () );
 	widgets->no_change_mtime_l = GTK_LABEL ( gtk_label_new ( _("Keep File Modification Timestamp:") ) );
 	widgets->no_change_mtime_b = GTK_CHECK_BUTTON ( gtk_check_button_new () );
+	widgets->auto_image_direction_b = GTK_CHECK_BUTTON ( gtk_check_button_new () );
 	widgets->interpolate_segments_b = GTK_CHECK_BUTTON ( gtk_check_button_new () );
 	widgets->time_zone_b = GTK_ENTRY ( gtk_entry_new () );
 	widgets->time_offset_b = GTK_ENTRY ( gtk_entry_new () );
+	widgets->time_is_local_b = GTK_CHECK_BUTTON ( gtk_check_button_new () );
 
 	gtk_entry_set_width_chars ( widgets->time_zone_b, 7);
 	gtk_entry_set_width_chars ( widgets->time_offset_b, 7);
@@ -617,6 +672,7 @@ void trw_layer_geotag_dialog ( GtkWindow *parent,
 	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(widgets->overwrite_waypoints_b), default_values.overwrite_waypoints );
 	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(widgets->write_exif_b), default_values.write_exif );
 	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(widgets->overwrite_gps_exif_b), default_values.overwrite_gps_exif );
+	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(widgets->auto_image_direction_b), default_values.auto_image_direction );
 	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(widgets->no_change_mtime_b), default_values.no_change_mtime );
 	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(widgets->interpolate_segments_b), default_values.interpolate_segments );
 	gchar tmp_string[7];
@@ -625,6 +681,8 @@ void trw_layer_geotag_dialog ( GtkWindow *parent,
 	snprintf (tmp_string, 7, "%d", default_values.time_offset );
 	gtk_entry_set_text ( widgets->time_offset_b, tmp_string );
 
+	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(widgets->time_is_local_b), default_values.time_is_local );
+
 	// Ensure sensitivities setup
 	write_exif_b_cb ( GTK_WIDGET(widgets->write_exif_b), widgets );
 	g_signal_connect ( G_OBJECT(widgets->write_exif_b), "toggled", G_CALLBACK(write_exif_b_cb), widgets );
@@ -653,6 +711,11 @@ void trw_layer_geotag_dialog ( GtkWindow *parent,
 	gtk_box_pack_start ( GTK_BOX(fm_hbox), GTK_WIDGET(widgets->no_change_mtime_l), FALSE, FALSE, 5 );
 	gtk_box_pack_start ( GTK_BOX(fm_hbox), GTK_WIDGET(widgets->no_change_mtime_b), FALSE, FALSE, 5 );
 
+	GtkWidget *aid_hbox = gtk_hbox_new ( FALSE, 0 );
+	gtk_box_pack_start ( GTK_BOX(aid_hbox), gtk_label_new ( _("Automatic Image Direction:") ), FALSE, FALSE, 5 );
+	gtk_box_pack_start ( GTK_BOX(aid_hbox), GTK_WIDGET(widgets->auto_image_direction_b), FALSE, FALSE, 5 );
+	gtk_widget_set_tooltip_text ( GTK_WIDGET(widgets->auto_image_direction_b), _("Set image direction based on a heading derived from the track") );
+
 	GtkWidget *is_hbox = gtk_hbox_new ( FALSE, 0 );
 	GtkWidget *interpolate_segments_l = gtk_label_new ( _("Interpolate Between Track Segments:") );
 	gtk_box_pack_start ( GTK_BOX(is_hbox), interpolate_segments_l, FALSE, FALSE, 5 );
@@ -664,11 +727,17 @@ void trw_layer_geotag_dialog ( GtkWindow *parent,
 	gtk_box_pack_start ( GTK_BOX(to_hbox), GTK_WIDGET(widgets->time_offset_b), FALSE, FALSE, 5 );
 	gtk_widget_set_tooltip_text ( GTK_WIDGET(widgets->time_offset_b), _("The number of seconds to ADD to the photos time to make it match the GPS data. Calculate this with (GPS - Photo). Can be negative or positive. Useful to adjust times when a camera's timestamp was incorrect.") );
 
+	GtkWidget *tl_hbox = gtk_hbox_new ( FALSE, 0 );
+	GtkWidget *time_is_local_l = gtk_label_new ( _("Image Time is Local:") );
+	gtk_box_pack_start ( GTK_BOX(tl_hbox), time_is_local_l, FALSE, FALSE, 5 );
+	gtk_box_pack_start ( GTK_BOX(tl_hbox), GTK_WIDGET(widgets->time_is_local_b), FALSE, FALSE, 5 );
+	gtk_widget_set_tooltip_text ( GTK_WIDGET(widgets->time_is_local_b), _("The timestamps in the images are in local time (rather than UTC) and so current local timezone adjustment will be automatically applied.") );
+
 	GtkWidget *tz_hbox = gtk_hbox_new ( FALSE, 0 );
 	GtkWidget *time_zone_l = gtk_label_new ( _("Image Timezone:") );
 	gtk_box_pack_start ( GTK_BOX(tz_hbox), time_zone_l, FALSE, FALSE, 5 );
 	gtk_box_pack_start ( GTK_BOX(tz_hbox), GTK_WIDGET(widgets->time_zone_b), FALSE, FALSE, 5 );
-	gtk_widget_set_tooltip_text ( GTK_WIDGET(widgets->time_zone_b), _("The timezone that was used when the images were created. For example, if a camera is set to AWST or +8:00 hours. Enter +8:00 here so that the correct adjustment to the images' time can be made. GPS data is always in UTC.") );
+	gtk_widget_set_tooltip_text ( GTK_WIDGET(widgets->time_zone_b), _("The timezone adjustment factor. e.g. for AWST use +8:00. This should be unnecessary if using 'time is local'. However time stamps may be offset due to timezone issues.") );
 
 	gchar *track_string = NULL;
 	if ( widgets->wpt ) {
@@ -684,6 +753,8 @@ void trw_layer_geotag_dialog ( GtkWindow *parent,
 		gtk_widget_set_sensitive ( GTK_WIDGET(time_offset_l), FALSE );
 		gtk_widget_set_sensitive ( GTK_WIDGET(widgets->time_zone_b), FALSE );
 		gtk_widget_set_sensitive ( GTK_WIDGET(time_zone_l), FALSE );
+		gtk_widget_set_sensitive ( GTK_WIDGET(time_is_local_l), FALSE );
+		gtk_widget_set_sensitive ( GTK_WIDGET(widgets->time_is_local_b), FALSE );
 	}
 	else if ( widgets->track )
 		track_string = g_strdup_printf ( _("Using track: %s"), track->name );
@@ -699,8 +770,10 @@ void trw_layer_geotag_dialog ( GtkWindow *parent,
 	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(widgets->dialog))), we_hbox,  FALSE, FALSE, 0);
 	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(widgets->dialog))), og_hbox,  FALSE, FALSE, 0);
 	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(widgets->dialog))), fm_hbox,  FALSE, FALSE, 0);
+	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(widgets->dialog))), aid_hbox, FALSE, FALSE, 0);
 	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(widgets->dialog))), is_hbox,  FALSE, FALSE, 0);
 	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(widgets->dialog))), to_hbox,  FALSE, FALSE, 0);
+	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(widgets->dialog))), tl_hbox,  FALSE, FALSE, 0);
 	gtk_box_pack_start ( GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(widgets->dialog))), tz_hbox,  FALSE, FALSE, 0);
 
 	g_signal_connect ( widgets->dialog, "response", G_CALLBACK(trw_layer_geotag_response_cb), widgets );
