@@ -2,7 +2,7 @@
 /*
  * viking -- GPS Data and Topo Analyzer, Explorer, and Manager
  *
- * Copyright (C) 2013, Rob Norris <rw_norris@hotmail.com>
+ * Copyright (C) 2013-2015, Rob Norris <rw_norris@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,17 +19,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include <math.h>
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <glib.h>
-#include <glib/gstdio.h>
-#include <glib/gi18n.h>
-
 #include "viking.h"
 #include "viktrwlayer_waypointlist.h"
 #include "viktrwlayer_wpwin.h"
+#include "dem.h"
 
 // Long formatted date+basic time - listing this way ensures the string comparison sort works - so no local type format %x or %c here!
 #define WAYPOINT_LIST_DATE_FORMAT "%Y-%m-%d %H:%M"
@@ -46,26 +39,6 @@ static void waypoint_close_cb ( GtkWidget *dialog, gint resp, GList *data )
 	gtk_widget_destroy (dialog);
 }
 
-/**
- * format_1f_cell_data_func:
- *
- * General purpose column double formatting
- *
-static void format_1f_cell_data_func ( GtkTreeViewColumn *col,
-                                       GtkCellRenderer   *renderer,
-                                       GtkTreeModel      *model,
-                                       GtkTreeIter       *iter,
-                                       gpointer           user_data )
-{
-	gdouble value;
-	gchar buf[20];
-	gint column = GPOINTER_TO_INT (user_data);
-	gtk_tree_model_get ( model, iter, column, &value, -1 );
-	g_snprintf ( buf, sizeof(buf), "%.1f", value );
-	g_object_set ( renderer, "text", buf, NULL );
-}
- */
-
 #define WPT_LIST_COLS 9
 #define WPT_COL_NUM WPT_LIST_COLS-1
 #define TRW_COL_NUM WPT_COL_NUM-1
@@ -221,9 +194,16 @@ static void trw_layer_show_picture ( menu_array_values values )
 
 typedef struct {
   gboolean has_layer_names;
+  gboolean include_positions;
   GString *str;
 } copy_data_t;
 
+/**
+ * At the moment allow copying the data displayed** with or without the positions
+ *  (since the position data is not shown in the list but is useful in copying to external apps)
+ *
+ * ** ATM The visibility flag is not copied and neither is a text representation of the waypoint symbol
+ */
 static void copy_selection (GtkTreeModel *model,
                             GtkTreePath *path,
                             GtkTreeIter *iter,
@@ -238,20 +218,35 @@ static void copy_selection (GtkTreeModel *model,
 	if ( comment == NULL )
 		comment = g_strdup ( "" );
 	gint hh; gtk_tree_model_get ( model, iter, 5, &hh, -1 );
+
+	VikWaypoint *wpt; gtk_tree_model_get ( model, iter, WPT_COL_NUM, &wpt, -1 );
+	struct LatLon ll;
+	if ( wpt ) {
+		vik_coord_to_latlon ( &wpt->coord, &ll );
+	}
 	gchar sep = '\t'; // Could make this configurable - but simply always make it a tab character for now
 	// NB Even if the columns have been reordered - this copies it out only in the original default order
 	// if col 0 is displayed then also copy the layername
-	if ( cd->has_layer_names )
-		g_string_append_printf ( cd->str, "%s%c%s%c%s%c%s%c%d\n", layername, sep, name, sep, date, sep, comment, sep, hh );
-	else
-		g_string_append_printf ( cd->str, "%s%c%s%c%s%c%d\n", name, sep, date, sep, comment, sep, hh );
+	// Note that the lat/lon data copy is using the users locale
+	if ( cd->has_layer_names ) {
+		if ( cd->include_positions )
+			g_string_append_printf ( cd->str, "%s%c%s%c%s%c%s%c%d%c%.6f%c%.6f\n", layername, sep, name, sep, date, sep, comment, sep, hh, sep, ll.lat, sep, ll.lon );
+		else
+			g_string_append_printf ( cd->str, "%s%c%s%c%s%c%s%c%d\n", layername, sep, name, sep, date, sep, comment, sep, hh );
+	}
+	else {
+		if ( cd->include_positions )
+			g_string_append_printf ( cd->str, "%s%c%s%c%s%c%d%c%.6f%c%.6f\n", name, sep, date, sep, comment, sep, hh, sep, ll.lat, sep, ll.lon );
+		else
+			g_string_append_printf ( cd->str, "%s%c%s%c%s%c%d\n", name, sep, date, sep, comment, sep, hh );
+	}
 	g_free ( layername );
 	g_free ( name );
 	g_free ( date );
 	g_free ( comment );
 }
 
-static void trw_layer_copy_selected ( GtkWidget *tree_view )
+static void trw_layer_copy_selected ( GtkWidget *tree_view, gboolean include_positions )
 {
 	GtkTreeSelection *selection = gtk_tree_view_get_selection ( GTK_TREE_VIEW(tree_view) );
 	// NB GTK3 has gtk_tree_view_get_n_columns() but we're GTK2 ATM
@@ -261,6 +256,7 @@ static void trw_layer_copy_selected ( GtkWidget *tree_view )
 	copy_data_t cd;
 	cd.has_layer_names = (count > WPT_LIST_COLS-3);
 	cd.str = g_string_new ( NULL );
+	cd.include_positions = include_positions;
 	gtk_tree_selection_selected_foreach ( selection, copy_selection, &cd );
 
 	a_clipboard_copy ( VIK_CLIPBOARD_DATA_TEXT, 0, 0, 0, cd.str->str, NULL );
@@ -268,20 +264,27 @@ static void trw_layer_copy_selected ( GtkWidget *tree_view )
 	g_string_free ( cd.str, TRUE );
 }
 
-static void add_copy_menu_item ( GtkMenu *menu, GtkWidget *tree_view )
+static void trw_layer_copy_selected_only_visible_columns ( GtkWidget *tree_view )
 {
-	GtkWidget *item = gtk_image_menu_item_new_with_mnemonic ( _("_Copy Data") );
-	gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_COPY, GTK_ICON_SIZE_MENU) );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(trw_layer_copy_selected), tree_view );
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show ( item );
+	trw_layer_copy_selected ( tree_view, FALSE );
+}
+
+static void trw_layer_copy_selected_with_position ( GtkWidget *tree_view )
+{
+	trw_layer_copy_selected ( tree_view, TRUE );
+}
+
+static void add_copy_menu_items ( GtkMenu *menu, GtkWidget *tree_view )
+{
+	(void)vu_menu_add_item ( menu, _("_Copy Data"), GTK_STOCK_COPY,
+	                         G_CALLBACK(trw_layer_copy_selected_only_visible_columns), tree_view );
+	(void)vu_menu_add_item ( menu, _("_Copy Data (with _positions)"), GTK_STOCK_COPY,
+	                         G_CALLBACK(trw_layer_copy_selected_with_position), tree_view );
 }
 
 static gboolean add_menu_items ( GtkMenu *menu, VikTrwLayer *vtl, VikWaypoint *wpt, gpointer wpt_uuid, VikViewport *vvp, GtkWidget *tree_view, gpointer data )
 {
 	static menu_array_values values;
-	GtkWidget *item;
-
 	values[MA_VTL]       = vtl;
 	values[MA_WPT]       = wpt;
 	values[MA_WPT_UUID]  = wpt_uuid;
@@ -289,35 +292,17 @@ static gboolean add_menu_items ( GtkMenu *menu, VikTrwLayer *vtl, VikWaypoint *w
 	values[MA_TREEVIEW]  = tree_view;
 	values[MA_WPTS_LIST] = data;
 
-	/*
-	item = gtk_image_menu_item_new_with_mnemonic ( _("_Select") );
-	gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_FIND, GTK_ICON_SIZE_MENU) );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(trw_layer_waypoint_select), values );
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show ( item );
-	*/
-
+	//(void)vu_menu_add_item ( menu, _("_Select"), GTK_STOCK_FIND, G_CALLBACK(trw_layer_waypoint_select), values );
 	// AUTO SELECT NOT TRUE YET...
 	// ATM view auto selects, so don't bother with separate select menu entry
-	item = gtk_image_menu_item_new_with_mnemonic ( _("_View") );
-	gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock (GTK_STOCK_ZOOM_FIT, GTK_ICON_SIZE_MENU) );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(trw_layer_waypoint_view), values );
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show ( item );
 
-	item = gtk_image_menu_item_new_from_stock ( GTK_STOCK_PROPERTIES, NULL );
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(trw_layer_waypoint_properties), values );
-	gtk_menu_shell_append ( GTK_MENU_SHELL(menu), item );
-	gtk_widget_show ( item );
-
-	item = gtk_image_menu_item_new_with_mnemonic ( _("_Show Picture...") );
-	gtk_image_menu_item_set_image ( (GtkImageMenuItem*)item, gtk_image_new_from_stock ("vik-icon-Show Picture", GTK_ICON_SIZE_MENU) ); // Own icon - see stock_icons in vikwindow.c
-	g_signal_connect_swapped ( G_OBJECT(item), "activate", G_CALLBACK(trw_layer_show_picture), values );
-	gtk_menu_shell_append ( GTK_MENU_SHELL(menu), item );
-	gtk_widget_show ( item );
+	(void)vu_menu_add_item ( menu, _("_View"), GTK_STOCK_ZOOM_FIT, G_CALLBACK(trw_layer_waypoint_view), values );
+	(void)vu_menu_add_item ( menu, NULL, GTK_STOCK_PROPERTIES, G_CALLBACK(trw_layer_waypoint_properties), values );
+	// Own icon - see stock_icons in vikwindow.c
+	GtkWidget *item = vu_menu_add_item ( menu, _("_Show Picture..."), VIK_ICON_SHOW_PICTURE, G_CALLBACK(trw_layer_show_picture), values );
 	gtk_widget_set_sensitive ( item, GPOINTER_TO_INT(wpt->image) );
 
-	add_copy_menu_item ( menu, tree_view );
+	add_copy_menu_items ( menu, tree_view );
 
 	return TRUE;
 }
@@ -328,8 +313,8 @@ static gboolean trw_layer_waypoint_menu_popup_multi  ( GtkWidget *tree_view,
 {
 	GtkWidget *menu = gtk_menu_new();
 
-	add_copy_menu_item ( GTK_MENU(menu), tree_view );
-
+	add_copy_menu_items ( GTK_MENU(menu), tree_view );
+	gtk_widget_show_all ( menu );
 	gtk_menu_popup ( GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, gtk_get_current_event_time() );
 
 	return TRUE;
@@ -373,7 +358,7 @@ static gboolean trw_layer_waypoint_menu_popup ( GtkWidget *tree_view,
 	udataU.wp   = wpt;
 	udataU.uuid = NULL;
 
-	gpointer *wptf;
+	gpointer wptf;
 	wptf = g_hash_table_find ( vik_trw_layer_get_waypoints(vtl), (GHRFunc) trw_layer_waypoint_find_uuid, &udataU );
 
 	if ( wptf && udataU.uuid ) {
@@ -393,6 +378,7 @@ static gboolean trw_layer_waypoint_menu_popup ( GtkWidget *tree_view,
 		                 tree_view,
 		                 data );
 
+		gtk_widget_show_all ( menu );
 		gtk_menu_popup ( GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, gtk_get_current_event_time() );
 		return TRUE;
 	}
@@ -430,7 +416,8 @@ static gboolean trw_layer_waypoint_button_pressed ( GtkWidget *tree_view,
  */
 static void trw_layer_waypoint_list_add ( vik_trw_waypoint_list_t *vtdl,
                                           GtkTreeStore *store,
-                                          vik_units_height_t height_units )
+                                          vik_units_height_t height_units,
+                                          const gchar* date_format )
 {
 	GtkTreeIter t_iter;
 	VikWaypoint *wpt = vtdl->wpt;
@@ -439,18 +426,18 @@ static void trw_layer_waypoint_list_add ( vik_trw_waypoint_list_t *vtdl,
 	// Get start date
 	gchar time_buf[32];
 	time_buf[0] = '\0';
-	if ( wpt->has_timestamp ) {
+	if ( !isnan(wpt->timestamp) ) {
 
 #if GLIB_CHECK_VERSION(2,26,0)
 		GDateTime* gdt = g_date_time_new_from_unix_utc ( wpt->timestamp );
-		gchar *time = g_date_time_format ( gdt, WAYPOINT_LIST_DATE_FORMAT );
-		strncpy ( time_buf, time, sizeof(time_buf) );
+		gchar *time = g_date_time_format ( gdt, date_format );
+		g_strlcpy ( time_buf, time, sizeof(time_buf) );
 		g_free ( time );
 		g_date_time_unref ( gdt);
 #else
 		GDate* gdate_start = g_date_new ();
 		g_date_set_time_t ( gdate_start, wpt->timestamp );
-		g_date_strftime ( time_buf, sizeof(time_buf), WAYPOINT_LIST_DATE_FORMAT, gdate_start );
+		g_date_strftime ( time_buf, sizeof(time_buf), date_format, gdate_start );
 		g_date_free ( gdate_start );
 #endif
 	}
@@ -460,11 +447,15 @@ static void trw_layer_waypoint_list_add ( vik_trw_waypoint_list_t *vtdl,
 	visible = visible && vik_trw_layer_get_waypoints_visibility(vtl);
 
 	gdouble alt = wpt->altitude;
-	switch (height_units) {
-	case VIK_UNITS_HEIGHT_FEET: alt = VIK_METERS_TO_FEET(alt); break;
-	default:
-		// VIK_UNITS_HEIGHT_METRES: no need to convert
-		break;
+	if ( isnan(alt) ) {
+		alt = VIK_DEM_INVALID_ELEVATION;
+	} else {
+		switch (height_units) {
+		case VIK_UNITS_HEIGHT_FEET: alt = VIK_METERS_TO_FEET(alt); break;
+		default:
+			// VIK_UNITS_HEIGHT_METRES: no need to convert
+			break;
+		}
 	}
 
 	gtk_tree_store_append ( store, &t_iter, NULL );
@@ -499,14 +490,27 @@ gint sort_pixbuf_compare_func ( GtkTreeModel *model,
 	return g_strcmp0 ( wpt1->symbol, wpt2->symbol );
 }
 
-static GtkTreeViewColumn *my_new_column_text ( const gchar *title, GtkCellRenderer *renderer, GtkWidget *view, gint column_runner )
+/**
+ * format_elev_cell_data_func
+ *
+ * Integer display handling invalid/undefined elevation values
+ *
+ */
+static void format_elev_cell_data_func ( GtkTreeViewColumn *col,
+                                         GtkCellRenderer   *renderer,
+                                         GtkTreeModel      *model,
+                                         GtkTreeIter       *iter,
+                                         gpointer           user_data )
 {
-	GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes ( title, renderer, "text", column_runner, NULL );
-	gtk_tree_view_column_set_sort_column_id ( column, column_runner );
-	gtk_tree_view_append_column ( GTK_TREE_VIEW(view), column );
-	gtk_tree_view_column_set_reorderable ( column, TRUE );
-	gtk_tree_view_column_set_resizable ( column, TRUE );
-	return column;
+	gint value;
+	gchar buf[20];
+	gint column = GPOINTER_TO_INT(user_data);
+	gtk_tree_model_get ( model, iter, column, &value, -1 );
+	if ( value == VIK_DEM_INVALID_ELEVATION )
+		g_snprintf ( buf, sizeof(buf), "--" );
+	else
+		g_snprintf ( buf, sizeof(buf), "%d", value );
+	g_object_set ( renderer, "text", buf, NULL );
 }
 
 /**
@@ -546,11 +550,16 @@ static void vik_trw_layer_waypoint_list_internal ( GtkWidget *dialog,
 
 	//GList *gl = get_waypoints_and_layers_cb ( vl, user_data );
 	//g_list_foreach ( waypoints_and_layers, (GFunc) trw_layer_waypoint_list_add, store );
+	gchar *date_format = NULL;
+	if ( !a_settings_get_string ( VIK_SETTINGS_LIST_DATE_FORMAT, &date_format ) )
+		date_format = g_strdup ( WAYPOINT_LIST_DATE_FORMAT );
+
 	GList *gl = waypoints_and_layers;
 	while ( gl ) {
-		trw_layer_waypoint_list_add ( (vik_trw_waypoint_list_t*)gl->data, store, height_units );
+		trw_layer_waypoint_list_add ( (vik_trw_waypoint_list_t*)gl->data, store, height_units, date_format );
 		gl = g_list_next ( gl );
 	}
+	g_free ( date_format );
 
 	GtkWidget *view = gtk_tree_view_new();
 	GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
@@ -561,7 +570,7 @@ static void vik_trw_layer_waypoint_list_internal ( GtkWidget *dialog,
 	gint column_runner = 0;
 	if ( show_layer_names ) {
 		// Insert column for the layer name when viewing multi layers
-		column = my_new_column_text ( _("Layer"), renderer, view, column_runner++ );
+		column = ui_new_column_text ( _("Layer"), renderer, view, column_runner++ );
 		g_object_set (G_OBJECT (renderer), "xalign", 0.0, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 		gtk_tree_view_column_set_expand ( column, TRUE );
 		// remember the layer column so we can sort by it later
@@ -570,13 +579,13 @@ static void vik_trw_layer_waypoint_list_internal ( GtkWidget *dialog,
 	else
 		column_runner++;
 
-	column = my_new_column_text ( _("Name"), renderer, view, column_runner++ );
+	column = ui_new_column_text ( _("Name"), renderer, view, column_runner++ );
 	gtk_tree_view_column_set_expand ( column, TRUE );
 	if ( !show_layer_names )
 		// remember the name column so we can sort by it later
 		sort_by_column = column;
 
-	column = my_new_column_text ( _("Date"), renderer, view, column_runner++ );
+	column = ui_new_column_text ( _("Date"), renderer, view, column_runner++ );
 	gtk_tree_view_column_set_resizable ( column, TRUE );
 
 	GtkCellRenderer *renderer_toggle = gtk_cell_renderer_toggle_new ();
@@ -585,13 +594,16 @@ static void vik_trw_layer_waypoint_list_internal ( GtkWidget *dialog,
 	gtk_tree_view_append_column ( GTK_TREE_VIEW(view), column );
 	column_runner++;
 
-	column = my_new_column_text ( _("Comment"), renderer, view, column_runner++ );
+	column = ui_new_column_text ( _("Comment"), renderer, view, column_runner++ );
 	gtk_tree_view_column_set_expand ( column, TRUE );
 
 	if ( height_units == VIK_UNITS_HEIGHT_FEET )
-		column = my_new_column_text ( _("Max Height\n(Feet)"), renderer, view, column_runner++ );
+		column = ui_new_column_text ( _("Max Height\n(Feet)"), renderer, view, column_runner++ );
 	else
-		column = my_new_column_text ( _("Max Height\n(Metres)"), renderer, view, column_runner++ );
+		column = ui_new_column_text ( _("Max Height\n(Metres)"), renderer, view, column_runner++ );
+	// Apply own formatting of the data
+	gtk_tree_view_column_set_cell_data_func ( column, renderer, format_elev_cell_data_func, GINT_TO_POINTER(column_runner-1), NULL);
+
 
 	GtkCellRenderer *renderer_pixbuf = gtk_cell_renderer_pixbuf_new ();
 	g_object_set (G_OBJECT (renderer_pixbuf), "xalign", 0.5, NULL);
