@@ -2,7 +2,7 @@
  * viking -- GPS Data and Topo Analyzer, Explorer, and Manager
  *
  * Copyright (C) 2003-2005, Evan Battaglia <gtoevan@gmx.net>
- * Copyright (C) 2010-2015, Rob Norris <rw_norris@hotmail.com>
+ * Copyright (C) 2010-2018, Rob Norris <rw_norris@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,29 +19,28 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include <stdlib.h>
-#include <glib/gi18n.h>
-
 #include "viktrwlayer_wpwin.h"
 #include "degrees_converters.h"
 #include "garminsymbols.h"
-#ifdef HAVE_LIBEXIF
+#ifdef VIK_CONFIG_GEOTAG
 #include "geotag_exif.h"
 #endif
 #include "thumbnails.h"
 #include "viking.h"
 #include "vikdatetime_edit_dialog.h"
 #include "vikgoto.h"
-#include "vikutils.h"
 
 static void update_time ( GtkWidget *widget, VikWaypoint *wp )
 {
-  gchar *msg = vu_get_time_string ( &(wp->timestamp), "%c", &(wp->coord), NULL );
+  time_t tt = (time_t)wp->timestamp;
+  gchar *msg = vu_get_time_string ( &tt, "%c", &(wp->coord), NULL );
   gtk_button_set_label ( GTK_BUTTON(widget), msg );
   g_free ( msg );
 }
 
 static VikWaypoint *edit_wp;
+static gulong direction_signal_id;
+static gchar *last_sym = NULL;
 
 /**
  * time_edit_click:
@@ -60,14 +59,14 @@ static void time_edit_click ( GtkWidget* widget, GdkEventButton *event, VikWaypo
   }
 
   GTimeZone *gtz = g_time_zone_new_local ();
-  time_t mytime = vik_datetime_edit_dialog ( GTK_WINDOW(gtk_widget_get_toplevel(widget)),
-                                             _("Date/Time Edit"),
-                                             wp->timestamp,
-                                             gtz );
+  gdouble mytime = vik_datetime_edit_dialog ( GTK_WINDOW(gtk_widget_get_toplevel(widget)),
+                                              _("Date/Time Edit"),
+                                              wp->timestamp,
+                                              gtz );
   g_time_zone_unref ( gtz );
 
   // Was the dialog cancelled?
-  if ( mytime == 0 )
+  if ( isnan(mytime) )
     return;
 
   // Otherwise use new value in the edit buffer
@@ -80,6 +79,22 @@ static void time_edit_click ( GtkWidget* widget, GdkEventButton *event, VikWaypo
   update_time ( widget, edit_wp );
 }
 
+/**
+ * direction_edit_click:
+ */
+static void direction_add_click ( GtkWidget* widget, GdkEventButton *event, GtkWidget *direction )
+{
+  // Replace 'Add' with text and stop further callbacks
+  if ( gtk_button_get_image ( GTK_BUTTON(widget) ) )
+    gtk_button_set_image ( GTK_BUTTON(widget), NULL );
+  gtk_button_set_label ( GTK_BUTTON(widget), _("True") );
+  g_signal_handler_disconnect ( G_OBJECT(widget), direction_signal_id );
+
+  // Enable direction value
+  gtk_widget_set_sensitive ( direction, TRUE );
+  gtk_spin_button_set_value ( GTK_SPIN_BUTTON(direction), 0.0 );
+}
+
 static void symbol_entry_changed_cb(GtkWidget *combo, GtkListStore *store)
 {
   GtkTreeIter iter;
@@ -112,10 +127,14 @@ gchar *a_dialog_waypoint ( GtkWindow *parent, gchar *default_name, VikTrwLayer *
   struct LatLon ll;
   GtkWidget *latlabel, *lonlabel, *namelabel, *latentry, *lonentry, *altentry, *altlabel, *nameentry=NULL;
   GtkWidget *commentlabel, *commententry, *descriptionlabel, *descriptionentry, *imagelabel, *imageentry, *symbollabel, *symbolentry;
+  GtkWidget *sourcelabel = NULL, *sourceentry = NULL;
+  GtkWidget *typelabel = NULL, *typeentry = NULL;
   GtkWidget *timelabel = NULL;
   GtkWidget *timevaluebutton = NULL;
   GtkWidget *hasGeotagCB = NULL;
   GtkWidget *consistentGeotagCB = NULL;
+  GtkWidget *direction_sb = NULL;
+  GtkWidget *direction_hb = NULL;
   GtkListStore *store;
 
   gchar *lat, *lon, *alt;
@@ -125,16 +144,20 @@ gchar *a_dialog_waypoint ( GtkWindow *parent, gchar *default_name, VikTrwLayer *
   lat = g_strdup_printf ( "%f", ll.lat );
   lon = g_strdup_printf ( "%f", ll.lon );
   vik_units_height_t height_units = a_vik_get_units_height ();
-  switch (height_units) {
-  case VIK_UNITS_HEIGHT_METRES:
-    alt = g_strdup_printf ( "%f", wp->altitude );
-    break;
-  case VIK_UNITS_HEIGHT_FEET:
-    alt = g_strdup_printf ( "%f", VIK_METERS_TO_FEET(wp->altitude) );
-    break;
-  default:
-    alt = g_strdup_printf ( "%f", wp->altitude );
-    g_critical("Houston, we've had a problem. height=%d", height_units);
+  if ( isnan(wp->altitude) ) {
+    alt = NULL;
+  } else {
+    switch (height_units) {
+    case VIK_UNITS_HEIGHT_METRES:
+      alt = g_strdup_printf ( "%f", wp->altitude );
+      break;
+    case VIK_UNITS_HEIGHT_FEET:
+      alt = g_strdup_printf ( "%f", VIK_METERS_TO_FEET(wp->altitude) );
+      break;
+    default:
+      alt = g_strdup_printf ( "%f", wp->altitude );
+      g_critical("Houston, we've had a problem. height=%d", height_units);
+    }
   }
 
   *updated = FALSE;
@@ -142,32 +165,29 @@ gchar *a_dialog_waypoint ( GtkWindow *parent, gchar *default_name, VikTrwLayer *
   namelabel = gtk_label_new (_("Name:"));
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), namelabel, FALSE, FALSE, 0);
   // Name is now always changeable
-  nameentry = gtk_entry_new ();
+  nameentry = ui_entry_new ( NULL, GTK_ENTRY_ICON_SECONDARY );
   if ( default_name )
     gtk_entry_set_text( GTK_ENTRY(nameentry), default_name );
   g_signal_connect_swapped ( nameentry, "activate", G_CALLBACK(a_dialog_response_accept), GTK_DIALOG(dialog) );
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), nameentry, FALSE, FALSE, 0);
 
   latlabel = gtk_label_new (_("Latitude:"));
-  latentry = gtk_entry_new ();
-  gtk_entry_set_text ( GTK_ENTRY(latentry), lat );
+  latentry = ui_entry_new ( lat, GTK_ENTRY_ICON_SECONDARY );
   g_free ( lat );
 
   lonlabel = gtk_label_new (_("Longitude:"));
-  lonentry = gtk_entry_new ();
-  gtk_entry_set_text ( GTK_ENTRY(lonentry), lon );
+  lonentry = ui_entry_new ( lon, GTK_ENTRY_ICON_SECONDARY );
   g_free ( lon );
 
   altlabel = gtk_label_new (_("Altitude:"));
-  altentry = gtk_entry_new ();
-  gtk_entry_set_text ( GTK_ENTRY(altentry), alt );
+  altentry = ui_entry_new ( alt, GTK_ENTRY_ICON_SECONDARY );
   g_free ( alt );
 
   if ( wp->comment && !strncmp(wp->comment, "http", 4) )
     commentlabel = gtk_link_button_new_with_label (wp->comment, _("Comment:") );
   else
     commentlabel = gtk_label_new (_("Comment:"));
-  commententry = gtk_entry_new ();
+  commententry = ui_entry_new ( NULL, GTK_ENTRY_ICON_SECONDARY );
   gchar *cmt =  NULL;
   // Auto put in some kind of 'name' as a comment if one previously 'goto'ed this exact location
   cmt = a_vik_goto_get_search_string_for_this_place(VIK_WINDOW(parent));
@@ -178,7 +198,17 @@ gchar *a_dialog_waypoint ( GtkWindow *parent, gchar *default_name, VikTrwLayer *
     descriptionlabel = gtk_link_button_new_with_label (wp->description, _("Description:") );
   else
     descriptionlabel = gtk_label_new (_("Description:"));
-  descriptionentry = gtk_entry_new ();
+  descriptionentry = ui_entry_new ( NULL, GTK_ENTRY_ICON_SECONDARY );
+
+  sourcelabel = gtk_label_new (_("Source:"));
+  if ( wp->source ) {
+    sourceentry = ui_entry_new ( wp->source, GTK_ENTRY_ICON_SECONDARY );
+  }
+
+  typelabel = gtk_label_new (_("Type:"));
+  if ( wp->type ) {
+    typeentry = ui_entry_new ( wp->type, GTK_ENTRY_ICON_SECONDARY );
+  }
 
   imagelabel = gtk_label_new (_("Image:"));
   imageentry = vik_file_entry_new (GTK_FILE_CHOOSER_ACTION_OPEN, VF_FILTER_IMAGE, NULL, NULL);
@@ -205,12 +235,12 @@ gchar *a_dialog_waypoint ( GtkWindow *parent, gchar *default_name, VikTrwLayer *
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (symbolentry), r, FALSE);
     gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (symbolentry), r, "text", 2, NULL);
 
-    if ( !is_new && wp->symbol ) {
+    if ( wp->symbol || (is_new && last_sym) ) {
       gboolean ok;
       gchar *sym;
       for (ok = gtk_tree_model_get_iter_first ( GTK_TREE_MODEL(store), &iter ); ok; ok = gtk_tree_model_iter_next ( GTK_TREE_MODEL(store), &iter)) {
-	gtk_tree_model_get ( GTK_TREE_MODEL(store), &iter, 0, (void *)&sym, -1 );
-	if (sym && !strcmp(sym, wp->symbol)) {
+        gtk_tree_model_get ( GTK_TREE_MODEL(store), &iter, 0, (void *)&sym, -1 );
+        if (sym && (!g_strcmp0(sym, wp->symbol) || !g_strcmp0(sym, last_sym)) ) {
 	  g_free(sym);
 	  break;
 	} else {
@@ -231,10 +261,14 @@ gchar *a_dialog_waypoint ( GtkWindow *parent, gchar *default_name, VikTrwLayer *
   if ( !is_new && wp->description )
     gtk_entry_set_text ( GTK_ENTRY(descriptionentry), wp->description );
 
+  if ( !edit_wp )
+    edit_wp = vik_waypoint_new ();
+  edit_wp = vik_waypoint_copy ( wp );
+
   if ( !is_new && wp->image ) {
     vik_file_entry_set_filename ( VIK_FILE_ENTRY(imageentry), wp->image );
 
-#ifdef HAVE_LIBEXIF
+#ifdef VIK_CONFIG_GEOTAG
     // Geotag Info [readonly]
     hasGeotagCB = gtk_check_button_new_with_label ( _("Has Geotag") );
     gtk_widget_set_sensitive ( hasGeotagCB, FALSE );
@@ -249,8 +283,38 @@ gchar *a_dialog_waypoint ( GtkWindow *parent, gchar *default_name, VikTrwLayer *
       struct LatLon ll = a_geotag_get_position ( wp->image );
       VikCoord coord;
       vik_coord_load_from_latlon ( &coord, coord_mode, &ll );
-      gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(consistentGeotagCB), vik_coord_equals(&coord, &wp->coord) );
+      gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(consistentGeotagCB), vik_coord_equalish(&coord, &wp->coord) );
     }
+
+    // ATM the direction value box is always shown, even when there is no information.
+    // It would be nice to be able to hide it until the 'Add' has been performed,
+    //  however I've not been able to achieve this.
+    // Thus simply sensistizing it instead.
+    GtkWidget *direction_label = gtk_label_new ( _("Image Direction:") );
+    direction_hb = gtk_hbox_new ( FALSE, 0 );
+    gtk_box_pack_start (GTK_BOX(direction_hb), direction_label, FALSE, FALSE, 0);
+    direction_sb = gtk_spin_button_new ( (GtkAdjustment*)gtk_adjustment_new (0, 0.0, 359.9, 5.0, 1, 0 ), 1, 1 );
+
+    if ( !is_new && !isnan(wp->image_direction) ) {
+      GtkWidget *direction_ref = gtk_label_new ( NULL );
+      if ( wp->image_direction_ref == WP_IMAGE_DIRECTION_REF_MAGNETIC )
+        gtk_label_set_label ( GTK_LABEL(direction_ref), _("Magnetic") );
+      else
+        gtk_label_set_label ( GTK_LABEL(direction_ref), _("True") );
+
+      gtk_box_pack_start (GTK_BOX(direction_hb), direction_ref, TRUE, FALSE, 0);
+      gtk_spin_button_set_value ( GTK_SPIN_BUTTON(direction_sb), wp->image_direction );
+    }
+    else {
+      GtkWidget *direction_ref_button = gtk_button_new ();
+      gtk_button_set_relief ( GTK_BUTTON(direction_ref_button), GTK_RELIEF_NONE );
+      GtkWidget *img = gtk_image_new_from_stock ( GTK_STOCK_ADD, GTK_ICON_SIZE_MENU );
+      gtk_button_set_image ( GTK_BUTTON(direction_ref_button), img );
+      gtk_box_pack_start (GTK_BOX(direction_hb), direction_ref_button, TRUE, FALSE, 0);
+      gtk_widget_set_sensitive ( direction_sb, FALSE );
+      direction_signal_id = g_signal_connect ( G_OBJECT(direction_ref_button), "button-release-event", G_CALLBACK(direction_add_click), direction_sb );
+    }
+
 #endif
   }
 
@@ -258,21 +322,19 @@ gchar *a_dialog_waypoint ( GtkWindow *parent, gchar *default_name, VikTrwLayer *
   timevaluebutton = gtk_button_new();
   gtk_button_set_relief ( GTK_BUTTON(timevaluebutton), GTK_RELIEF_NONE );
 
-  if ( !edit_wp )
-    edit_wp = vik_waypoint_new ();
-  edit_wp = vik_waypoint_copy ( wp );
-
   // TODO: Consider if there should be a remove time button...
 
-  if ( !is_new && wp->has_timestamp ) {
+  if ( !is_new && !isnan(wp->timestamp) ) {
     update_time ( timevaluebutton, wp );
   }
   else {
     GtkWidget *img = gtk_image_new_from_stock ( GTK_STOCK_ADD, GTK_ICON_SIZE_MENU );
     gtk_button_set_image ( GTK_BUTTON(timevaluebutton), img );
     // Initially use current time or otherwise whatever the last value used was
-    if ( edit_wp->timestamp == 0 ) {
-      time ( &edit_wp->timestamp );
+    if ( isnan(edit_wp->timestamp) ) {
+      time_t tt;
+      time (&tt);
+      edit_wp->timestamp = (gdouble)tt;
     }
   }
   g_signal_connect ( G_OBJECT(timevaluebutton), "button-release-event", G_CALLBACK(time_edit_click), edit_wp );
@@ -289,6 +351,14 @@ gchar *a_dialog_waypoint ( GtkWindow *parent, gchar *default_name, VikTrwLayer *
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), commententry, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), descriptionlabel, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), descriptionentry, FALSE, FALSE, 0);
+  if ( wp->source ) {
+    gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), sourcelabel, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), sourceentry, FALSE, FALSE, 0);
+  }
+  if ( wp->type ) {
+    gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), typelabel, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), typeentry, FALSE, FALSE, 0);
+  }
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), imagelabel, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), imageentry, FALSE, FALSE, 0);
   if ( hasGeotagCB ) {
@@ -297,6 +367,10 @@ gchar *a_dialog_waypoint ( GtkWindow *parent, gchar *default_name, VikTrwLayer *
     gtk_box_pack_start (GTK_BOX(hbox), consistentGeotagCB, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), hbox, FALSE, FALSE, 0);
   }
+  if ( direction_hb )
+    gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), direction_hb, FALSE, FALSE, 0);
+  if ( direction_sb )
+    gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), direction_sb, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), symbollabel, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), GTK_WIDGET(symbolentry), FALSE, FALSE, 0);
 
@@ -321,17 +395,23 @@ gchar *a_dialog_waypoint ( GtkWindow *parent, gchar *default_name, VikTrwLayer *
       ll.lat = convert_dms_to_dec ( gtk_entry_get_text ( GTK_ENTRY(latentry) ) );
       ll.lon = convert_dms_to_dec ( gtk_entry_get_text ( GTK_ENTRY(lonentry) ) );
       vik_coord_load_from_latlon ( &(wp->coord), coord_mode, &ll );
-      // Always store in metres
-      switch (height_units) {
-      case VIK_UNITS_HEIGHT_METRES:
-        wp->altitude = atof ( gtk_entry_get_text ( GTK_ENTRY(altentry) ) );
-        break;
-      case VIK_UNITS_HEIGHT_FEET:
-        wp->altitude = VIK_FEET_TO_METERS(atof ( gtk_entry_get_text ( GTK_ENTRY(altentry) ) ));
-        break;
-      default:
-        wp->altitude = atof ( gtk_entry_get_text ( GTK_ENTRY(altentry) ) );
-        g_critical("Houston, we've had a problem. height=%d", height_units);
+      gchar const *alttext = gtk_entry_get_text ( GTK_ENTRY(altentry) );
+      if ( alttext && strlen(alttext) ) {
+        // Always store in metres
+        switch (height_units) {
+        case VIK_UNITS_HEIGHT_METRES:
+          wp->altitude = atof ( alttext );
+          break;
+        case VIK_UNITS_HEIGHT_FEET:
+          wp->altitude = VIK_FEET_TO_METERS(atof ( alttext ));
+          break;
+        default:
+          wp->altitude = atof ( alttext );
+          g_critical("Houston, we've had a problem. height=%d", height_units);
+        }
+      }
+      else {
+        wp->altitude = NAN;
       }
       if ( g_strcmp0 ( wp->comment, gtk_entry_get_text ( GTK_ENTRY(commententry) ) ) )
         vik_waypoint_set_comment ( wp, gtk_entry_get_text ( GTK_ENTRY(commententry) ) );
@@ -339,21 +419,35 @@ gchar *a_dialog_waypoint ( GtkWindow *parent, gchar *default_name, VikTrwLayer *
         vik_waypoint_set_description ( wp, gtk_entry_get_text ( GTK_ENTRY(descriptionentry) ) );
       if ( g_strcmp0 ( wp->image, vik_file_entry_get_filename ( VIK_FILE_ENTRY(imageentry) ) ) )
         vik_waypoint_set_image ( wp, vik_file_entry_get_filename ( VIK_FILE_ENTRY(imageentry) ) );
+      if ( sourceentry && g_strcmp0 ( wp->source, gtk_entry_get_text ( GTK_ENTRY(sourceentry) ) ) )
+        vik_waypoint_set_source ( wp, gtk_entry_get_text ( GTK_ENTRY(sourceentry) ) );
+      if ( typeentry && g_strcmp0 ( wp->type, gtk_entry_get_text ( GTK_ENTRY(typeentry) ) ) )
+        vik_waypoint_set_type ( wp, gtk_entry_get_text ( GTK_ENTRY(typeentry) ) );
       if ( wp->image && *(wp->image) && (!a_thumbnails_exists(wp->image)) )
         a_thumbnails_create ( wp->image );
-      if ( edit_wp->timestamp ) {
+      if ( !isnan(edit_wp->timestamp) ) {
         wp->timestamp = edit_wp->timestamp;
-        wp->has_timestamp = TRUE;
       }
 
+      if ( direction_sb ) {
+        if ( gtk_widget_get_sensitive (direction_sb) ) {
+          wp->image_direction = gtk_spin_button_get_value ( GTK_SPIN_BUTTON(direction_sb) );
+          if ( wp->image_direction != edit_wp->image_direction )
+            a_geotag_write_exif_gps ( wp->image, wp->coord, wp->altitude, wp->image_direction, wp->image_direction_ref, TRUE );
+        }
+      }
+
+      g_free ( last_sym );
       GtkTreeIter iter, first;
       gtk_tree_model_get_iter_first ( GTK_TREE_MODEL(store), &first );
       if ( !gtk_combo_box_get_active_iter ( GTK_COMBO_BOX(symbolentry), &iter ) || !memcmp(&iter, &first, sizeof(GtkTreeIter)) ) {
         vik_waypoint_set_symbol ( wp, NULL );
+        last_sym = NULL;
       } else {
         gchar *sym;
         gtk_tree_model_get ( GTK_TREE_MODEL(store), &iter, 0, (void *)&sym, -1 );
         vik_waypoint_set_symbol ( wp, sym );
+        last_sym = g_strdup ( sym );
         g_free(sym);
       }
 
