@@ -2,7 +2,7 @@
 /*
  * viking -- GPS Data and Topo Analyzer, Explorer, and Manager
  *
- * Copyright (C) 2013, Rob Norris <rw_norris@hotmail.com>
+ * Copyright (C) 2013-2015, Rob Norris <rw_norris@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 #include "acquire.h"
 #include "maputils.h"
 #include "dialog.h"
+#include "ui_util.h"
 
 static GObjectClass *parent_class;
 static GHashTable *last_user_strings = NULL;
@@ -98,7 +99,7 @@ static void webtool_datasource_set_property (GObject      *object,
 	case PROP_FILE_TYPE:
 		g_free ( priv->file_type );
 		priv->file_type = g_value_dup_string ( value );
-		g_debug ( "VikWebtoolDatasource.file_type: %s", priv->url_format_code );
+		g_debug ( "VikWebtoolDatasource.file_type: %s", priv->file_type );
 		break;
 
 	case PROP_BABEL_FILTER_ARGS:
@@ -194,7 +195,7 @@ static void datasource_add_setup_widgets ( GtkWidget *dialog, VikViewport *vvp,
 	GtkWidget *user_string_label;
     gchar *label = g_strdup_printf( "%s:", priv->input_label );
 	user_string_label = gtk_label_new ( label );
-	widgets->user_string = gtk_entry_new ( );
+	widgets->user_string = ui_entry_new ( NULL, GTK_ENTRY_ICON_SECONDARY );
 
     gchar *last_str = get_last_user_string ( widgets );
     if ( last_str )
@@ -217,7 +218,7 @@ static void datasource_add_setup_widgets ( GtkWidget *dialog, VikViewport *vvp,
 
 
 
-static void datasource_get_cmd_string ( gpointer user_data, gchar **cmd, gchar **extra, DownloadMapOptions *options )
+static void datasource_get_process_options ( gpointer user_data, ProcessOptions *po, DownloadFileOptions *options, const gchar *notused1, const gchar *notused2 )
 {
 	datasource_t *data = (datasource_t*) user_data;
 
@@ -235,7 +236,7 @@ static void datasource_get_cmd_string ( gpointer user_data, gchar **cmd, gchar *
 	gchar *url = vik_webtool_get_url ( vwd, data->vw );
 	g_debug ("%s: %s", __FUNCTION__, url );
 
-	*cmd = g_strdup ( url );
+	po->url = g_strdup ( url );
 
 	// Only use first section of the file_type string
 	// One can't use values like 'kml -x transform,rte=wpt' in order to do fancy things
@@ -245,22 +246,13 @@ static void datasource_get_cmd_string ( gpointer user_data, gchar **cmd, gchar *
 	if ( priv->file_type )
 		parts = g_strsplit ( priv->file_type, " ", 0);
 	if ( parts )
-		*extra = g_strdup ( parts[0] );
+		po->input_file_type = g_strdup ( parts[0] );
 	else
-		*extra = NULL;
+		po->input_file_type = NULL;
 	g_strfreev ( parts );
 
 	options = NULL;
-}
-
-static gboolean datasource_process ( VikTrwLayer *vtl, const gchar *cmd, const gchar *extra, BabelStatusFunc status_cb, acq_dialog_widgets_t *adw, DownloadMapOptions *options )
-{
-	datasource_t *data = (datasource_t *)adw->user_data;
-	VikWebtoolDatasourcePrivate *priv = WEBTOOL_DATASOURCE_GET_PRIVATE ( data->self );
-	// Dependent on the ExtTool / what extra has been set to...
-	// When extra is NULL - then it interprets results as a GPX
-	gboolean result = a_babel_convert_from_url_filter ( vtl, cmd, extra, priv->babel_filter_args, status_cb, adw, options);
-	return result;
+	po->babel_filters = priv->babel_filter_args;
 }
 
 static void cleanup ( gpointer data )
@@ -289,8 +281,8 @@ static void webtool_datasource_open ( VikExtTool *self, VikWindow *vw )
 		(VikDataSourceInitFunc)               datasource_init,
 		(VikDataSourceCheckExistenceFunc)     NULL,
 		(VikDataSourceAddSetupWidgetsFunc)    (search ? datasource_add_setup_widgets : NULL),
-		(VikDataSourceGetCmdStringFunc)       datasource_get_cmd_string,
-		(VikDataSourceProcessFunc)            datasource_process,
+		(VikDataSourceGetProcessOptionsFunc)  datasource_get_process_options,
+		(VikDataSourceProcessFunc)            a_babel_convert_from,
 		(VikDataSourceProgressFunc)           NULL,
 		(VikDataSourceAddProgressWidgetsFunc) NULL,
 		(VikDataSourceCleanupFunc)            cleanup,
@@ -495,8 +487,7 @@ static gchar *webtool_datasource_get_url ( VikWebtool *self, VikWindow *vw )
 	gchar *url = g_strdup_printf ( priv->url, values[0], values[1], values[2], values[3], values[4], values[5], values[6] );
 
 	for ( i = 0; i < MAX_NUMBER_CODES; i++ ) {
-		if ( values[i] != '\0' )
-			g_free ( values[i] );
+		g_free ( values[i] );
 	}
 	
 	return url;
